@@ -1,15 +1,15 @@
 /*
  * IceWM
  *
- * Copyright (C) 1998 Marko Macek
+ * Copyright (C) 1998-2001 Marko Macek
  *
  * CPU Status
  */
 #include "config.h"
 
-#ifdef CONFIG_CPUSTATUS
+#ifdef CONFIG_APPLET_CPU_STATUS
 #include "ylib.h"
-#include "yapp.h"
+#include "wmapp.h"
 
 #include "acpustatus.h"
 #include "sysdep.h"
@@ -20,26 +20,37 @@
 #include <sys/sysinfo.h>
 #endif
 
+#ifdef HAVE_KSTAT_H
+#include <kstat.h>
+#endif
+
+#include "intl.h"
+
 #if (defined(linux) || defined(HAVE_KSTAT_H))
 
 #define UPDATE_INTERVAL 500
 
-CPUStatus::CPUStatus(const char *CpuCommand, YWindow *aParent): YWindow(aParent) {
+extern YPixmap *taskbackPixmap;
+
+CPUStatus::CPUStatus(YWindow *aParent): YWindow(aParent) {
     cpu = new int *[taskBarCPUSamples];
-    for (unsigned int a = 0; a < taskBarCPUSamples; a++) {
+
+    for (int a(0); a < taskBarCPUSamples; a++)
         cpu[a] = new int[IWM_STATES];
-    }
+
     fUpdateTimer = new YTimer(UPDATE_INTERVAL);
     if (fUpdateTimer) {
         fUpdateTimer->setTimerListener(this);
         fUpdateTimer->startTimer();
     }
-    fCPUCommand = CpuCommand;
+
     color[IWM_USER] = new YColor(clrCpuUser);
     color[IWM_NICE] = new YColor(clrCpuNice);
     color[IWM_SYS]  = new YColor(clrCpuSys);
-    color[IWM_IDLE] = new YColor(clrCpuIdle);
-    for (unsigned int i = 0; i < taskBarCPUSamples; i++) {
+    color[IWM_IDLE] = *clrCpuIdle
+    		    ? new YColor(clrCpuIdle) : NULL;
+
+    for (int i(0); i < taskBarCPUSamples; i++) {
         cpu[i][IWM_USER] = cpu[i][IWM_NICE] = cpu[i][IWM_SYS] = 0;
         cpu[i][IWM_IDLE] = 1;
     }
@@ -51,7 +62,7 @@ CPUStatus::CPUStatus(const char *CpuCommand, YWindow *aParent): YWindow(aParent)
 }
 
 CPUStatus::~CPUStatus() {
-    for (unsigned int a = 0; a < taskBarCPUSamples; a++) {
+    for (int a(0); a < taskBarCPUSamples; a++) {
         delete cpu[a]; cpu[a] = 0;
     }
     delete cpu; cpu = 0;
@@ -64,7 +75,7 @@ CPUStatus::~CPUStatus() {
 void CPUStatus::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
     int n, h = height();
 
-    for (unsigned int i = 0; i < taskBarCPUSamples; i++) {
+    for (int i(0); i < taskBarCPUSamples; i++) {
         int user = cpu[i][IWM_USER];
         int nice = cpu[i][IWM_NICE];
         int sys = cpu[i][IWM_SYS];
@@ -99,9 +110,23 @@ void CPUStatus::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/,
             }
         }
         if (idle) {
-            g.setColor(color[IWM_IDLE]);
-            g.drawLine(i, 0, i, y);
-        }
+	    if (color[IWM_IDLE]) {
+		g.setColor(color[IWM_IDLE]);
+		g.drawLine(i, 0, i, y);
+            } else {
+#ifdef CONFIG_GRADIENTS
+		class YPixbuf * gradient(parent()->getGradient());
+
+		if (gradient)
+		    g.copyPixbuf(*gradient,
+		    		 this->x() + i, this->y(), width(), y + 1, i, 0);
+		else 
+#endif		
+		if (taskbackPixmap)
+		    g.fillPixmap(taskbackPixmap,
+		    		 i, 0, width(), y + 1, this->x() + i, this->y());
+	    }
+	}
     }
 }
 
@@ -124,7 +149,7 @@ void CPUStatus::updateToolTip() {
     l1 = (float)sys.loads[0] / 65536.0;
     l5 = (float)sys.loads[1] / 65536.0;
     l15 = (float)sys.loads[2] / 65536.0;
-    sprintf(load, "CPU Load: %3.2f %3.2f %3.2f, %d processes.",
+    sprintf(load, _("CPU Load: %3.2f %3.2f %3.2f, %d processes."),
             l1, l5, l15, sys.procs);
     setToolTip(load);
 #endif
@@ -132,15 +157,14 @@ void CPUStatus::updateToolTip() {
 
 void CPUStatus::handleClick(const XButtonEvent &up, int count) {
     if (up.button == 1) {
-	 if ((count % 2) == 0) {
-            if (fCPUCommand && fCPUCommand[0])
-                app->runCommand(fCPUCommand);
-        }
+        if (cpuCommand && cpuCommand[0] &&
+	   (taskBarLaunchOnSingleClick ? count == 1 : !(count % 2)))
+	    wmapp->runCommandOnce(cpuClassHint, cpuCommand);
     }
 }
 
 void CPUStatus::updateStatus() {
-    for (unsigned int i = 1; i < taskBarCPUSamples; i++) {
+    for (int i(1); i < taskBarCPUSamples; i++) {
         cpu[i - 1][IWM_USER] = cpu[i][IWM_USER];
         cpu[i - 1][IWM_NICE] = cpu[i][IWM_NICE];
         cpu[i - 1][IWM_SYS]  = cpu[i][IWM_SYS];
@@ -181,7 +205,7 @@ void CPUStatus::getStatus() {
     }
     close(fd);
 #if 0
-    fprintf(stderr, "cpu: %d %d %d %d\n",
+    msg(_("cpu: %d %d %d %d"),
             cpu[taskBarCPUSamples-1][IWM_USER], cpu[taskBarCPUSamples-1][IWM_NICE],
             cpu[taskBarCPUSamples-1][IWM_SYS],  cpu[taskBarCPUSamples-1][IDLE]);
 #endif
@@ -268,7 +292,8 @@ void CPUStatus::getStatus() {
                     cpu_ks[thiscpu] = ks;
                     thiscpu++;
                     if (thiscpu > ncpus) {
-                        fprintf(stderr, "kstat finds too many cpus: should be %d\n",ncpus);
+                        warn(_("kstat finds too many cpus: should be %d"),
+                             ncpus);
                         return;/* FIXME : need err handler? */
                     }
                 }
