@@ -14,14 +14,15 @@
 #include "ylib.h"
 #include "amailbox.h"
 
-#include "yapp.h"
 #include "sysdep.h"
 #include "base.h"
 #include "prefs.h"
+#include "wmapp.h"
 #include <sys/socket.h>
 #include <netdb.h>
 
 extern YColor *taskBarBg;
+extern YPixmap *taskbackPixmap;
 
 YPixmap *mailPixmap = 0;
 YPixmap *noMailPixmap = 0;
@@ -283,24 +284,13 @@ void MailCheck::socketDataRead(char *buf, int len) {
     sk.read(bf, sizeof(bf));
 }
 
-MailBoxStatus::MailBoxStatus(const char *mailBox, const char *mailCommand, YWindow *aParent): YWindow(aParent), check(this) {
-    char *mail = getenv("MAIL");
-
-    fMailBox = 0;
-
-    if (mailBox && mailBox[0])
-        fMailBox = newstr(mailBox);
-    else if (mail)
-        fMailBox = newstr(mail);
-    else
-        fMailBox = newstr("/dev/null");
-
-    fMailCommand = mailCommand;
+MailBoxStatus::MailBoxStatus(const char *mailbox, YWindow *aParent): 
+    YWindow(aParent), fMailBox(newstr(mailbox)), check(this) {
     setSize(16, 16);
     fMailboxCheckTimer = 0;
     fState = mbxNoMail;
     if (fMailBox) {
-        MSG((_("Using MailBox: '%s'\n"), fMailBox));
+        MSG((_("Using MailBox \"%s\"\n"), fMailBox));
         check.setURL(fMailBox);
 
         fMailboxCheckTimer = new YTimer(mailCheckDelay * 1000);
@@ -340,24 +330,34 @@ void MailBoxStatus::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*widt
         pixmap = errMailPixmap;
         break;
     }
-    if (!pixmap || pixmap->mask()) {
-        g.setColor(taskBarBg);
-        // !!! fix this to draw taskbar background pixmap too
-        g.fillRect(0, 0, width(), height());
+    
+    if (pixmap == NULL || pixmap->mask()) {
+#ifdef CONFIG_GRADIENTS
+	class YPixbuf * gradient(parent()->getGradient());
+
+	if (gradient)
+	    g.copyPixbuf(*gradient, x(), y(), width(), height(), 0, 0);
+	else 
+#endif	
+	if (taskbackPixmap)
+	    g.fillPixmap(taskbackPixmap, 0, 0,
+			 width(), height(), this->x(), this->y());
+        else {
+	    g.setColor(taskBarBg);
+	    g.fillRect(0, 0, width(), height());
+	}
     }
     if (pixmap)
         g.drawPixmap(pixmap, 0, 0);
 }
 
 void MailBoxStatus::handleClick(const XButtonEvent &up, int count) {
-    if (up.button == 1) {
-        if (count == 1) {
-            checkMail();
-        } else if ((count % 2) == 0) {
-            if (fMailCommand && fMailCommand[0])
-                app->runCommand(fMailCommand);
-        }
-    }
+    if ((taskBarLaunchOnSingleClick ? up.button == 2
+				    : up.button == 1) && count == 1)
+	checkMail();
+    else if (mailCommand && mailCommand[0] && up.button == 1 &&
+	(taskBarLaunchOnSingleClick ? count == 1 : !(count % 2)))
+	wmapp->runCommandOnce(mailClassHint, mailCommand);
 }
 
 void MailBoxStatus::handleCrossing(const XCrossingEvent &crossing) {
@@ -417,7 +417,7 @@ void MailBoxStatus::mailChecked(MailBoxState mst, long count) {
 
 void MailBoxStatus::newMailArrived() {
     if (beepOnNewMail)
-        XBell(app->display(), 100);
+	app->alert();
     if (newMailCommand && newMailCommand[0])
         app->runCommand(newMailCommand);
 }
