@@ -8,16 +8,18 @@
 //
 // !!! share code with cpu status
 // //////////////////////////////////////////////////////////////////////////
-#include "config.h"
 
+#define NEED_TIME_H
+
+#include "config.h"
 #include "ylib.h"
-#include "yapp.h"
+#include "sysdep.h"
 
 #include "apppstatus.h"
 
-#ifdef HAVE_NET_STATUS
+#include "wmapp.h"
 
-#include "sysdep.h"
+#ifdef HAVE_NET_STATUS
 #include "prefs.h"
 #include "intl.h"
 #include <net/if.h>
@@ -30,7 +32,10 @@
 #include <net/if_mib.h>
 #endif
 
-NetStatus::NetStatus(const char * netCommand, YWindow *aParent): YWindow(aParent) {
+extern YPixmap *taskbackPixmap;
+
+NetStatus::NetStatus(char const * netdev, YWindow *aParent): 
+    YWindow(aParent), fNetDev(newstr(netdev)) {
     // clear out the data
     for (int i = 0; i < NET_SAMPLES + 1; i++) {
         ppp_in[i] = ppp_out[i] = ppp_tot[i] = 0;
@@ -38,13 +43,11 @@ NetStatus::NetStatus(const char * netCommand, YWindow *aParent): YWindow(aParent
 
     color[0] = new YColor(clrNetReceive);
     color[1] = new YColor(clrNetSend);
-    color[2] = new YColor(clrNetIdle);
+    color[2] = *clrNetIdle
+	     ? new YColor(clrNetIdle) : NULL;
 
     setSize(NET_SAMPLES, 20);
 
-    fNetCommand = netCommand;
-
-
     fUpdateTimer = new YTimer();
     if (fUpdateTimer) {
         fUpdateTimer->setInterval(NET_UPDATE_INTERVAL);
@@ -58,9 +61,7 @@ NetStatus::NetStatus(const char * netCommand, YWindow *aParent): YWindow(aParent
     wasUp = false;
 
     // test for isdn-device
-    useIsdn = false;
-    if (strncmp(netDevice,"ippp",4)==0)
-        useIsdn = true;
+    useIsdn = !strncmp(fNetDev,"ippp", 4);
     // unset phoneNumber
     strcpy(phoneNumber,"");
 
@@ -73,7 +74,8 @@ NetStatus::NetStatus(const char * netCommand, YWindow *aParent): YWindow(aParent
 }
 
 NetStatus::~NetStatus() {
-    delete [] color;
+    delete[] fNetDev;
+    delete[] color;
     delete fUpdateTimer;
 }
 
@@ -86,56 +88,86 @@ bool NetStatus::handleTimer(YTimer *t) {
     if (up) {
         if (!wasUp) {
             // clear out the data
-            for (int i = 0; i < NET_SAMPLES + 1; i++) {
-                ppp_in[i] = ppp_out[i] = ppp_tot[i] = 0;
-            }
+	    memset(ppp_in, 0, sizeof(ppp_in));
+	    memset(ppp_out, 0, sizeof(ppp_out));
+	    memset(ppp_tot, 0, sizeof(ppp_tot));
+
             start_time = time(NULL);
             start_ibytes = cur_ibytes;
             start_obytes = cur_obytes;
+            maxBytes = 0;
 
             updateStatus();
             prev_ibytes = cur_ibytes;
             prev_obytes = cur_obytes;
-            this->show();
+            show();
         }
         updateStatus();
+
         if (toolTipVisible())
             updateToolTip();
     }
     else // link is down
-        if (wasUp) this->hide();
+        if (wasUp) hide();
 
     wasUp = up;
     return true;
 }
 
 void NetStatus::updateToolTip() {
-    char status[96];
-    int t = time(NULL) - start_time;
-    int o = cur_obytes - start_obytes;
-    int i = cur_ibytes - start_ibytes;
+    char status[400];
 
-    if (t <= 0)
-        sprintf(status, "%s:",
-                netDevice);
-    else
-        sprintf(status, _("%s@%s: Sent: %db Rcvd: %db in %ds"),
-                phoneNumber, netDevice,
-                o, i, t);
+    if (isUp()) {
+	char const * const sizeUnits[] = { "b", "kb", "Mb", "Gb", "Tb", NULL };
+	char const * const rateUnits[] = { "bps", "kps", "Mps", NULL };
+
+	int const t(time(NULL) - start_time);
+
+	int vi(cur_ibytes - start_ibytes);
+	int vo(cur_obytes - start_obytes);
+	
+	int ci(ppp_in[NET_SAMPLES - 1]);
+	int co(ppp_out[NET_SAMPLES - 1]);
+
+	int ai(t ? vi / t : 0);
+	int ao(t ? vo / t : 0);
+    
+	const char * const viUnit(niceUnit(vi, sizeUnits));
+	const char * const voUnit(niceUnit(vo, sizeUnits));
+	const char * const ciUnit(niceUnit(ci, rateUnits));
+	const char * const coUnit(niceUnit(co, rateUnits));
+	const char * const aiUnit(niceUnit(ai, rateUnits));
+	const char * const aoUnit(niceUnit(ao, rateUnits));
+
+        snprintf(status, sizeof(status),
+		_("Interface %s:\n"
+		  "  Current rate (in/out):\t%d %s/%d %s\n"
+		  "  Average rate (in/out):\t%d %s/%d %s\n"
+		  "  Transferred (in/out):\t%d %s/%d %s\n"
+		  "  Online time:\t%d:%02d:%02d"
+		  "%s%s"),
+		fNetDev,
+		ci, ciUnit, co, coUnit, ai, aiUnit, ao, aoUnit,
+		vi, viUnit, vo, voUnit,
+                t / 3600, t / 60 % 60, t % 60,
+		*phoneNumber ? _("\n  Caller id:\t") : "", phoneNumber);
+    } else
+        snprintf(status, sizeof(status), "%s:", fNetDev);
 
     setToolTip(status);
 }
 
 void NetStatus::handleClick(const XButtonEvent &up, int count) {
     if (up.button == 1) {
-        if ((count % 2) == 0) {
+        if (taskBarLaunchOnSingleClick ? count == 1 : !(count % 2)) {
             if (up.state & ControlMask) {
                 start_time = time(NULL);
                 start_ibytes = cur_ibytes;
                 start_obytes = cur_obytes;
+                maxBytes = 0;
             } else {
-                if (fNetCommand && fNetCommand[0])
-                    app->runCommand(fNetCommand);
+                if (netCommand && netCommand[0])
+		    wmapp->runCommandOnce(netClassHint, netCommand);
             }
         }
     }
@@ -170,15 +202,47 @@ void NetStatus::paint(Graphics &g, int /*x*/, int /*y*/,
             }
 
             if (l < t) {
+/*	    
                 g.setColor(color[2]);
                 //g.drawLine(i, 0, i, h - tot - 2);
                 g.drawLine(i, l, i, t - l);
+*/		
+		if (color[2]) {
+		    g.setColor(color[2]);
+		    g.drawLine(i, l, i, t - 1);
+		} else {
+#ifdef CONFIG_GRADIENTS
+		    class YPixbuf * gradient(parent()->getGradient());
+
+		    if (gradient)
+			g.copyPixbuf(*gradient,
+				     x() + i, y() + l, width(), t - l, i, l);
+		    else 
+#endif		    
+		    if (taskbackPixmap)
+			g.fillPixmap(taskbackPixmap,
+				     i, l, width(), t - l, x() + i, y() + l);
+		}
             }
         }
         else {
-            g.setColor(color[2]);
-            g.drawLine(i, 0, i, h - 1);
-        }
+	    if (color[2]) {
+		g.setColor(color[2]);
+		g.drawLine(i, 0, i, h - 1);
+            } else {
+#ifdef CONFIG_GRADIENTS
+		class YPixbuf * gradient(parent()->getGradient());
+
+		if (gradient)
+		    g.copyPixbuf(*gradient,
+		    		 x() + i, y(), width(), h, i, 0);
+		else 
+#endif
+		if (taskbackPixmap)
+		    g.fillPixmap(taskbackPixmap,
+		    		 i, 0, width(), h, x() + i, y());
+	    }
+	}
     }
 }
 
@@ -275,7 +339,7 @@ bool NetStatus::isUp() {
     struct ifreq *ifr;
     int len;
 
-    if (netDevice == 0)
+    if (fNetDev == 0)
         return false;
 
     int s = socket(PF_INET, SOCK_STREAM, 0);
@@ -292,7 +356,7 @@ bool NetStatus::isUp() {
     len = ifc.ifc_len;
     ifr = ifc.ifc_req;
     while (len > 0) {
-        if (strcmp(netDevice, ifr->ifr_name) == 0) {
+        if (strcmp(fNetDev, ifr->ifr_name) == 0) {
             close(s);
             return true;
         }
@@ -361,8 +425,8 @@ void NetStatus::getCurrent(int *in, int *out, int *tot) {
         char *p = buf;
         while (*p == ' ')
             p++;
-        if (strncmp(p, netDevice, strlen(netDevice)) == 0 &&
-            p[strlen(netDevice)] == ':')
+        if (strncmp(p, fNetDev, strlen(fNetDev)) == 0 &&
+            p[strlen(fNetDev)] == ':')
         {
             int ipackets, opackets;
             int ierrs, oerrs;
@@ -420,7 +484,7 @@ void NetStatus::getCurrent(int *in, int *out, int *tot) {
                                printf(_("%s@%d: %s\n"),__FILE__,__LINE__,strerror(errno));
                                continue;
                        }
-               if (strncmp(ifmd.ifmd_name, netDevice, strlen(netDevice)) == 0) {
+               if (strncmp(ifmd.ifmd_name, fNetDev, strlen(fNetDev)) == 0) {
                                cur_ibytes = ifmd.ifmd_data.ifi_ibytes;
                                cur_obytes = ifmd.ifmd_data.ifi_obytes;
                                break;
