@@ -9,9 +9,9 @@
 #define NETSTATUS_H
 
 #if defined(linux) || defined(__FreeBSD__)
-#ifdef CONFIG_NET_STATUS
+#ifdef CONFIG_APPLET_NET_STATUS
 
-#define HAVE_NET_STATUS
+#define HAVE_NET_STATUS 1
 
 #include "ywindow.h"
 #include "ytimer.h"
@@ -23,7 +23,7 @@
 
 class NetStatus: public YWindow, public YTimerListener {
 public:
-    NetStatus(const char *netCommand, YWindow *aParent = 0);
+    NetStatus(char const * netdev, YWindow *aParent = 0);
     ~NetStatus();
 private:
     YColor *color[3];
@@ -42,11 +42,14 @@ private:
     struct timeval prev_time;
 
     bool wasUp;               // previous link status
+    bool useIsdn;             // netdevice is an IsdnDevice
+    char *fNetDev;		// name of the device
     
-    const char *fNetCommand;
+    char phoneNumber[32];
 
     // methods local to this class
     bool isUp();
+    bool isUpIsdn();
     void getCurrent(int *in, int *out, int *tot);
     void updateStatus();
     void updateToolTip();
@@ -57,8 +60,8 @@ private:
     virtual void paint(Graphics & g, int x, int y, unsigned int width, unsigned int height);
 };
 
-#endif
+#endif // CONFIG_APPLET_NET_STATUS
 
-#endif
+#endif // linux || __FreeBSD__
 
-#endif
+#endif // NETSTATUS_H
