@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1998 Marko Macek
+ * Copyright (C) 1998-2001 Marko Macek
  */
 
 #include "config.h"
@@ -89,10 +89,10 @@ void BrowseMenu::updatePopup() {
 #ifndef LITE
                             if (sub) {
                                 if (folder)
-                                    item->setPixmap(folder->small());
+                                    item->setIcon(folder->small());
                             } else {
                                 if (file)
-                                    item->setPixmap(file->small());
+                                    item->setIcon(file->small());
                             }
 #endif
                         }
