@@ -25,6 +25,13 @@ YPixmap *frameB[2][2] = {{ 0, 0 }, { 0, 0 }};
 YPixmap *frameBR[2][2] = {{ 0, 0 }, { 0, 0 }};
 #endif
 
+#ifdef CONFIG_GRADIENTS
+YPixbuf *rgbFrameT[2][2] = {{ 0, 0 }, { 0, 0 }};
+YPixbuf *rgbFrameL[2][2] = {{ 0, 0 }, { 0, 0 }};
+YPixbuf *rgbFrameR[2][2] = {{ 0, 0 }, { 0, 0 }};
+YPixbuf *rgbFrameB[2][2] = {{ 0, 0 }, { 0, 0 }};
+#endif
+
 void YFrameWindow::updateMenu() {
     YMenu *windowMenu = this->windowMenu();
     // enable all commands
@@ -65,37 +72,48 @@ void YFrameWindow::updateMenu() {
     if (!(func & ffClose))
         windowMenu->disableCommand(actionClose);
 
-    YMenuItem *item = windowMenu->findAction(actionRollup);
-    if (item)
-        item->setChecked(isRollup() ? true : false);
-    item = windowMenu->findAction(actionOccupyAllOrCurrent);
-    if (item)
-        item->setChecked(isSticky() ? true : false);
-    item = windowMenu->findSubmenu(moveMenu);
-    if (item != 0)
-        item->setEnabled(isSticky() ? false : true);
-    for (int i = 0; i < moveMenu->itemCount(); i++) {
+    YMenuItem *item;
+
+    if ((item = windowMenu->findAction(actionRollup)))
+        item->setChecked(isRollup());
+    if ((item = windowMenu->findAction(actionOccupyAllOrCurrent)))
+        item->setChecked(isSticky());
+    if ((item = windowMenu->findAction(actionDoNotCover)))
+        item->setChecked(doNotCover());
+    if ((item = windowMenu->findSubmenu(moveMenu)))
+        item->setEnabled(!isSticky());
+
+    for (int i(0); i < moveMenu->itemCount(); i++) {
         item = moveMenu->item(i);
-        for (int w = 0; w < workspaceCount; w++) {
-            if (item && item->action() == workspaceActionMoveTo[w]) {
-                bool t = (w == getWorkspace()) ? false : true;
-                item->setEnabled(t);
-            }
-        }
+        for (int w(0); w < workspaceCount; w++)
+            if (item && item->action() == workspaceActionMoveTo[w])
+                item->setEnabled(w != getWorkspace());
     }
-    for (int j = 0; j < layerMenu->itemCount(); j++) {
+
+    for (int j(0); j < layerMenu->itemCount(); j++) {
         item = layerMenu->item(j);
-        for (int layer = 0; layer < WinLayerCount; layer++) {
+        for (int layer(0); layer < WinLayerCount; layer++)
             if (item && item->action() == layerActionSet[layer]) {
-                bool e = (layer == getLayer()) ? true : false;
+                bool const e(layer == getLayer());
                 item->setEnabled(!e);
                 item->setChecked(e);
             }
-        }
     }
+
+#ifdef CONFIG_TRAY
+    if (trayMenu) for (int k(0); k < trayMenu->itemCount(); k++) {
+        item = trayMenu->item(k);
+        for (int opt(0); opt < WinTrayOptionCount; opt++)
+            if (item && item->action() == trayOptionActionSet[opt]) {
+                bool const e(opt == getTrayOption());
+                item->setEnabled(!e);
+                item->setChecked(e);
+            }
+    }
+#endif    
 }
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
 void YFrameWindow::setShape() {
     if (!shapesSupported)
         return ;
@@ -147,6 +165,7 @@ void YFrameWindow::setShape() {
                 rect[nrect].height = titleY();
                 nrect++;
             }
+
             if (nrect !=  0)
                 XShapeCombineRectangles(app->display(), handle(),
                                         ShapeBounding,
@@ -167,37 +186,130 @@ void YFrameWindow::setShape() {
 }
 #endif
 
-void YFrameWindow::configure(int x, int y, unsigned int width, unsigned int height) {
+void YFrameWindow::layoutShape() {
+#ifdef CONFIG_SHAPED_DECORATION
+    if (shapesSupported && (frameDecors() & fdBorder))
+	if(isIconic())
+	    XShapeCombineMask(app->display(), handle(),
+		              ShapeBounding, 0, 0, None, ShapeSet);
+	else {
+	    int const a(focused() ? 1 : 0);
+	    int const t((frameDecors() & fdResize) ? 0 : 1);
+
+	    Pixmap shape(YPixmap::createMask(width(), height()));
+	    Graphics g(shape);
+
+	    g.setColor(YColor::white);
+	    g.fillRect(0, 0, width(), height());
+
+	    const int xTL(frameTL[t][a] ? frameTL[t][a]->width() : 0),
+    		      xTR(width() - 
+		         (frameTR[t][a] ? frameTR[t][a]->width() : 0)),
+    		      xBL(frameBL[t][a] ? frameBL[t][a]->width() : 0),
+		      xBR(width() - 
+		         (frameBR[t][a] ? frameBR[t][a]->width() : 0));
+	    const int yTL(frameTL[t][a] ? frameTL[t][a]->height() : 0),
+    		      yBL(height() -
+		         (frameTR[t][a] ? frameTR[t][a]->height() : 0)),
+    		      yTR(frameBL[t][a] ? frameBL[t][a]->height() : 0),
+    		      yBR(height() -
+		         (frameBR[t][a] ? frameBR[t][a]->height() : 0));
+
+	    if (frameTL[t][a]) {
+		g.copyDrawable(frameTL[t][a]->mask(), 0, 0,
+			       frameTL[t][a]->width(), frameTL[t][a]->height(),
+			       0, 0);
+		if (protectClientWindow)
+		    g.fillRect(borderX(), borderY(),
+			       frameTL[t][a]->width() - borderX(),
+			       frameTL[t][a]->height() - borderY());
+	    }
+	    if (frameTR[t][a]) {
+		g.copyDrawable(frameTR[t][a]->mask(), 0, 0,
+			       frameTR[t][a]->width(), frameTR[t][a]->height(),
+			       xTR, 0);
+		if (protectClientWindow)
+		    g.fillRect(xTR, borderY(),
+			       frameTR[t][a]->width() - borderX(),
+			       frameTR[t][a]->height() - borderY());
+	    }
+	    if (frameBL[t][a]) {
+		g.copyDrawable(frameBL[t][a]->mask(), 0, 0,
+			       frameBL[t][a]->width(), frameBL[t][a]->height(),
+			       0, yBL);
+		if (protectClientWindow)
+		    g.fillRect(borderX(), yBL,
+			       frameBL[t][a]->width() - borderX(),
+			       frameBL[t][a]->height() - borderY());
+	    }
+	    if (frameBR[t][a]) {
+		g.copyDrawable(frameBR[t][a]->mask(), 0, 0,
+			       frameBR[t][a]->width(), frameBL[t][a]->height(),
+			       xBR, yBR);
+		if (protectClientWindow)
+		    g.fillRect(xBR, yBR,
+			       frameBR[t][a]->width() - borderX(),
+			       frameBR[t][a]->width() - borderY());
+	    }
+	    
+	    if (frameT[t][a])
+		g.repHorz(frameT[t][a]->mask(), 
+			  frameT[t][a]->width(), frameT[t][a]->height(),
+			  xTL, 0, xTR - xTL);
+	    if (frameB[t][a])
+		g.repHorz(frameB[t][a]->mask(), 
+			  frameB[t][a]->width(), frameB[t][a]->height(),
+			  xBL, height() - frameB[t][a]->height(), xBR - xBL);
+	    if (frameL[t][a])
+		g.repVert(frameL[t][a]->mask(), 
+			  frameL[t][a]->width(), frameL[t][a]->height(),
+			  0, yTL, yBL - yTL);
+	    if (frameR[t][a])
+		g.repVert(frameR[t][a]->mask(), 
+			  frameR[t][a]->width(), frameR[t][a]->height(),
+			  width() - frameR[t][a]->width(), yTR, yBR - yTR);
+
+	    if (titlebar() && titleY())
+		titlebar()->renderShape(shape);
+	    XShapeCombineMask(app->display(), handle(),
+			      ShapeBounding, 0, 0, shape, ShapeSet);
+	    XFreePixmap(app->display(), shape);
+	}
+#endif
+}
+
+void YFrameWindow::configure(const int x, const int y, 
+			     const unsigned width, const unsigned height, 
+			     const bool resized) {
     //int oldX = this->x();
     //int oldY= this->y();
 
     MSG(("configure %d %d %d %d", x, y, width, height));
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     unsigned int oldWidth = container()->width();
     unsigned int oldHeight = container()->height();
     int oldcx = container()->x();
     int oldcy = container()->y();
 #endif
 
-    YWindow::configure(x, y, width, height);
+    YWindow::configure(x, y, width, height, resized);
 
     layoutTitleBar();
-
     layoutButtons();
-
     layoutResizeIndicators();
-
+    if (resized) layoutShape();
     layoutClient();
 
     // ??? !!!
     //if (x != oldX || y != oldY)
     sendConfigure();
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     int cx = container()->x();
     int cy = container()->y();
-    if (oldWidth != container()->width() || oldHeight != container()->height() ||
+    if (oldWidth != container()->width() ||
+    	oldHeight != container()->height() ||
         cx != oldcx || cy != oldcy)
         setShape();
 #endif
@@ -243,11 +355,16 @@ YFrameButton *YFrameWindow::getButton(char c) {
 void YFrameWindow::positionButton(YFrameButton *b, int &xPos, bool onRight) {
     /// !!! clean this up
     if (b == fMenuButton) {
-        if (onRight) xPos -= titleY();
-        b->setGeometry(xPos, 0, titleY(), titleY());
-        if (!onRight) xPos += titleY();
+	const unsigned bw((wmLook == lookPixmap || wmLook == lookMetal || 
+			   wmLook == lookGtk) && 
+			   showFrameIcon || !b->getImage(0) ?
+			   titleY() : b->getImage(0)->width());
+
+        if (onRight) xPos -= bw;
+        b->setGeometry(xPos, 0, bw, titleY());
+        if (!onRight) xPos += bw;
     } else if (wmLook == lookPixmap || wmLook == lookMetal || wmLook == lookGtk) {
-        int bw = b->getImage(0) ? b->getImage(0)->width() : titleY();
+	const unsigned bw(b->getImage(0) ? b->getImage(0)->width() : titleY());
 
         if (onRight) xPos -= bw;
         b->setGeometry(xPos, 0, bw, titleY());
@@ -368,7 +485,7 @@ void YFrameWindow::layoutButtons() {
 }
 
 void YFrameWindow::layoutResizeIndicators() {
-    if (((frameDecors() & (fdResize | fdBorder)) == fdResize + fdBorder) &&
+    if (((frameDecors() & (fdResize | fdBorder)) == (fdResize | fdBorder)) &&
         !isRollup() && !isMinimized()) {
         if (!indicatorsVisible) {
             indicatorsVisible = 1;
