@@ -48,12 +48,12 @@
 #error Configure with "--enable-guievents"
 #else /* CONFIG_GUIEVENTS */
 
-#include <cstdio>
-#include <cassert>
-#include <cstring>
-#include <cstdlib>
-#include <cstdarg>
-#include <cerrno>
+#include <stdio.h>
+#include <assert.h>
+#include <stdlib.h>
+#include <stdarg.h>
+#include <errno.h>
+#include <string.h>
 
 #include <ctype.h>
 #include <unistd.h>
@@ -83,10 +83,6 @@ char const * YApplication::Name = "icesound";
 #define OSS_DEFAULT_DEVICE "/dev/dsp"
 #define YIFF_DEFAULT_SERVER "127.0.0.1:9433"
 
-#define THROW(Result) { rc = (Result); goto exceptionHandler; }
-#define TRY(Command) { if ((rc = (Command))) THROW(rc); }
-#define CATCH(Handler) { exceptionHandler: { Handler } return rc; }
-
 /******************************************************************************
  * IceSound application
  ******************************************************************************/
@@ -147,33 +143,35 @@ public:
 	return findSample(basename);
     }
 
+    char * findSample(char const * basename);
+};
+
 /**
- * Finds a the sample specified.
+ * Finds the sample specified.
  * Returns NULL on error or the the full path to the sample file.
  * The string returned has to be freed by the caller
  */
-    char * findSample(char const * basename) {
-	static char const * paths[] = {
-	    IceSound::samples,
-	    strJoin(getenv("HOME"), "/.icewm/sounds/", NULL),
-	    strJoin(CFGDIR, "/sounds/", NULL),
-	    strJoin(LIBDIR, "/sounds/", NULL)
-	};
+char * YAudioInterface::findSample(char const * basename) {
+    static char const * paths[] = {
+	IceSound::samples,
+	strJoin(getenv("HOME"), "/.icewm/sounds/", NULL),
+	strJoin(CFGDIR, "/sounds/", NULL),
+	strJoin(LIBDIR, "/sounds/", NULL)
+    };
 
-	for(unsigned i(0); i < ACOUNT(gui_events); i++)
-	    for (unsigned n(0); n < ACOUNT(paths); ++n)
-		if(paths[n] != NULL) {
-		    char * filename(strJoin(paths[n], basename, NULL));
+    for(unsigned i(0); i < ACOUNT(gui_events); i++)
+	for (unsigned n(0); n < ACOUNT(paths); ++n)
+	    if(paths[n] != NULL) {
+		char * filename(strJoin(paths[n], basename, NULL));
 
-		    if (access(filename, R_OK) == 0)
-			return filename;
+		if (access(filename, R_OK) == 0)
+		    return filename;
 
-		    delete[] filename;
-		}
+		delete[] filename;
+	    }
 
-	return NULL;
-    }
-};
+    return NULL;
+}
 
 /******************************************************************************
  * General (OSS) audio interface
@@ -183,67 +181,8 @@ class YOSSAudio : public YAudioInterface {
 public:
     YOSSAudio(): device(OSS_DEFAULT_DEVICE) {}
     
-    /**
-    * Play a sound sample directly to the digital signal processor.
-    */
-    virtual void play(int sound) {
-	if (device == NULL) return;
-
-	for(unsigned i(0); i < ACOUNT(gui_events); i++)
-	    if(gui_events[i].type == sound) {
-		char * samplefile(findSample(sound));
-
-#ifndef DEBUG
-		if (IceSound::verbose)
-#endif
-		    msg(_("Playing sample #%d (%s)"), sound, samplefile);
-
-		if (samplefile) {
-		    int ifd(open(samplefile, O_RDONLY));
-
-		    if(ifd == -1) {
-			warn("%s: %s", samplefile, strerror(errno));
-			return;
-		    }
-
-		    int ofd(open(device, O_WRONLY));
-
-		    if(ofd == -1) {
-			warn("%s: %s", device, strerror(errno));
-			close(ifd);
-			return;
-		    }
-		    
-		    if (IceSound::verbose)
-			msg("TODO: adjust audio format"); // !!!
-
-#ifdef DEBUG
-		    msg("copying sound %s to %s\n", samplefile, device);
-#endif
-		    delete[] samplefile;
-
-		    char sbuf[4096];
-		    for (int n; (n = read(ifd, sbuf, sizeof(sbuf))) > 0; )
-			write(ofd, sbuf, n);
-
-		    close(ofd);
-		    close(ifd);
-		}
-	    }
-    }
-
-    virtual int init(int & argc, char **& argv) {
-	int rc(0);
-
-	TRY(CommandLine(argc, argv, *this).parse())
-	
-	if (access(device, W_OK) != 0) {
-	    warn(_("No such device: %s"), device);
-	    THROW(3)
-	}
-
-	CATCH(/**/)
-    }
+    virtual void play(int sound);
+    virtual int init(int & argc, char **& argv);
     
 private:
     friend class CommandLine : public YCommandLine {
@@ -285,6 +224,68 @@ private:
     char const * device;
 };
 
+/**
+ * Play a sound sample directly to the digital signal processor.
+ */
+void YOSSAudio::play(int sound) {
+    if (device == NULL) return;
+
+    for(unsigned i(0); i < ACOUNT(gui_events); i++)
+	if(gui_events[i].type == sound) {
+	    char * samplefile(findSample(sound));
+
+#ifndef DEBUG
+	    if (IceSound::verbose)
+#endif
+		msg(_("Playing sample #%d (%s)"), sound, samplefile);
+
+	    if (samplefile) {
+		int ifd(open(samplefile, O_RDONLY));
+
+		if(ifd == -1) {
+		    warn("%s: %s", samplefile, strerror(errno));
+		    return;
+		}
+
+		int ofd(open(device, O_WRONLY));
+
+		if(ofd == -1) {
+		    warn("%s: %s", device, strerror(errno));
+		    close(ifd);
+		    return;
+		}
+		    
+		if (IceSound::verbose)
+		    msg("TODO: adjust audio format"); // !!!
+
+#ifdef DEBUG
+		msg("copying sound %s to %s\n", samplefile, device);
+#endif
+		delete[] samplefile;
+
+		char sbuf[4096];
+		for (int n; (n = read(ifd, sbuf, sizeof(sbuf))) > 0; )
+		    write(ofd, sbuf, n);
+
+		close(ofd);
+		close(ifd);
+	    }
+	}
+}
+
+int YOSSAudio::init(int & argc, char **& argv) {
+    int rc(0);
+
+    TRY(CommandLine(argc, argv, *this).parse())
+	
+    if (access(device, W_OK) != 0) {
+	warn(_("No such device: %s"), device);
+	THROW(3)
+    }
+
+    CATCH(/**/)
+}
+    
 /******************************************************************************
  * Enlightenment Sound Daemon audio interface
  ******************************************************************************/
@@ -317,22 +318,7 @@ private:
     unsigned unloadSamples();
 
     int uploadSample(int sound, char const * path);
-
-    virtual int init(int & argc, char **& argv) {
-	int rc(0);
-
-	TRY(CommandLine(argc, argv, *this).parse())
-	    
-	if ((socket = esd_open_sound(speaker)) == -1) {
-	    warn(_("Can't connect to ESound daemon: %s"),
-	    	   speaker ? speaker : _("<none>"));
-	    THROW(3);
-	}
-
-	CATCH(
-	    uploadSamples();
-	)
-    }
+    virtual int init(int & argc, char **& argv);
 
 private:
     friend class CommandLine : public YCommandLine {
@@ -378,6 +364,22 @@ protected:
     int socket;				// socket to ESound Daemon
 };
 
+int YESDAudio::init(int & argc, char **& argv) {
+    int rc(0);
+
+    TRY(CommandLine(argc, argv, *this).parse())
+	    
+    if ((socket = esd_open_sound(speaker)) == -1) {
+	warn(_("Can't connect to ESound daemon: %s"),
+	    speaker ? speaker : _("<none>"));
+	THROW(3);
+    }
+
+    CATCH(
+	uploadSamples();
+    )
+}
+
 /**
  * Upload a sample in the ESounD server.
  * Returns sample ID or negative on error.
@@ -482,30 +484,7 @@ public:
     
     virtual void play(int sound);
 
-    virtual int init(int & argc, char **& argv) {
-	YConnection * con(NULL);
-	int rc(0);
-
-	TRY(CommandLine(argc, argv, *this).parse())
-	
-	if (recorder == NULL) recorder = YIFF_DEFAULT_SERVER;
-	con = YOpenConnection(NULL, recorder);
-
-	if (NULL == con) {
-	    warn(_("Can't connect to YIFF server: %s"),
-		   recorder ? recorder : _("<none>"));
-	    THROW(3);
-	}
-
-	if(mode != NULL && YChangeAudioModePreset(con, mode))
-	    warn(_("Can't change to audio mode `%s'."), mode);
-
-	/* Now set descriptor to server, this is incase a SIGHUP
-         * or other async event occured during initialization. */
-	server = con;
-
-	CATCH(/**/)
-    }
+    virtual int init(int & argc, char **& argv);
 
     virtual void idle();
     
@@ -540,38 +519,7 @@ private:
 	    return '\0';
 	}
 
-	virtual int setOption(char const * arg, char opt, char const * val) {
-	    switch(opt) {
-		case 'S': // ====================================== recorder ===
-		case 'r':
-		    yiff.recorder = val;
-		    return 0;
-		    
-		case 'm': // ==================================== audio-mode ===
-		    if (yiff.mode != NULL)
-			warn(_("Overriding previous audio mode `%s'."),
-			       yiff.mode);
-
-		    if (val == NULL || strcmp(val, "?") == 0) {
-			yiff.listAudioModes();
-			return -1;
-		    }
-		
-		    yiff.mode = val;
-		    return 0;
-
-		case 'A': // =============================== audio-mode-auto ===
-		    if (yiff.mode != NULL)
-			warn(_("Overriding previous audio mode `%s'."),
-			       yiff.mode);
-
-		    yiff.matchMode = true;
-		    return 0;
-		    
-		default:
-		    return YCommandLine::setOption(arg, opt, val);
-	    }
-	}
+	virtual int setOption(char const * arg, char opt, char const * val);
 
     private:
 	YY2Audio & yiff;
@@ -584,6 +532,31 @@ protected:
     bool matchMode;			// Automantically adjust audio mode
 };
 
+int YY2Audio::init(int & argc, char **& argv) {
+    YConnection * con(NULL);
+    int rc(0);
+
+    TRY(CommandLine(argc, argv, *this).parse())
+    
+    if (recorder == NULL) recorder = YIFF_DEFAULT_SERVER;
+    con = YOpenConnection(NULL, recorder);
+
+    if (NULL == con) {
+        warn(_("Can't connect to YIFF server: %s"),
+           recorder ? recorder : _("<none>"));
+        THROW(3);
+    }
+
+    if(mode != NULL && YChangeAudioModePreset(con, mode))
+        warn(_("Can't change to audio mode `%s'."), mode);
+
+    /* Now set descriptor to server, this is incase a SIGHUP
+         * or other async event occured during initialization. */
+    server = con;
+
+    CATCH(/**/)
+}
+
 /**
  * Prints list of preset Audio modes on the Y server.
  */
@@ -760,6 +733,40 @@ void YY2Audio::idle() {
 	IceSound::running = (server != NULL);
     }
 }
+
+int YY2Audio::CommandLine::setOption(char const * arg, char opt, char const * val) {
+    switch(opt) {
+	case 'S': // ====================================== recorder ===
+	case 'r':
+	    yiff.recorder = val;
+	    return 0;
+	    
+	case 'm': // ==================================== audio-mode ===
+	    if (yiff.mode != NULL)
+		warn(_("Overriding previous audio mode `%s'."),
+		       yiff.mode);
+
+	    if (val == NULL || strcmp(val, "?") == 0) {
+		yiff.listAudioModes();
+		return -1;
+	    }
+		
+	    yiff.mode = val;
+	    return 0;
+
+	case 'A': // =============================== audio-mode-auto ===
+	    if (yiff.mode != NULL)
+		warn(_("Overriding previous audio mode `%s'."),
+		       yiff.mode);
+
+	    yiff.matchMode = true;
+	    return 0;
+		    
+	default:
+	    return YCommandLine::setOption(arg, opt, val);
+    }
+}
+
 #endif	/* ENABLE_YIFF */
 
 /******************************************************************************
@@ -847,7 +854,8 @@ int IceSound::run() {
     }
 
     if(NULL == (display = XOpenDisplay(dpyname))) { // ====== connect to X11 ===
-	warn(_("Can't open display: %s. X must be running and $DISPLAY set."));
+	warn(_("Can't open display: %s. X must be running and $DISPLAY set."),
+	       XDisplayName(dpyname));
 	THROW(3)
     }
 
