@@ -78,41 +78,35 @@ bool getWorkspace() {
 long bgCount;
 Pixmap defbg, bg[64] = { 0 };
 
-Pixmap loadPixmap(const char *fileName) {
+Pixmap loadPixmap(const char *filename) {
     Pixmap pixmap = 0;
 #ifdef CONFIG_IMLIB
     if(!hImlib) hImlib=Imlib_init(display);
 
-    ImlibImage *im = Imlib_load_image(hImlib, (char *)fileName);
+    ImlibImage *im = Imlib_load_image(hImlib, (char *)filename);
     if(im) {
         Imlib_render(hImlib, im, im->rgb_width, im->rgb_height);
         pixmap = (Pixmap)Imlib_move_image(hImlib, im);
         Imlib_destroy_image(hImlib, im);
     } else {
-        fprintf(stderr, _("Loading image %s failed"), fileName);
+        fprintf(stderr, _("Loading image %s failed"), filename);
         fputs("\n", stderr);
     }
 #else
     XpmAttributes xpmAttributes;
-    Pixmap fake;
-    int rc;
-
     xpmAttributes.colormap  = defaultColormap;
     xpmAttributes.closeness = 65535;
     xpmAttributes.valuemask = XpmSize|XpmReturnPixels|XpmColormap|XpmCloseness;
 
-    rc = XpmReadFileToPixmap(display, root,
-                             (char *)fileName,
-                             &pixmap, &fake,
-                             &xpmAttributes);
+    Pixmap mask;
+    int const rc(XpmReadFileToPixmap(display, root, (char *)filename,
+				     &pixmap, &mask, &xpmAttributes));
 
-    if (rc == 0) {
-        if (fake != None)
-            XFreePixmap(display, fake);
-    } else {
-        fprintf(stderr, _("Load pixmap %s failed with rc=%d"), fileName, rc);
-        fputs("\n", stderr);
-    }
+    if (rc != XpmSuccess)
+        warn(_("Loading of pixmap \"%s\" failed: %s"),
+	       filename, XpmGetErrorString(rc));
+    else
+	if (mask != None) XFreePixmap(display, mask);
 #endif
     return pixmap;
 }
