@@ -48,7 +48,7 @@ void logEvent(XEvent xev) {
     case MapRequest:
         break;
 #endif
-#if 0
+#if 1
     case MapNotify:
         msg("window=0x%lX: mapNotify event=0x%lX, override_redirect=%s",
             xev.xmap.window,
@@ -86,7 +86,7 @@ void logEvent(XEvent xev) {
         break;
 #endif
 
-#if 1
+#if 0
     case FocusIn:
     case FocusOut:
         msg("window=0x%lX: %s mode=%d, detail=%d",
@@ -113,6 +113,21 @@ void logEvent(XEvent xev) {
     case ColormapNotify:
         break;
 #endif
+
+#if 1
+    case ReparentNotify:
+        msg("window=0x%lX: reparentNotify event=0x%lX, parent=0x%lX, (%d:%d), override_redirect=%s",
+            xev.xreparent.window,
+            xev.xreparent.event,
+            xev.xreparent.parent,
+            xev.xreparent.x, xev.xreparent.y,
+            xev.xreparent.override_redirect ? "True" : "False");
+        break;
+#else
+    case ReparentNotify:
+        break;
+#endif
+
 #if 0
     case ConfigureNotify:
         msg("window=0x%lX: configureNotify event=0x%lX, (%d:%d-%dx%d) border_width=%d, above=0x%lX, override_redirect=%s",
@@ -220,7 +235,7 @@ void logEvent(XEvent xev) {
     case LeaveNotify:
         break;
 #endif
-#if 1
+#if 0
     case KeyPress:
     case KeyRelease:
         msg("window=0x%lX: %s root=0x%lX, subwindow=0x%lX, time=%ld, (%d:%d %d:%d) state=0x%X keycode=0x%x same_screen=%s",
@@ -257,41 +272,43 @@ void logEvent(XEvent xev) {
 }
 #endif
 
-void die(int exitcode, const char *msg, ...) {
-    va_list ap;
-
+void die(int exitcode, char const *msg, ...) {
     fprintf(stderr, "%s: ", YApplication::Name);
+
+    va_list ap;
     va_start(ap, msg);
     vfprintf(stderr, msg, ap);
     va_end(ap);
     fputs("\n", stderr);
+
     exit(exitcode);
 }
 
-void warn(const char *msg, ...) {
-    va_list ap;
-
+void warn(char const *msg, ...) {
     fprintf(stderr, "%s: ", YApplication::Name);
     fputs(_("Warning: "), stderr);
+
+    va_list ap;
     va_start(ap, msg);
     vfprintf(stderr, msg, ap);
     va_end(ap);
+
     fputs("\n", stderr);
 }
 
-void msg(const char *msg, ...) {
-    va_list ap;
-
+void msg(char const *msg, ...) {
     fprintf(stderr, "%s: ", YApplication::Name);
+
+    va_list ap;
     va_start(ap, msg);
     vfprintf(stderr, msg, ap);
     va_end(ap);
     fputs("\n", stderr);
 }
 
-char *strJoin(const char *str, ...) {
+char *strJoin(char const *str, ...) {
     va_list ap;
-    const char *s;
+    char const *s;
     char *res, *p;
     int len = 0;
 
@@ -346,24 +363,56 @@ void FREE(void *p) {
         free(p);
 }
 
-char *newstr(const char *str) {
-    if (!str)
-        return 0;
-    int len = strlen(str) + 1;
-    char *s = new char[len];
-    if (s)
-        memcpy(s, str, len);
-    return s;
+#if __GNUC__ == 3
+
+extern "C" void __cxa_pure_virtual() {
+    warn ("BUG: Pure virtual method called. Terminating.");
+    abort();
 }
 
-char *newstr(const char *str, int len) {
-    if (!str)
-        return 0;
-    char *s = new char[len + 1];
-    if (s) {
+#endif
+
+#ifdef NEED_ALLOC_OPERATORS
+
+#warning Using custom allocation operators. Expect trouble.
+// !!! Trouble: actionLogout is broken with them. What else?
+
+void *operator new(size_t len) {
+    return MALLOC(len);
+}
+
+void operator delete (void *p) {
+    FREE(p);
+}
+
+void *operator new[](size_t len) {
+    if (len == 0) 
+        len = 1;
+    return MALLOC(len);
+}
+
+void operator delete[](void *p) {
+    FREE(p);
+}
+
+#endif
+
+char *newstr(char const *str) {
+    return (str != NULL ? newstr(str, strlen(str)) : NULL);
+}
+
+char *newstr(char const *str, char const *delim) {
+    return (str != NULL ? newstr(str, strcspn(str, delim)) : NULL);
+}
+
+char *newstr(char const *str, int len) {
+    char *s(NULL);
+    
+    if (str != NULL && (s = new char[len + 1]) != NULL) {
         memcpy(s, str, len);
-        s[len] = 0;
+        s[len] = '\0';
     }
+
     return s;
 }
 
@@ -375,14 +424,43 @@ char *newstr(const char *str, int len) {
  *
  *		"--interface=/tmp" "--interface"
  */
-int strpcmp(char const * str, char const * pfx, char const * dlim) {
+int strpcmp(char const * str, char const * pfx, char const * delim) {
     if(str == NULL || pfx == NULL) return -1;
     while(*pfx == *str && *pfx != '\0') ++str, ++pfx;
     
-    return (*pfx == '\0' && strchr(dlim, *str) ? 0 : *str - *pfx);
+    return (*pfx == '\0' && strchr(delim, *str) ? 0 : *str - *pfx);
 }
 
-char const * basename(char const * path) {
-    char const * base = ::strrchr(path, DIR_DELIMINATOR);
+char const * strnxt(const char * str, const char * delim) {
+    str+= strcspn(str, delim);
+    str+= strspn(str, delim);
+    return str;
+
+}
+
+/*
+ *	Counts the tokens separated by delim
+ */
+unsigned strTokens(const char * str, const char * delim) {
+    unsigned count = 0;
+
+    if (str)
+	while (*str) { 
+	    str = strnxt(str, delim);
+	    ++count;
+        }
+
+    return count;	     
+}
+
+#ifndef HAVE_BASENAME
+const * basename(char const *path) {
+    char * base = ::strrchr(path, DIR_DELIMINATOR);
     return (base ? base + 1 : path);
 }
+#endif
+
+bool isreg(char const *path) {
+    struct stat sb;
+    return (stat(path, &sb) == 0 && S_ISREG(sb.st_mode));
+}
