@@ -4,6 +4,7 @@
  * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
+
 #include "yfull.h"
 #include "ykey.h"
 #include "wmframe.h"
@@ -13,10 +14,11 @@
 #include "yapp.h"
 #include "wmapp.h"
 
+#include <stdio.h>
+
 void YFrameWindow::snapTo(int &wx, int &wy,
                           int rx1, int ry1, int rx2, int ry2,
-                          int &flags)
-{
+                          int &flags) {
     int d = snapDistance;
 
     if (flags & 4) { // snap to container window (root, workarea)
@@ -108,20 +110,20 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
     
     // try snapping to the border first
     flags |= 4;
-    if (xp < manager->minX(getLayer()) || xp + int(width()) > manager->maxX(getLayer())) {
+    if (xp < manager->minX(this) || 
+        xp + int(width()) > manager->maxX(this)) {
         xp += borderX();
         flags |= 8;
     }
-    if (yp < manager->minY(getLayer()) || yp + int(height()) > manager->maxY(getLayer())) {
+    if (yp < manager->minY(this) || 
+        yp + int(height()) > manager->maxY(this)) {
         yp += borderY();
         flags |= 16;
     }
-    snapTo(xp, yp,
-           manager->minX(getLayer()),
-           manager->minY(getLayer()),
-           manager->maxX(getLayer()),
-           manager->maxY(getLayer()),
-           flags);
+
+    snapTo(xp, yp, manager->minX(this), manager->minY(this),
+		   manager->maxX(this), manager->maxY(this), flags);
+
     if (flags & 8) {
         xp -= borderX();
         flags &= ~8;
@@ -168,43 +170,296 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
     wy = yp;
 }
 
-void YFrameWindow::drawOutline(int x, int y, int w, int h) {
-    int bw = (wsBorderX + wsBorderY) / 2;
-    static GC outlineGC = None;
+/******************************************************************************/
 
-    if (outlineGC == None) {
+#ifdef CONFIG_MOVESIZE_FX
+void YFrameWindow::drawMoveSizeFX(int x, int y, int w, int h, bool /*interior*/) {
+    struct FXFrame { 
+	FXFrame(int const l, int const c, int const r,
+		int const t, int const m, int const b):
+	    l(l), c(c), r(r), t(t), m(m), b(b) {}
+
+	int const l, c, r, t, m, b;
+    };
+
+    struct FXRect { 
+	FXRect(int const l, int const r, int const t, int const b): 
+	    l(l), r(r), t(t), b(b) {}
+
+	int const l, r, t, b;
+    };
+
+    static YFont * font(NULL);
+    static Graphics * gc(NULL);
+
+    if (font == NULL)
+	font = YFont::getFont(moveSizeFontName, false);
+
+    if (gc == NULL) {
+        XGCValues gcv;
+
+        gcv.function = GXxor;
+        gcv.line_width = 0;
+        gcv.subwindow_mode = IncludeInferiors;
+        gcv.graphics_exposures = False;
+
+        gc = new Graphics(*desktop, GCFunction | GCGraphicsExposures | 
+				    GCLineWidth | GCSubwindowMode, &gcv);
+        gc->setFont(font);
+        gc->setColor(new YColor(clrActiveBorder));
+    }
+
+    const int dBase(font->height() + 4);
+    const int titlebar(moveSizeInterior & 2 ? titleY() : 0);
+
+    FXFrame const frame(x, x + w/2, x + w - 1, y, y + h/2, y + h - 1);
+    FXRect const client(x + borderX(), x + w - borderX() - 1, 
+    			y + borderY() + titlebar, y + h - borderY() - 1);
+    FXRect const dimBase(frame.l - dBase, frame.r + dBase,
+    			 frame.t - dBase, frame.b + dBase);
+    FXRect const dimLine(moveSizeGaugeLines & 1 ? dimBase.l - 4 : frame.l,
+    			 moveSizeGaugeLines & 2 ? dimBase.r + 4 : frame.r,
+    			 moveSizeGaugeLines & 4 ? dimBase.t - 4 : frame.t,
+			 moveSizeGaugeLines & 8 ? dimBase.b + 4 : frame.b);
+    FXRect const outerLabel(dimBase.l + font->descent() + 2,
+			    dimBase.r - font->ascent() - 2,
+			    dimBase.t + font->ascent() + 2,
+			    dimBase.b - font->descent() - 2);
+    FXRect const desktop(0, desktop->width() - 1, 0, desktop->height() - 1);
+
+    int const cw(client.r - client.l + 1);
+    int const ch(client.b - client.t + 1);
+
+/*** FX: Frame Border *********************************************************/
+
+    if (!((movingWindow && opaqueMove) ||
+	  (sizingWindow && opaqueResize)))
+	if (moveSizeInterior & 1) {
+	    XRectangle border[] = {
+		{ frame.l, frame.t, w, h }, 
+		{ client.l, client.t, cw, ch }
+	    };
+
+	    gc->drawRects(border, frameDecors() & fdBorder ? 2 : 1);
+	} else {
+            XRectangle border[] = {
+		{ frame.l, frame.t, frame.r - frame.l + 1, client.t - frame.t + 1 },
+		{ frame.l, client.t + 1, client.l - frame.l + 1, client.b - client.t - 1 },
+		{ client.r, client.t + 1, frame.r - client.r + 1, client.b - client.t - 1 },
+		{ frame.l, client.b, frame.r - frame.l + 1, frame.b - client.b + 1 }
+	     };
+
+	     gc->fillRects(border, 4);
+	}
+    
+/*** FX: Dimension Lines ******************************************************/
+    
+    if (moveSizeDimLines & 00001)
+	gc->drawLine(dimLine.r, frame.t, desktop.r, frame.t);
+    if (moveSizeDimLines & 00002)
+	gc->drawLine(dimLine.r, frame.m, desktop.r, frame.m);
+    if (moveSizeDimLines & 00004)
+	gc->drawLine(dimLine.r, frame.b, desktop.r, frame.b);
+
+    if (moveSizeDimLines & 00010)
+	gc->drawLine(desktop.l, frame.t, dimLine.l, frame.t);
+    if (moveSizeDimLines & 00020)
+	gc->drawLine(desktop.l, frame.m, dimLine.l, frame.m);
+    if (moveSizeDimLines & 00040)
+	gc->drawLine(desktop.l, frame.b, dimLine.l, frame.b);
+
+    if (moveSizeDimLines & 00100)
+	gc->drawLine(frame.l, desktop.t, frame.l, dimLine.t);
+    if (moveSizeDimLines & 00200)
+	gc->drawLine(frame.c, desktop.t, frame.c, dimLine.t);
+    if (moveSizeDimLines & 00400)
+	gc->drawLine(frame.r, desktop.t, frame.r, dimLine.t);
+	
+    if (moveSizeDimLines & 01000)
+	gc->drawLine(frame.l, dimLine.b, frame.l, desktop.b);
+    if (moveSizeDimLines & 02000)
+	gc->drawLine(frame.c, dimLine.b, frame.c, desktop.b);
+    if (moveSizeDimLines & 04000)
+	gc->drawLine(frame.r, dimLine.b, frame.r, desktop.b);
+
+/*** FX: Dimension Base Lines *************************************************/
+    
+    if (moveSizeGaugeLines & 1)
+	gc->drawLine(frame.l, dimBase.t, frame.r, dimBase.t);
+    if (moveSizeGaugeLines & 2)
+	gc->drawLine(dimBase.l, frame.t, dimBase.l, frame.b);
+    if (moveSizeGaugeLines & 4)
+	gc->drawLine(dimBase.r, frame.t, dimBase.r, frame.b);
+    if (moveSizeGaugeLines & 8)
+	gc->drawLine(frame.l, dimBase.b, frame.r, dimBase.b);
+	
+/*** FX: Dimension Labels *****************************************************/
+    
+    if (moveSizeDimLabels & 01001) {
+	char const * label(itoa(x));
+	int const pos(frame.l);
+	
+	if (moveSizeDimLabels & 00001)
+	    gc->drawString(pos, outerLabel.t, label);
+	if (moveSizeDimLabels & 01000)
+	    gc->drawString(pos, outerLabel.b, label);
+    }
+
+    if (moveSizeDimLabels & 02002) {
+	char const * label(itoa(w));
+	int const pos(frame.c - font->textWidth(label)/2);
+	
+	if (moveSizeDimLabels & 00002)
+	    gc->drawString(pos, outerLabel.t, label);
+	if (moveSizeDimLabels & 02000)
+	    gc->drawString(pos, outerLabel.b, label);
+    }
+
+    if (moveSizeDimLabels & 04004) {
+	char const * label(itoa(x + w));
+	int const pos(frame.r - font->textWidth(label));
+	
+	if (moveSizeDimLabels & 00004)
+	    gc->drawString(pos, outerLabel.t, label);
+	if (moveSizeDimLabels & 04000)
+	    gc->drawString(pos, outerLabel.b, label);
+    }
+
+    if (moveSizeDimLabels & 00110) {
+	char const * label(itoa(y));
+	int const pos(frame.t);
+	
+	if (moveSizeDimLabels & 00010)
+	    gc->drawString270(outerLabel.l, pos, label);
+	if (moveSizeDimLabels & 00100)
+	    gc->drawString90(outerLabel.r, pos, label);
+    }
+
+    if (moveSizeDimLabels & 00220) {
+	char const * label(itoa(h));
+	int const pos(frame.m - font->textWidth(label)/2);
+	
+	if (moveSizeDimLabels & 00020)
+	    gc->drawString270(outerLabel.l, pos, label);
+	if (moveSizeDimLabels & 00200)
+	    gc->drawString90(outerLabel.r, pos, label);
+    }
+
+    if (moveSizeDimLabels & 00440) {
+	char const * label(itoa(x + h));
+	int const pos(frame.b - font->textWidth(label));
+	
+	if (moveSizeDimLabels & 00040)
+	    gc->drawString270(outerLabel.l, pos, label);
+	if (moveSizeDimLabels & 00400)
+	    gc->drawString90(outerLabel.r, pos, label);
+    }
+    
+/*** FX: Geometry Labels ******************************************************/
+    
+    if (moveSizeGeomLabels & 0x7f) {
+	static char label[50];
+	snprintf(label, sizeof(label), "%dx%d+%d+%d", w, h, x, y);
+
+	int const tw(font->textWidth(label));
+	int const th(font->height());
+
+	FXFrame const innerLabel
+	    (client.l + 2, frame.c - tw/2, client.r - tw - 2,
+	     client.t + font->ascent() + 2,
+	     frame.m + th/2 - font->descent(),
+	     client.b - font->descent() - 2);
+
+	if (moveSizeGeomLabels & 0x01)
+	    gc->drawString(innerLabel.l, innerLabel.t, label);
+	if (moveSizeGeomLabels & 0x02)
+	    gc->drawString(innerLabel.c, innerLabel.t, label);
+	if (moveSizeGeomLabels & 0x04)
+	    gc->drawString(innerLabel.r, innerLabel.t, label);
+	if (moveSizeGeomLabels & 0x08)
+	    gc->drawString(innerLabel.c, innerLabel.m, label);
+	if (moveSizeGeomLabels & 0x10)
+	    gc->drawString(innerLabel.l, innerLabel.b, label);
+	if (moveSizeGeomLabels & 0x20)
+	    gc->drawString(innerLabel.c, innerLabel.b, label);
+	if (moveSizeGeomLabels & 0x40)
+	    gc->drawString(innerLabel.r, innerLabel.b, label);
+    }
+
+/*** FX: Grids ****************************************************************/
+
+    if (moveSizeInterior & 4) {
+        XSegment grid[] = { 
+	    { client.l + cw * 1/3, client.t + 1,
+	      client.l + cw * 1/3, client.b },
+	    { client.l + cw * 2/3, client.t + 1,
+	      client.l + cw * 2/3, client.b },
+	    { client.l + 1, client.t + ch * 1/3,
+	      client.r, client.t + ch * 1/3 },
+	    { client.l + 1, client.t + ch * 2/3,
+	      client.r, client.t + ch * 2/3 }
+	};
+			  
+	gc->drawSegments(grid, 4);
+    }
+
+    if (moveSizeInterior & 8) {
+        XSegment grid[] = { 
+	    { client.l + cw / 2, client.t + 1,
+	      client.l + cw / 2, client.b },
+	    { client.l + 1, client.t + ch / 2,
+	      client.r, client.t + ch / 2 }
+	};
+
+	gc->drawSegments(grid, 2);
+    }
+
+    if (moveSizeInterior & 8) {
+        XSegment grid[] = { 
+	    { client.l + 1, client.t + 1, client.r, client.b },
+	    { client.r, client.t + 1, client.l + 1, client.b }
+	};
+
+	gc->drawSegments(grid, 2);
+    }
+}
+#else
+void YFrameWindow::drawMoveSizeFX(int x, int y, int w, int h, bool) {
+    if ((movingWindow && opaqueMove) ||
+	(sizingWindow && opaqueResize))
+	return;
+
+    int const bw((wsBorderX + wsBorderY) / 2);
+    int const bo((wsBorderX + wsBorderY) / 4);
+    static Graphics * gc(NULL);
+
+    if (gc == NULL) {
         XGCValues gcv;
 
         gcv.foreground = YColor(clrActiveBorder).pixel();
         gcv.function = GXxor;
         gcv.graphics_exposures = False;
-        gcv.line_width = (wsBorderX + wsBorderY) / 2;
+        gcv.line_width = bw;
         gcv.subwindow_mode = IncludeInferiors;
 
-        outlineGC = XCreateGC(app->display(), desktop->handle(),
-                              GCForeground |
-                              GCFunction |
-                              GCGraphicsExposures |
-                              GCLineWidth |
-                              GCSubwindowMode,
-                              &gcv);
+        gc = new Graphics(*desktop, GCForeground | GCFunction |
+				    GCGraphicsExposures | GCLineWidth |
+				    GCSubwindowMode, &gcv);
     }
 
-    x += bw / 2;
-    y += bw / 2;
-    w -= bw;
-    h -= bw;
-    XDrawRectangle(app->display(), manager->handle(), outlineGC,
-                   x, y, w, h);
+    gc->drawRect(x + bo, y + bo, w - bw, h - bw);
 }
+#endif
 
 int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
-    if (m == ShiftMask)
+    if (m & ShiftMask)
         factor = 4;
+    if (m & ControlMask)
+        factor<<= 4;
 
     if (k == XK_Left || k == XK_KP_Left)
         newX -= factor;
@@ -215,14 +470,19 @@ int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     else if (k == XK_Down || k == XK_KP_Down)
         newY += factor;
     else if (k == XK_Home || k == XK_KP_Home)
-        newX = manager->minX(getLayer()) - borderX();
+        newX = manager->minX(this) - borderX();
     else if (k == XK_End || k == XK_KP_End)
-        newX = manager->maxX(getLayer()) - width() + borderX();
+        newX = manager->maxX(this) - width() + borderX();
     else if (k == XK_Prior || k == XK_KP_Prior)
-        newY = manager->minY(getLayer()) - borderY();
+        newY = manager->minY(this) - borderY();
     else if (k == XK_Next || k == XK_KP_Next)
-        newY = manager->maxY(getLayer()) - height() + borderY();
-    else if (k == XK_Return || k == XK_KP_Enter)
+        newY = manager->maxY(this) - height() + borderY();
+    else if (k == XK_KP_Begin) {
+	newX = (manager->minX(getLayer()) + 
+		manager->maxX(getLayer()) - (int)width()) / 2;
+	newY = (manager->minY(getLayer()) + 
+		manager->maxY(getLayer()) - (int)height()) / 2;
+    } else if (k == XK_Return || k == XK_KP_Enter)
         return -1;
     else if (k ==  XK_Escape) {
         newX = origX;
@@ -233,6 +493,8 @@ int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     return 1;
 }
 
+/******************************************************************************/
+
 int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
                                    int &newX, int &newY, int &newWidth, int &newHeight,
                                    int incX, int incY)
@@ -241,8 +503,10 @@ int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
-    if (m == ShiftMask)
+    if (m & ShiftMask)
         factor = 4;
+    if (m & ControlMask)
+        factor<<= 4;
 
     if (k == XK_Left || k == XK_KP_Left) {
         if (grabX == 0) {
@@ -310,33 +574,33 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
 
     if (!(motion.state & app->AltMask)) {
         if (EdgeResistance == 10000) {
-            if (newX + int(width() + n * borderX()) > manager->maxX(getLayer()))
-                newX = manager->maxX(getLayer()) - width() - n * borderX();
-            if (newY + int(height() + n * borderY()) > manager->maxY(getLayer()))
-                newY = manager->maxY(getLayer()) - height() - n * borderY();
-            if (newX < manager->minX(getLayer()))
-                newX = manager->minX(getLayer());
-            if (newY < manager->minY(getLayer()))
-                newY = manager->minY(getLayer());
+            if (newX + int(width() + n * borderX()) > manager->maxX(this))
+                newX = manager->maxX(this) - width() - n * borderX();
+            if (newY + int(height() + n * borderY()) > manager->maxY(this))
+                newY = manager->maxY(this) - height() - n * borderY();
+            if (newX < manager->minX(this))
+                newX = manager->minX(this);
+            if (newY < manager->minY(this))
+                newY = manager->minY(this);
         } else if (/*EdgeResistance >= 0 && %%% */ EdgeResistance < 10000) {
-            if (newX + int(width() + n * borderX()) > manager->maxX(getLayer()))
-                if (newX + int(width() + n * borderX()) < int(manager->maxX(getLayer()) + EdgeResistance))
-                    newX = manager->maxX(getLayer()) - width() - n * borderX();
+            if (newX + int(width() + n * borderX()) > manager->maxX(this))
+                if (newX + int(width() + n * borderX()) < int(manager->maxX(this) + EdgeResistance))
+                    newX = manager->maxX(this) - width() - n * borderX();
                 else if (motion.state & ShiftMask)
                     newX -= EdgeResistance;
-            if (newY + int(height() + n * borderY()) > manager->maxY(getLayer()))
-                if (newY + int(height() + n * borderY()) < int(manager->maxY(getLayer()) + EdgeResistance))
-                    newY = manager->maxY(getLayer()) - height() - n * borderY();
+            if (newY + int(height() + n * borderY()) > manager->maxY(this))
+                if (newY + int(height() + n * borderY()) < int(manager->maxY(this) + EdgeResistance))
+                    newY = manager->maxY(this) - height() - n * borderY();
                 else if (motion.state & ShiftMask)
                     newY -= EdgeResistance;
-            if (newX < manager->minX(getLayer()))
-                if (newX > int(- EdgeResistance + manager->minX(getLayer())))
-                    newX = manager->minX(getLayer());
+            if (newX < manager->minX(this))
+                if (newX > int(- EdgeResistance + manager->minX(this)))
+                    newX = manager->minX(this);
                 else if (motion.state & ShiftMask)
                     newX += EdgeResistance;
-            if (newY < manager->minY(getLayer()))
-                if (newY > int(- EdgeResistance + manager->minY(getLayer())))
-                    newY = manager->minY(getLayer());
+            if (newY < manager->minY(this))
+                if (newY > int(- EdgeResistance + manager->minY(this)))
+                    newY = manager->minY(this);
                 else if (motion.state & ShiftMask)
                     newY += EdgeResistance;
         }
@@ -384,79 +648,87 @@ void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
 }
 
 void YFrameWindow::outlineMove() {
-    int xx = x(), yy = y();
-    unsigned int modifiers = 0;
+    int xx(x()), yy(y());
+    unsigned modifiers(0);
 
     XGrabServer(app->display());
     XSync(app->display(), False);
-    drawOutline(xx, yy, width(), height());
-    while (1) {
+
+    for(;;) {
         XEvent xev;
 
-        XWindowEvent(app->display(),
-                     handle(),
-                     KeyPressMask | ExposureMask |
-                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask, &xev);
+        XWindowEvent(app->display(), handle(),
+                     KeyPressMask | ExposureMask | 
+		     ButtonPressMask | ButtonReleaseMask | 
+		     PointerMotionMask, &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            modifiers = xev.xkey.state;
-            {
-                int ox = xx;
-                int oy = yy;
-                int r;
+	    case KeyPress: {
+		modifiers = xev.xkey.state;
+
+                int const ox(xx), oy(yy);
+		int r;
 
                 switch (r = handleMoveKeys(xev.xkey, xx, yy)) {
-                case 0:
-                    break;
-                case 1:
-                case -2:
-                    if (xx != ox || yy != oy) {
-                        drawOutline(ox, oy, width(), height());
+		    case 1:
+		    case -2:
+			if (xx != ox || yy != oy) {
+			    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, width(), height());
+			    statusMoveSize->setStatus(this, xx, yy, width(), height());
 #endif
-                        drawOutline(xx, yy, width(), height());
-                    }
-                    if (r == -2)
-                        goto end;
-                    break;
-                case -1:
-                    goto end;
+			    drawMoveSizeFX(xx, yy, width(), height());
+			}
+
+			if (r == -2)
+			    goto end;
+
+			break;
+
+		    case 0:
+			break;
+
+		    case -1:
+			goto end;
                 }
+
+		break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            modifiers = xev.xbutton.state;
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
+
+	    case ButtonPress:
+	    case ButtonRelease:
+		modifiers = xev.xbutton.state;
+		goto end;
+
+	    case MotionNotify: {
+                int const ox(xx), oy(yy);
 
                 handleMoveMouse(xev.xmotion, xx, yy);
+
                 if (xx != ox || yy != oy) {
+                    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
                     statusMoveSize->setStatus(this, xx, yy, width(), height());
 #endif
-                    drawOutline(ox, oy, width(), height());
-                    drawOutline(xx, yy, width(), height());
+                    drawMoveSizeFX(xx, yy, width(), height());
                 }
+
+		break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, width(), height());
+    drawMoveSizeFX(xx, yy, width(), height());
+
     XSync(app->display(), False);
     moveWindow(xx, yy);
     XUngrabServer(app->display());
 }
 
 void YFrameWindow::outlineResize() {
-    int xx = x(), yy = y(), ww = width(), hh = height();
-    int incX = 1;
-    int incY = 1;
+    int xx(x()), yy(y()), ww(width()), hh(height());
+    int incX(1), incY(1);
 
     if (client()->sizeHints()) {
         incX = client()->sizeHints()->width_inc;
@@ -465,74 +737,79 @@ void YFrameWindow::outlineResize() {
 
     XGrabServer(app->display());
     XSync(app->display(), False);
-    drawOutline(xx, yy, ww, hh);
-    while (1) {
+
+    for(;;) {
         XEvent xev;
 
-        XWindowEvent(app->display(),
-                     handle(),
+        XWindowEvent(app->display(), handle(),
                      KeyPressMask | ExposureMask |
-                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask, &xev);
+                     ButtonPressMask | ButtonReleaseMask | 
+		     PointerMotionMask, &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            {
-                int ox = xx;
-                int oy = yy;
-                int ow = ww;
-                int oh = hh;
+            case KeyPress: {
+                int const ox(xx), oy(yy), ow(ww), oh(hh);
                 int r;
 
-                switch (r = handleResizeKeys(xev.xkey, xx, yy, ww, hh, incX, incY)) {
-                case 0:
-                    break;
-                case -2:
-                case 1:
-                    if (ox != xx || oy != yy || ow != ww || oh != hh) {
-                        drawOutline(ox, oy, ow, oh);
+                switch (r = handleResizeKeys(xev.xkey, xx, yy, ww, hh,
+					     incX, incY)) {
+		    case -2:
+		    case 1:
+			if (ox != xx || oy != yy || ow != ww || oh != hh) {
+			    drawMoveSizeFX(ox, oy, ow, oh);
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, ww, hh);
+			    statusMoveSize->setStatus(this, xx, yy, ww, hh);
 #endif
-                        drawOutline(xx, yy, ww, hh);
-                    }
-                    if (r == -2)
-                        goto end;
-                    break;
-                case -1:
-                    goto end;
+			    drawMoveSizeFX(xx, yy, ww, hh);
+			}
+
+			if (r == -2)
+			    goto end;
+
+			break;
+
+		    case 0:
+			break;
+
+		    case -1:
+			goto end;
                 }
+		
+		break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
-                int ow = ww;
-                int oh = hh;
+
+	    case ButtonPress:
+	    case ButtonRelease:
+		goto end;
+
+	    case MotionNotify: {
+		int const ox(xx), oy(yy), ow(ww), oh(hh);
 
                 handleResizeMouse(xev.xmotion, xx, yy, ww,hh);
+
                 if (ox != xx || oy != yy || ow != ww || oh != hh) {
-                    drawOutline(ox, oy, ow, oh);
+                    drawMoveSizeFX(ox, oy, ow, oh);
 #ifndef LITE
                     statusMoveSize->setStatus(this, xx, yy, ww, hh);
 #endif
-                    drawOutline(xx, yy, ww, hh);
+                    drawMoveSizeFX(xx, yy, ww, hh);
                 }
+
+		break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, ww, hh);
+    drawMoveSizeFX(xx, yy, ww, hh);
+
     XSync(app->display(), False);
     setGeometry(xx, yy, ww, hh);
     XUngrabServer(app->display());
 }
 
 void YFrameWindow::manualPlace() {
-    int xx = x(), yy = y();
+    int xx(x()), yy(y());
 
     grabX = borderX();
     grabY = borderY();
@@ -548,68 +825,80 @@ void YFrameWindow::manualPlace() {
                          ButtonPressMask |
                          ButtonReleaseMask |
                          PointerMotionMask))
-        return ;
+        return;
+
     XGrabServer(app->display());
 #ifndef LITE
     statusMoveSize->begin(this);
 #endif
-    drawOutline(xx, yy, width(), height());
-    while (1) {
+
+    drawMoveSizeFX(xx, yy, width(), height());
+
+    for(;;) {
         XEvent xev;
 
         XMaskEvent(app->display(),
                    KeyPressMask |
                    ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                    &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            {
-                int ox = xx;
-                int oy = yy;
+	    case KeyPress: {
+                int const ox(xx), oy(yy);
+
                 int r;
 
                 switch (r = handleMoveKeys(xev.xkey, xx, yy)) {
-                case 0:
-                    break;
-                case 1:
-                case -2:
-                    if (xx != ox || yy != oy) {
-                        drawOutline(ox, oy, width(), height());
+		    case 1:
+		    case -2:
+			if (xx != ox || yy != oy) {
+			    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, width(), height());
+                            statusMoveSize->setStatus(this, xx, yy, width(), height());
 #endif
-                        drawOutline(xx, yy, width(), height());
-                    }
-                    if (r == -2)
-                        goto end;
-                    break;
-                case -1:
-                    goto end;
+                            drawMoveSizeFX(xx, yy, width(), height());
+			}
+
+			if (r == -2)
+			    goto end;
+
+			break;
+
+		    case 0:
+			break;
+
+		    case -1:
+			goto end;
                 }
+
+		break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
+	    
+
+	    case ButtonPress:
+	    case ButtonRelease:
+		goto end;
+
+	    case MotionNotify: {
+                int const ox(xx), oy(yy);
 
                 handleMoveMouse(xev.xmotion, xx, yy);
                 if (xx != ox || yy != oy) {
+                    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
                     statusMoveSize->setStatus(this, xx, yy, width(), height());
 #endif
-                    drawOutline(ox, oy, width(), height());
-                    drawOutline(xx, yy, width(), height());
+                    drawMoveSizeFX(xx, yy, width(), height());
                 }
+
+		break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, width(), height());
+    drawMoveSizeFX(xx, yy, width(), height());
+
 #ifndef LITE
     statusMoveSize->end();
 #endif
@@ -668,18 +957,20 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                 if (grabY == -1)
                     newY = y() + height() - newHeight;
 
-                setGeometry(newX,
-                            newY,
-                            newWidth,
-                            newHeight);
+		drawMoveSizeFX(x(), y(), width(), height());
+                setGeometry(newX, newY, newWidth, newHeight);
+		drawMoveSizeFX(x(), y(), width(), height());
 
 #ifndef LITE
                 statusMoveSize->setStatus(this);
 #endif
                 break;
             case -2:
+		drawMoveSizeFX(x(), y(), width(), height());
                 setGeometry(newX, newY, newWidth, newHeight);
+		drawMoveSizeFX(x(), y(), width(), height());
                 /* nobreak */
+
             case -1:
                 endMoveSize();
                 break;
@@ -754,14 +1045,8 @@ void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEven
 }
 
 void YFrameWindow::constrainMouseToWorkspace(int &x, int &y) {
-    if (x < manager->minX(getLayer()))
-        x = manager->minX(getLayer());
-    else if (x >= manager->maxX(getLayer()))
-        x = manager->maxX(getLayer()) - 1;
-    if (y < manager->minY(getLayer()))
-        y = manager->minY(getLayer());
-    else if (y >= manager->maxY(getLayer()))
-        y = manager->maxY(getLayer()) - 1;
+    x = clamp(x, manager->minX(this), manager->maxX(this) - 1);
+    y = clamp(y, manager->minY(this), manager->maxY(this) - 1);
 }
 
 bool YFrameWindow::canSize(bool horiz, bool vert) {
@@ -789,8 +1074,7 @@ bool YFrameWindow::canMove() {
 
 void YFrameWindow::startMoveSize(int doMove, int byMouse,
                                  int sideX, int sideY,
-                                 int mouseXroot, int mouseYroot)
-{
+                                 int mouseXroot, int mouseYroot) {
     Cursor grabPointer = None;
 
     sizeByMouse = byMouse;
@@ -866,6 +1150,9 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
 #ifndef LITE
     statusMoveSize->begin(this);
 #endif
+
+    drawMoveSizeFX(x(), y(), width(), height());
+
     if (doMove && !opaqueMove) {
         outlineMove();
         endMoveSize();
@@ -880,10 +1167,16 @@ void YFrameWindow::endMoveSize() {
 #ifndef LITE
     statusMoveSize->end();
 #endif
+
+    if ((movingWindow && opaqueMove) ||
+	(sizingWindow && opaqueResize))
+	drawMoveSizeFX(x(), y(), width(), height());
+
     movingWindow = 0;
     sizingWindow = 0;
 
-    updateNormalSize();
+    if (client()) // !!! this can happen at destruction
+        updateNormalSize();
 }
 
 void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
@@ -904,12 +1197,12 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 
         if (grabY != 0 && grabX == 0) {
             if (down.x < int(wsCornerX)) grabX = -1;
-            else if (width() - down.x <= wsCornerX) grabX = 1;
+            else if ((int)width() - down.x <= wsCornerX) grabX = 1;
         }
 
         if (grabX != 0 && grabY == 0) {
             if (down.y < int(wsCornerY)) grabY = -1;
-            else if (height() - down.y <= wsCornerY) grabY = 1;
+            else if ((int)height() - down.y <= wsCornerY) grabY = 1;
         }
 
         if (grabX != 0 || grabY != 0) {
@@ -922,20 +1215,21 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 }
 
 void YFrameWindow::moveWindow(int newX, int newY) {
-    if (newX >= int(manager->maxX(getLayer()) - borderX()))
-        newX = manager->maxX(getLayer()) - borderX();
+    if (!doNotCover()) {
+	newX = clamp(newX, (int)(manager->minX(this) + borderX() - width()),
+			   (int)(manager->maxX(this) - borderX()));
+	newY = clamp(newY, (int)(manager->minY(this) + borderY() - height()),
+			   (int)(manager->maxY(this) - borderY()));
+    }
 
-    if (newY >= int(manager->maxY(getLayer()) - borderY() - titleY()))
-        newY = manager->maxY(getLayer()) - borderY() - titleY();
-
-    if (newX < int(manager->minX(getLayer()) - (width() - borderX())))
-        newX = manager->minX(getLayer()) - (width() - borderX());
-
-    if (newY < int(manager->minY(getLayer()) - (height() - borderY())))
-        newY = manager->minY(getLayer()) - (height() - borderY());
+    if (opaqueMove)
+	drawMoveSizeFX(x(), y(), width(), height());
 
     setPosition(newX, newY);
 
+    if (opaqueMove)
+	drawMoveSizeFX(x(), y(), width(), height());
+
 #ifndef LITE
     statusMoveSize->setStatus(this);
 #endif
@@ -970,10 +1264,10 @@ void YFrameWindow::handleMotion(const XMotionEvent &motion) {
 
         handleResizeMouse(motion, newX, newY, newWidth, newHeight);
 
-        setGeometry(newX,
-                    newY,
-                    newWidth,
-                    newHeight);
+	drawMoveSizeFX(x(), y(), width(), height());
+        setGeometry(newX, newY, newWidth, newHeight);
+	drawMoveSizeFX(x(), y(), width(), height());
+
 #ifndef LITE
         statusMoveSize->setStatus(this);
 #endif
