@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998,1999 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  *
  * Dialogs
  */
@@ -17,18 +17,23 @@
 #include "sysdep.h"
 #include "WinMgr.h"
 
+#include "intl.h"
+
 AboutDlg *aboutDlg = 0;
 
 AboutDlg::AboutDlg(): YDialog() {
-    char title[128], copyright[128];
+    char const * version("IceWM "VERSION"-"RELEASE" ("HOSTOS"/"HOSTCPU")");
+    char * copyright(strJoin("Copyright ", _("(C)"), " 1997-2001 Marko Macek, ",
+			                   _("(C)"), " 2001 Mathias Hasselmann",
+					   NULL));
 
-    sprintf(title, "icewm " VERSION);
-    fProgTitle = new YLabel(title, this);
-    sprintf(copyright, "Copyright 1997-1999 Marko Macek");
+    fProgTitle = new YLabel(version, this);
     fCopyright = new YLabel(copyright, this);
-    fThemeNameS = new YLabel("Theme:", this);
-    fThemeDescriptionS = new YLabel("Theme Description:", this);
-    fThemeAuthorS = new YLabel("Theme Author:", this);
+    delete[] copyright;
+
+    fThemeNameS = new YLabel(_("Theme:"), this);
+    fThemeDescriptionS = new YLabel(_("Theme Description:"), this);
+    fThemeAuthorS = new YLabel(_("Theme Author:"), this);
     fThemeName = new YLabel(themeName, this);
     fThemeDescription = new YLabel(themeDescription, this);
     fThemeAuthor = new YLabel(themeAuthor, this);
@@ -42,7 +47,7 @@ AboutDlg::AboutDlg(): YDialog() {
     fThemeAuthorS->show();
     fThemeAuthor->show();
 
-    setWindowTitle("icewm - About");
+    setWindowTitle(_("icewm - About"));
     //setIconTitle("icewm - About");
     setWinLayerHint(WinLayerAboveDock);
     setWinStateHint(WinStateAllWorkspaces, WinStateAllWorkspaces);
