@@ -2,44 +2,45 @@
 #include "wmaction.h"
 #include "yaction.h"
 
-YAction *actionCascade = 0;
-YAction *actionArrange = 0;
-YAction *actionTileVertical = 0;
-YAction *actionTileHorizontal = 0;
-YAction *actionUndoArrange = 0;
-YAction *actionArrangeIcons = 0;
-YAction *actionMinimizeAll = 0;
-YAction *actionHideAll = 0;
+YAction *actionCascade(0);
+YAction *actionArrange(0);
+YAction *actionTileVertical(0);
+YAction *actionTileHorizontal(0);
+YAction *actionUndoArrange(0);
+YAction *actionArrangeIcons(0);
+YAction *actionMinimizeAll(0);
+YAction *actionHideAll(0);
 
 #ifndef CONFIG_PDA
-YAction *actionHide = 0;
+YAction *actionHide(0);
 #endif
-YAction *actionShow = 0;
-YAction *actionRaise = 0;
-YAction *actionLower = 0;
-YAction *actionDepth = 0;
-YAction *actionMove = 0;
-YAction *actionSize = 0;
-YAction *actionMaximize = 0;
-YAction *actionMaximizeVert = 0;
-YAction *actionMinimize = 0;
-YAction *actionRestore = 0;
-YAction *actionRollup = 0;
-YAction *actionClose = 0;
-YAction *actionKill = 0;
-YAction *actionOccupyAllOrCurrent = 0;
+YAction *actionShow(0);
+YAction *actionRaise(0);
+YAction *actionLower(0);
+YAction *actionDepth(0);
+YAction *actionMove(0);
+YAction *actionSize(0);
+YAction *actionMaximize(0);
+YAction *actionMaximizeVert(0);
+YAction *actionMinimize(0);
+YAction *actionRestore(0);
+YAction *actionRollup(0);
+YAction *actionClose(0);
+YAction *actionKill(0);
+YAction *actionOccupyAllOrCurrent(0);
+YAction *actionDoNotCover(0);
 
-YAction *actionWindowList = 0;
-YAction *actionLogout = 0;
-YAction *actionCancelLogout = 0;
-YAction *actionRefresh = 0;
+YAction *actionWindowList(0);
+YAction *actionLogout(0);
+YAction *actionCancelLogout(0);
+YAction *actionRefresh(0);
 #ifndef LITE
-YAction *actionAbout = 0;
-YAction *actionHelp = 0;
-YAction *actionLicense = 0;
+YAction *actionAbout(0);
+YAction *actionHelp(0);
+YAction *actionLicense(0);
 #endif
-YAction *actionRun = 0;
-YAction *actionExit = 0;
+YAction *actionRun(0);
+YAction *actionExit(0);
 
 void initActions() {
     actionCascade = new YAction();
@@ -67,6 +68,7 @@ void initActions() {
     actionClose = new YAction();
     actionKill = new YAction();
     actionOccupyAllOrCurrent = new YAction();
+    actionDoNotCover = new YAction();
     actionWindowList = new YAction();
     actionLogout = new YAction();
     actionCancelLogout = new YAction();
