@@ -35,15 +35,16 @@ public:
     void logout();
     void cancelLogout();
 
-#ifdef SM
+#ifdef CONFIG_SESSION
     virtual void smSaveYourself(bool shutdown, bool fast);
     virtual void smSaveYourselfPhase2();
     virtual void smShutdownCancelled();
     virtual void smDie();
 #endif
 
-    void restartClient(const char *str, char **args);
-    void runOnce(const char *resource, const char *str, char **args);
+    void restartClient(const char *str, const char **args);
+    void runOnce(const char *resource, const char *str, const char **args);
+    void runCommandOnce(const char *resource, const char *cmdline);
 
     static YCursor sizeRightPointer;
     static YCursor sizeTopRightPointer;
@@ -53,6 +54,10 @@ public:
     static YCursor sizeBottomLeftPointer;
     static YCursor sizeBottomPointer;
     static YCursor sizeBottomRightPointer;
+    static YCursor scrollLeftPointer;
+    static YCursor scrollRightPointer;
+    static YCursor scrollUpPointer;
+    static YCursor scrollDownPointer;
 
 private:
     YWindowManager *fWindowManager;
@@ -66,13 +71,20 @@ extern YMenu *windowMenu;
 extern YMenu *occupyMenu;
 extern YMenu *layerMenu;
 extern YMenu *moveMenu;
+
+#ifdef CONFIG_TRAY
+extern YMenu *trayMenu;
+#endif
+
 #ifdef CONFIG_WINMENU
 extern YMenu *windowListMenu;
 #endif
+
 #ifdef CONFIG_WINLIST
 extern YMenu *windowListPopup;
 extern YMenu *windowListAllPopup;
 #endif
+
 extern YMenu *maximizeMenu;
 extern YMenu *logoutMenu;
 
