@@ -22,7 +22,7 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win): YW
     fWindowTitle = 0;
     fIconTitle = 0;
     fColormap = None;
-    fShaped = 0;
+    fShaped = false;
     fHints = 0;
     fWinHints = 0;
     //fSavedFrameState =
@@ -47,7 +47,11 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win): YW
     getMwmHints();
 #endif
 
-#ifdef SHAPE
+#ifdef CONFIG_WM_SESSION
+    getPidHint();
+#endif
+
+#ifdef CONFIG_SHAPE
     if (shapesSupported) {
         XShapeSelectInput(app->display(), handle(), ShapeNotifyMask);
         queryShape();
@@ -236,8 +240,13 @@ void YFrameClient::constrainSize(int &w, int &h, long layer, int flags) {
 	w = clamp(w, wMin, wMax);
 
         if (limitSize) {
-            w = min(w, manager->maxWidth(layer));
-            h = min(h, manager->maxHeight(layer));
+            w = min(w, (int)(considerHorizBorder && !getFrame()->doNotCover()
+	      ? manager->maxWidth(layer) - 2 * getFrame()->borderX()
+	      : manager->maxWidth(layer)));
+            h = min(h, (int)(considerVertBorder && !getFrame()->doNotCover()
+	      ? manager->maxHeight(layer) - getFrame()->titleY()
+	      				  - 2 * getFrame()->borderY()
+	      : manager->maxHeight(layer) - getFrame()->titleY()));
         }
 
 #if 0
@@ -331,6 +340,9 @@ void YFrameClient::setFrameState(FrameState state) {
             MSG(("deleting window properties id=%lX", handle()));
             XDeleteProperty(app->display(), handle(), _XA_WIN_WORKSPACE);
             XDeleteProperty(app->display(), handle(), _XA_WIN_LAYER);
+#ifdef CONFIG_TRAY
+            XDeleteProperty(app->display(), handle(), _XA_WIN_TRAY);
+#endif	    
             XDeleteProperty(app->display(), handle(), _XA_WIN_STATE);
             XDeleteProperty(app->display(), handle(), _XA_WM_STATE);
         }
@@ -362,82 +374,92 @@ FrameState YFrameClient::getFrameState() {
 
 void YFrameClient::handleUnmap(const XUnmapEvent &unmap) {
     MSG(("Unmap: unmapped %d visible %d", unmapped(), visible()));
+
     if (!unmapped()) {
         MSG(("UnmapWindow"));
-#ifdef NEED_GRAB2
-        XGrabServer(app->display());
-        XSync(app->display(), False);
-#endif
+
         XEvent ev;
-        if (XCheckTypedWindowEvent(app->display(), unmap.window, DestroyNotify, &ev)) {
+        if (XCheckTypedWindowEvent(app->display(), unmap.window,
+				   DestroyNotify, &ev)) {
             manager->destroyedClient(unmap.window);
-            XUngrabServer(app->display());
-            return ; // gets destroyed
+            return; // gets destroyed
+        } else if (XCheckTypedWindowEvent(app->display(), unmap.window,
+					  ReparentNotify, &ev)) {
+            manager->unmanageClient(unmap.window, true, false);
+            return; // gets destroyed
         } else {
             manager->unmanageClient(unmap.window, false);
-            return ; // gets destroyed
+            return; // gets destroyed
         }
-#ifdef NEED_GRAB2
-        XUngrabServer(app->display());
-#endif
     }
+
     YWindow::handleUnmap(unmap);
 }
 
 void YFrameClient::handleProperty(const XPropertyEvent &property) {
     switch (property.atom) {
-    case XA_WM_NAME:
-        getNameHint();
-        break;
-    case XA_WM_ICON_NAME:
-        getIconNameHint();
-        break;
-    case XA_WM_CLASS:
-        getClassHint();
-        if (getFrame())
-            getFrame()->getFrameHints();
-        break;
-    case XA_WM_HINTS:
-        getWMHints();
-        break;
-    case XA_WM_NORMAL_HINTS:
-        getSizeHints();
-        if (getFrame()) {
-            getFrame()->updateMwmHints();
-            getFrame()->updateNormalSize();
-	}
-        break;
-    case XA_WM_TRANSIENT_FOR:
-        getTransient();
-        break;
-    default:
-        if (property.atom == _XA_WM_PROTOCOLS) {
-            getProtocols();
+	case XA_WM_NAME:
+            getNameHint();
+	    break;
+
+	case XA_WM_ICON_NAME:
+            getIconNameHint();
+            break;
+
+	case XA_WM_CLASS:
+	    getClassHint();
+	    if (getFrame()) getFrame()->getFrameHints();
+	    break;
+
+	case XA_WM_HINTS:
+	    getWMHints();
 #ifndef LITE
-        } else if (property.atom == _XA_KWM_WIN_ICON) {
-            if (getFrame())
-                getFrame()->updateIcon();
-        } else if (property.atom == _XA_WIN_ICONS) {
-            if (getFrame())
-                getFrame()->updateIcon();
+            if (getFrame()) getFrame()->updateIcon();
+#endif            
+	    break;
+
+	case XA_WM_NORMAL_HINTS:
+	    getSizeHints();
+	    if (getFrame()) {
+		getFrame()->updateMwmHints();
+		getFrame()->updateNormalSize();
+	    }
+	    break;
+
+	case XA_WM_TRANSIENT_FOR:
+	    getTransient();
+	    break;
+
+	default: // `extern Atom' does not reduce to an integer constant...
+	    if (_XA_WM_PROTOCOLS == property.atom) {
+		getProtocols();
+#ifndef LITE
+	    } else if (_XA_KWM_WIN_ICON == property.atom ||
+		       _XA_WIN_ICONS == property.atom) {
+		if (getFrame()) getFrame()->updateIcon();
 #endif
-        } else if (property.atom == _XA_WIN_HINTS) {
-            getWinHintsHint(&fWinHints);
-            if (getFrame()) {
-                getFrame()->getFrameHints();
+
+	    } else if (_XA_WIN_HINTS == property.atom) {
+		getWinHintsHint(&fWinHints);
+
+		if (getFrame()) {
+                    getFrame()->getFrameHints();
+                    manager->updateWorkArea();
 #ifdef CONFIG_TASKBAR
-                getFrame()->updateTaskBar();
+                    getFrame()->updateTaskBar();
 #endif
-            }
-        }
+		}
 #ifndef NO_MWM_HINTS
-        else if (property.atom == _XATOM_MWM_HINTS) {
-            getMwmHints();
-            if (getFrame())
-                getFrame()->updateMwmHints();
-        }
+	    } else if (_XATOM_MWM_HINTS == property.atom) {
+		getMwmHints();
+		if (getFrame()) getFrame()->updateMwmHints();
+		break;
 #endif
-        break;
+	    } else
+		MSG(("Unknown property changed: %s, window=0x%lX",
+		     XGetAtomName(app->display(), property.atom), handle()));
+
+	    break;
     }
 }
 
@@ -456,14 +478,14 @@ void YFrameClient::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow)
         manager->destroyedClient(destroyWindow.window);
 }
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
 void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
     if (shapesSupported) {
         MSG(("shape event: %d %d %d:%d=%dx%d time=%ld",
              shape.shaped, shape.kind,
              shape.x, shape.y, shape.width, shape.height, shape.time));
         if (shape.kind == ShapeBounding) {
-            int newShaped = shape.shaped ? 1 : 0;
+            bool const newShaped(shape.shaped);
             if (newShaped)
                 fShaped = newShaped;
             if (getFrame())
@@ -474,54 +496,50 @@ void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
 }
 #endif
 
-void YFrameClient::setWindowTitle(const char *aWindowTitle) {
-    delete fWindowTitle;
-    fWindowTitle = newstr(aWindowTitle);
-    if (getFrame())
-        getFrame()->updateTitle();
+void YFrameClient::setWindowTitle(const char *title) {
+    delete[] fWindowTitle; fWindowTitle = newstr(title);
+    if (getFrame()) getFrame()->updateTitle();
 }
 
-#ifdef I18N
-void YFrameClient::setWindowTitle(XTextProperty  *prop) {
-    Status status;
-    char **cl;
-    int n;
-    if (!prop->value || prop->encoding == XA_STRING) {
-        setWindowTitle((const char *)prop->value);
-        return;
-    }
-    status = XmbTextPropertyToTextList(app->display(), prop, &cl, &n);
-    if (status >= Success && n > 0 && cl[0]) {
-        setWindowTitle((const char *)cl[0]);
-        XFreeStringList(cl);
-    } else {
-        setWindowTitle((const char *)prop->value);
-    }
-}
-#endif
-
-void YFrameClient::setIconTitle(const char *aIconTitle) {
-    delete fIconTitle;
-    fIconTitle = newstr(aIconTitle);
-    if (getFrame())
-        getFrame()->updateIconTitle();
+void YFrameClient::setIconTitle(const char *title) {
+    delete[] fIconTitle; fIconTitle = newstr(title);
+    if (getFrame()) getFrame()->updateIconTitle();
 }
 
-#ifdef I18N
-void YFrameClient::setIconTitle(XTextProperty  *prop) {
-    Status status;
-    char **cl;
-    int n;
-    if (!prop->value || prop->encoding == XA_STRING) {
-        setIconTitle((const char *)prop->value);
-        return;
+#ifdef CONFIG_I18N
+void YFrameClient::setWindowTitle(const XTextProperty & title) {
+    if (NULL == title.value || title.encoding == XA_STRING)
+        setWindowTitle((const char *)title.value);
+    else {
+        int count;
+        char ** strings(NULL);
+
+        if (XmbTextPropertyToTextList(app->display(), &title,
+                                      &strings, &count) >= 0 &&
+            count > 0 && strings[0])
+            setWindowTitle((const char *)strings[0]);
+        else
+            setWindowTitle((const char *)title.value);
+
+        if (strings) XFreeStringList(strings);
     }
-    status = XmbTextPropertyToTextList(app->display(), prop, &cl, &n);
-    if (status >= Success && n > 0 && cl[0]) {
-        setIconTitle((const char *)cl[0]);
-        XFreeStringList(cl);
-    } else {
-        setIconTitle((const char *)prop->value);
+}
+
+void YFrameClient::setIconTitle(const XTextProperty & title) {
+    if (NULL == title.value || title.encoding == XA_STRING)
+        setIconTitle((const char *)title.value);
+    else {
+        int count;
+        char ** strings(NULL);
+
+        if (XmbTextPropertyToTextList(app->display(), &title,
+                                      &strings, &count) >= 0 &&
+            count > 0 && strings[0])
+            setIconTitle((const char *)strings[0]);
+        else
+            setIconTitle((const char *)title.value);
+
+        if (strings) XFreeStringList(strings);
     }
 }
 #endif
@@ -532,7 +550,7 @@ void YFrameClient::setColormap(Colormap cmap) {
         manager->installColormap(cmap);
 }
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
 void YFrameClient::queryShape() {
     fShaped = 0;
 
@@ -544,7 +562,7 @@ void YFrameClient::queryShape() {
         XShapeQueryExtents(app->display(), handle(),
                            &boundingShaped, &xws, &yws, &wws, &hws,
                            &clipShaped, &xbs, &ybs, &wbs, &hbs);
-        fShaped = boundingShaped ? 1 : 0;
+        fShaped = boundingShaped;
   }
 }
 #endif
@@ -573,6 +591,13 @@ void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
             getFrame()->setLayer(message.data.l[0]);
         else
             setWinLayerHint(message.data.l[0]);
+#ifdef CONFIG_TRAY	    
+    } else if (message.message_type == _XA_WIN_TRAY) {
+        if (getFrame())
+            getFrame()->setTrayOption(message.data.l[0]);
+        else
+            setWinTrayHint(message.data.l[0]);
+#endif	    
     } else if (message.message_type == _XA_WIN_STATE) {
         if (getFrame())
             getFrame()->setState(message.data.l[0], message.data.l[1]);
@@ -583,44 +608,33 @@ void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
 }
 
 void YFrameClient::getNameHint() {
-    XTextProperty prop;
-
-    if (XGetWMName(app->display(), handle(), &prop)) {
-#ifdef I18N
-        if (multiByte) {
-            setWindowTitle(&prop);
-        } else
+#ifdef CONFIG_I18N
+    XTextProperty name;
+    if (XGetWMName(app->display(), handle(), &name))
+#else    
+    char * name;
+    if (XFetchName(app->display(), handle(), &name))
 #endif
-        {
-            setWindowTitle((char *)prop.value);
-        }
-        if (prop.value) XFree(prop.value);
-    } else {
-        setWindowTitle((const char*)0);
-    }
+        setWindowTitle(name);
+    else
+        setWindowTitle(NULL);
 }
 
 void YFrameClient::getIconNameHint() {
-    XTextProperty prop;
-
-    if (XGetWMIconName(app->display(), handle(), &prop)) {
-#ifdef I18N
-        if (multiByte) {
-            setIconTitle(&prop);
-        } else
+#ifdef CONFIG_I18N
+    XTextProperty name;
+    if (XGetWMIconName(app->display(), handle(), &name))
+#else    
+    char * name;
+    if (XGetIconName(app->display(), handle(), &name))
 #endif
-        {
-            setIconTitle((char *)prop.value);
-        }
-        if (prop.value) XFree(prop.value);
-    } else {
-        setIconTitle((const char *)0);
-    }
+        setIconTitle(name);
+    else
+        setIconTitle(NULL);
 }
 
 void YFrameClient::getWMHints() {
-    if (fHints)
-        XFree(fHints);
+    if (fHints) XFree(fHints);
     fHints = XGetWMHints(app->display(), handle());
 }
 
@@ -859,6 +873,44 @@ bool YFrameClient::getWinLayerHint(long *layer) {
     return false;
 }
 
+#ifdef CONFIG_TRAY
+bool YFrameClient::getWinTrayHint(long *tray_opt) {
+    Atom r_type;
+    int r_format;
+    unsigned long count;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+    
+    if (XGetWindowProperty(app->display(),
+                           handle(),
+                           _XA_WIN_TRAY,
+                           0, 1, False, XA_CARDINAL,
+                           &r_type, &r_format,
+                           &count, &bytes_remain, &prop) == Success && prop)
+    {
+        if (r_type == XA_CARDINAL && r_format == 32 && count == 1U) {
+            long o = *(long *)prop;
+            if (o < WinTrayOptionCount) {
+                *tray_opt = o;
+                XFree(prop);
+                return true;
+            }
+        }
+        XFree(prop);
+    }
+    return false;
+}
+
+void YFrameClient::setWinTrayHint(long tray_opt) {
+    XChangeProperty(app->display(),
+                    handle(),
+                    _XA_WIN_TRAY,
+                    XA_CARDINAL,
+                    32, PropModeReplace,
+                    (unsigned char *)&tray_opt, 1);
+}
+#endif /* CONFIG_TRAY */
+
 bool YFrameClient::getWinStateHint(long *mask, long *state) {
     Atom r_type;
     int r_format;
@@ -950,6 +1002,31 @@ void YFrameClient::setWinHintsHint(long hints) {
                     (unsigned char *)&s, 1);
 }
 
+#ifdef CONFIG_WM_SESSION
+void YFrameClient::getPidHint() {
+    Atom r_type; int r_format;
+    unsigned long count, bytes_remain;
+    unsigned char *prop;
+
+    fPid = PID_MAX;
+
+    if (XGetWindowProperty(app->display(), handle(),
+    			   XA_ICEWM_PID, 0, 1, False, XA_CARDINAL,
+                           &r_type, &r_format, &count, &bytes_remain,
+			   &prop) == Success && prop) {
+	if (r_type == XA_CARDINAL && r_format == 32 && count == 1U) 
+	    fPid = *((pid_t*)prop);
+
+        XFree(prop);
+	return;
+    }
+    
+    warn(_("Window %p has no XA_ICEWM_PID property. "
+	   "Export the LD_PRELOAD variable to preload the preice library."),
+	   handle());
+}
+#endif
+
 void YFrameClient::getClientLeader() {
     Atom r_type;
     int r_format;
