@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "ykey.h"
@@ -19,98 +19,64 @@ int configurationLoaded = 0;
 #include "bindkey.h"
 #include "default.h"
 
+#include "intl.h"
+
 long workspaceCount = 0;
 char *workspaceNames[MAXWORKSPACES];
 YAction *workspaceActionActivate[MAXWORKSPACES];
 YAction *workspaceActionMoveTo[MAXWORKSPACES];
 
-int is_reg(const char *path) {
-    struct stat sb;
-
-    if (stat(path, &sb) != 0)
-        return 0;
-    if (!S_ISREG(sb.st_mode))
-        return 0;
-    return 1;
-}
-
-int findPath(const char *path, int mode, const char *name, char **fullname, bool /*path_relative*/) {
+char * findPath(const char *path, int mode, const char *name, bool /*path_relative*/) {
 #ifdef __EMX__
-    char tmp[1024];
-    strcpy(tmp, name);
+    char name_exe[1024];
+
     if (mode & X_OK)
-        strcat(tmp, ".exe");
-    name = tmp;
+	name = strcat(strcpy(name_exe, name), ".exe");
 #endif
 
-    //remove!!!
-    //fprintf(stderr, "path=%s file=%s\n", path, name);
-/*
-    if (!path_relative && (strchr(name, '/') != 0
+    if (*name == '/') { // check for root in XFreeOS/2
 #ifdef __EMX__
-        || strchr(name, '\\') != 0
-#endif
-        ) ||
-        (path_relative && name[0] == '/'
-#ifdef __EMX__
-         || // check for root
-#endif
-))
-*/
-    if (name[0] == '/') { // check for root in XFreeOS/2
-#ifdef __EMX__
-        if (access(name, 0) == 0) {
-            *fullname = newstr(name);
-            return 1;
-        }
+        if (!access(name, 0))
+            return newstr(name);
 #else
-        if (access(name, mode) == 0 && is_reg(name)) {
-            *fullname = newstr(name);
-            return 1;
-        }
+        if (!access(name, mode) && isreg(name))
+            return newstr(name);
 #endif
     } else {
-        if (path == 0)
-            return 0;
+        if (NULL == path) return NULL;
 
+        unsigned const nameLen(strlen(name));
         char prog[1024];
-        const char *p, *q;
-        unsigned int len, nameLen = strlen(name);
 
         if (nameLen > sizeof(prog))
-            return 0;
+            return NULL;
 
-        p = path;
-        while (*p) {
-            q = p;
-            while (*p && *p != PATHSEP)
-                p++;
+        for (char const *p(path), *q(path); *q; q = p) {
+            while (*p && *p != PATHSEP) p++;
 
-            len = p - q;
+            unsigned len(p - q);
+	    if (*p) ++p;
 
             if (len > 0 && len < sizeof(prog) - nameLen - 2) {
                 strncpy(prog, q, len);
+
                 if (!ISSLASH(prog[len - 1]))
                     prog[len++] = SLASH;
+
                 strcpy(prog + len, name);
 
 #ifdef __EMX__
-                if (access(prog, 0) == 0) {
-                    *fullname = newstr(prog);
-                    return 1;
-                }
+                if (!access(prog, 0))
+                    return newstr(prog);
 #else
-                if (access(prog, mode) == 0 && is_reg(prog)) {
-                    *fullname = newstr(prog);
-                    return 1;
-                }
+                if (!access(prog, mode) && isreg(prog))
+                    return newstr(prog);
 #endif
             }
-            if (*p == PATHSEP)
-                p++;
         }
     }
-    return 0;
+
+    return NULL;
 }
 
 void addWorkspace(const char *name) {
@@ -139,8 +105,7 @@ char *getArgument(char *dest, int maxLen, char *p, bool comma) {
            (in_str || (*p != ' ' && *p != '\t' && *p != '\n' && (!comma || *p != ','))))
     {
         if (in_str && *p == '\\' && p[1]) {
-            p++;
-            char c = *p++; // *++p++ doesn't work :(
+            p++; char c = *p++; // *++p++ doesn't work :(
 
             switch (c) {
             case 'a': *d++ = '\a'; break;
@@ -164,7 +129,7 @@ char *getArgument(char *dest, int maxLen, char *p, bool comma) {
 
                     int n = (a << 4) + b;
 
-                    p += 3;
+                    p += 2;
                     *d++ = (unsigned char)(n & 0xFF);
 
                     a -= '0';
@@ -243,7 +208,7 @@ bool parseKey(const char *arg, KeySym *key, unsigned int *mod) {
     }
 
     if (*key == NoSymbol) {
-        fprintf(stderr, "unknown key %s in %s\n", arg, orig_arg);
+        msg(_("Unknown key name %s in %s"), arg, orig_arg);
         return false;
     }
     return true;
@@ -252,14 +217,20 @@ bool parseKey(const char *arg, KeySym *key, unsigned int *mod) {
 char *setOption(char *name, char *arg, char *rest) {
     unsigned int a;
 
-    MSG(("SET %s := %s ;\n", name, arg));
+    MSG(("SET %s := %s ;", name, arg));
     
     for (a = 0; a < ACOUNT(bool_options); a++)
         if (strcmp(name, bool_options[a].option) == 0) {
-            if ((arg[0] == '1' || arg[0] == '0') && arg[1] == 0)
+            if ((arg[0] == '1' || arg[0] == '0') && arg[1] == 0) {
                 *(bool_options[a].value) = (arg[0] == '1') ? true : false;
-            else {
-                fprintf(stderr, "bad argument: %s for %s\n", arg, name);
+
+		// !!! dirty compatibility hack for TitleBarCentered
+		if (bool_options[a].value == &titleBarCentered) {
+		    warn(_("Obsolete option: %s"), name);
+		    titleBarJustify = (titleBarCentered ? 50 : 0);
+		}
+            } else {
+                msg(_("Bad argument: %s for %s"), arg, name);
                 return rest;
             }
             return rest;
@@ -267,12 +238,12 @@ char *setOption(char *name, char *arg, char *rest) {
 
     for (a = 0; a < ACOUNT(uint_options); a++)
         if (strcmp(name, uint_options[a].option) == 0) {
-            unsigned int v = atoi(arg);
+            int const v(atoi(arg));
             
             if (v >= uint_options[a].min && v <= uint_options[a].max)
                 *(uint_options[a].value) = v;
             else {
-                fprintf(stderr, "bad argument: %s for %s\n", arg, name);
+                msg(_("Bad argument: %s for %s"), arg, name);
                 return rest;
             }
             return rest;
@@ -345,12 +316,12 @@ char *setOption(char *name, char *arg, char *rest) {
         else
 #endif
         {
-            fprintf(stderr, "Bad Look name\n");
+            msg(_("Bad Look name"));
             return 0;
         }
         return rest;
     }
-    fprintf(stderr, "Bad option: %s\n", name);
+    msg(_("Bad option: %s"), name);
     ///!!! check
     return rest;
 }
