@@ -37,7 +37,7 @@ static bool canShutdown(bool reboot) {
             return false;
     if (logoutCommand && logoutCommand[0])
         return false;
-#ifdef SM
+#ifdef CONFIG_SESSION
     if (app->haveSessionManager())
         return false;
 #endif
@@ -125,12 +125,14 @@ CtrlAltDelete::~CtrlAltDelete() {
 }
 
 void CtrlAltDelete::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
-    g.setColor(cadBg);
+#ifdef CONFIG_GRADIENTS    
+    YSurface surface(cadBg, logoutPixmap, logoutPixbuf);
+#else
+    YSurface surface(cadBg, logoutPixmap);
+#endif
+    g.setColor(surface.color);
+    g.drawSurface(surface, 1, 1, width() - 2, height() - 2);
     g.draw3DRect(0, 0, width() - 1, height() - 1, true);
-    if (logoutPixmap)
-        g.fillPixmap(logoutPixmap, 1, 1, width() - 2, height() - 2);
-    else
-        g.fillRect(1, 1, width() - 2, height() - 2);
 }
 
 void CtrlAltDelete::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
