@@ -9,6 +9,8 @@
 #include "wmframe.h"
 
 #include "atasks.h"
+#include "atray.h"
+#include "aaddressbar.h"
 #include "wmaction.h"
 #include "wmclient.h"
 #include "wmcontainer.h"
@@ -20,6 +22,7 @@
 #include "wmwinlist.h"
 #include "wmmgr.h"
 #include "wmapp.h"
+#include "ypixbuf.h"
 #include "sysdep.h"
 
 #include "intl.h"
@@ -68,6 +71,9 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
 #ifdef CONFIG_TASKBAR
     fTaskBarApp = 0;
 #endif
+#ifdef CONFIG_TRAY
+    fTrayApp = 0;
+#endif
 #ifdef CONFIG_WINLIST
     fWinListItem = 0;
 #endif
@@ -86,7 +92,11 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
 
     fWinWorkspace = manager->activeWorkspace();
     fWinLayer = WinLayerNormal;
+#ifdef CONFIG_TRAY
+    fWinTrayOption = WinTrayIgnore;
+#endif
     fWinState = 0;
+    fWinOptionMask = ~0;
 
     createPointerWindows();
 
@@ -109,7 +119,7 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
         fMinimizeButton = 0;
     else {
         fMinimizeButton = new YFrameButton(fTitleBar, this,
-#ifndef CONFIG_PDA		    
+#ifndef CONFIG_PDA
 					   actionMinimize, actionHide);
 #else
 					   actionMinimize, 0);
@@ -128,11 +138,11 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
         fCloseButton->show();
     }
 
-#ifndef CONFIG_PDA		    
+#ifndef CONFIG_PDA
     if (!isButton('h'))
 #endif
         fHideButton = 0;
-#ifndef CONFIG_PDA		    
+#ifndef CONFIG_PDA
     else {
         fHideButton = new YFrameButton(fTitleBar, this, actionHide, actionHide);
         //fHideButton->setWinGravity(NorthEastGravity);
@@ -233,6 +243,15 @@ YFrameWindow::~YFrameWindow() {
         fTaskBarApp = 0;
     }
 #endif
+#ifdef CONFIG_TRAY
+    if (fTrayApp) {
+        if (taskBar && taskBar->trayPane())
+            taskBar->trayPane()->removeApp(this);
+        else
+            delete fTrayApp;
+        fTrayApp = 0;
+    }
+#endif
 #ifdef CONFIG_WINLIST
     if (fWinListItem) {
         if (windowList)
@@ -373,7 +392,7 @@ void YFrameWindow::manage(YFrameClient *client) {
     sendConfigure();
 }
 
-void YFrameWindow::unmanage() {
+void YFrameWindow::unmanage(bool reparent) {
     PRECONDITION(fClient != 0);
 
     if (!fClient->destroyed()) {
@@ -396,7 +415,9 @@ void YFrameWindow::unmanage() {
         else if (gy > 0)
             posY += borderY() - 2 * client()->getBorder();
 
-        client()->reparent(manager, posX, posY);
+	if (reparent)
+		client()->reparent(manager, posX, posY);
+
         client()->setSize(posWidth, posHeight);
 
         if (phase != phaseRestart)
@@ -640,9 +661,7 @@ void YFrameWindow::handleFocus(const XFocusChangeEvent &focus) {
         focus.detail != NotifyInferior &&
         focus.detail != NotifyPointer &&
         focus.detail != NotifyPointerRoot)
-    {
         manager->switchFocusTo(this);
-    }
 #endif
 #if 0
     else if (focus.type == FocusOut &&
@@ -651,10 +670,10 @@ void YFrameWindow::handleFocus(const XFocusChangeEvent &focus) {
                focus.detail != NotifyPointer &&
                focus.detail != NotifyPointerRoot &&
                focus.window == handle())
-    {
         manager->switchFocusFrom(this);
-    }
 #endif
+
+    layoutShape();
 }
 
 bool YFrameWindow::handleTimer(YTimer *t) {
@@ -888,11 +907,11 @@ void YFrameWindow::actionPerformed(YAction *action, unsigned int modifiers) {
             wmClose();
     } else if (action == actionKill) {
         wmConfirmKill();
-#ifndef CONFIG_PDA		    
+#ifndef CONFIG_PDA
     } else if (action == actionHide) {
         if (canHide())
             wmHide();
-#endif	    
+#endif
     } else if (action == actionShow) {
         wmShow();
     } else if (action == actionMove) {
@@ -903,19 +922,29 @@ void YFrameWindow::actionPerformed(YAction *action, unsigned int modifiers) {
             wmSize();
     } else if (action == actionOccupyAllOrCurrent) {
         wmOccupyAllOrCurrent();
+    } else if (action == actionDoNotCover) {
+        wmToggleDoNotCover();
     } else {
-        for (int l = 0; l < WinLayerCount; l++) {
+        for (int l(0); l < WinLayerCount; l++) {
             if (action == layerActionSet[l]) {
                 wmSetLayer(l);
                 return ;
             }
         }
-        for (int w = 0; w < workspaceCount; w++) {
+        for (int w(0); w < workspaceCount; w++) {
             if (action == workspaceActionMoveTo[w]) {
                 wmMoveToWorkspace(w);
                 return ;
             }
         }
+#ifdef CONFIG_TRAY
+        for (int o(0); o < WinTrayOptionCount; o++) {
+            if (action == trayOptionActionSet[o]) {
+                wmSetTrayOption(o);
+                return;
+            }
+        }
+#endif
         wmapp->actionPerformed(action, modifiers);
     }
 }
@@ -924,6 +953,16 @@ void YFrameWindow::wmSetLayer(long layer) {
     setLayer(layer);
 }
 
+#ifdef CONFIG_TRAY
+void YFrameWindow::wmSetTrayOption(long option) {
+    setTrayOption(option);
+}
+#endif
+
+void YFrameWindow::wmToggleDoNotCover() {
+    setDoNotCover(!doNotCover());
+}
+
 void YFrameWindow::wmMove() {
     Window root, child;
     int rx, ry, wx, wy;
@@ -1173,7 +1212,7 @@ void YFrameWindow::wmClose() {
         wmConfirmKill();
     }
     XUngrabServer(app->display());
-#endif    
+#endif
 }
 
 void YFrameWindow::wmConfirmKill() {
@@ -1185,7 +1224,8 @@ void YFrameWindow::wmConfirmKill() {
 
         msgbox->setTitle(title);
         delete title; title = 0;
-        msgbox->setText(_("Warning! Unsaved changes will be lost!\nProceed?"));
+        msgbox->setText(_("WARNING! All unsaved changes will be lost when\n"
+			  "this client is killed. Do you wish to proceed?"));
         msgbox->autoSize();
         msgbox->setMsgBoxListener(this);
         msgbox->showFocused();
@@ -1341,6 +1381,10 @@ void YFrameWindow::focus(bool canWarp) {
 void YFrameWindow::activate(bool canWarp) {
     if (fWinState & (WinStateHidden | WinStateMinimized))
         setState(WinStateHidden | WinStateMinimized, 0);
+
+#ifdef CONFIG_WM_SESSION
+    manager->setTopLevelProcess(client()->pid());
+#endif
     focus(canWarp);
 }
 
@@ -1390,8 +1434,8 @@ void YFrameWindow::paint(Graphics &g, int , int , unsigned int , unsigned int )
 
 #ifdef CONFIG_LOOK_MOTIF
         if (wmLook == lookMotif && canSize()) {
-            YColor *b = bg->brighter();
-            YColor *d = bg->darker();
+            YColor *b(bg->brighter());
+            YColor *d(bg->darker());
 
 
             g.setColor(d);
@@ -1416,23 +1460,71 @@ void YFrameWindow::paint(Graphics &g, int , int , unsigned int , unsigned int )
             int n = focused() ? 1 : 0;
             int t = (frameDecors() & fdResize) ? 0 : 1;
 
-            if (frameTL[t][n] &&
-                frameT[t][n] &&
-                frameTR[t][n] &&
-                frameL[t][n] &&
-                frameR[t][n] &&
-                frameBL[t][n] &&
-                frameB[t][n] &&
-                frameBR[t][n])
-            {
-                g.drawPixmap(frameTL[t][n], 0, 0);
-                g.repHorz(frameT[t][n], wsCornerX, 0, width() - 2 * wsCornerX);
-                g.drawPixmap(frameTR[t][n], width() - wsCornerX, 0);
-                g.repVert(frameL[t][n], 0, wsCornerY, height() - 2 * wsCornerY);
-                g.repVert(frameR[t][n], width() - borderX(), wsCornerY, height() - 2 * wsCornerY);
-                g.drawPixmap(frameBL[t][n], 0, height() - wsCornerY);
-                g.repHorz(frameB[t][n], wsCornerX, height() - borderY(), width() - 2 * wsCornerX);
-                g.drawPixmap(frameBR[t][n], width() - wsCornerX, height() - wsCornerY);
+            if ((frameT[t][n] || TEST_GRADIENT(rgbFrameT[t][n])) &&
+		(frameL[t][n] || TEST_GRADIENT(rgbFrameL[t][n])) &&
+		(frameR[t][n] || TEST_GRADIENT(rgbFrameR[t][n])) &&
+		(frameB[t][n] || TEST_GRADIENT(rgbFrameB[t][n])) &&
+		frameTL[t][n] && frameTR[t][n] &&
+		frameBL[t][n] && frameBR[t][n]) {
+		unsigned const xtl(frameTL[t][n]->width());
+		unsigned const ytl(frameTL[t][n]->height());
+		unsigned const xtr(frameTR[t][n]->width());
+		unsigned const ytr(frameTR[t][n]->height());
+		unsigned const xbl(frameBL[t][n]->width());
+		unsigned const ybl(frameBL[t][n]->height());
+		unsigned const xbr(frameBR[t][n]->width());
+		unsigned const ybr(frameBR[t][n]->height());
+
+		unsigned const cx(width()/2);
+		unsigned const cy(height()/2);
+
+		g.copyPixmap(frameTL[t][n], 0, 0,
+			     min(xtl, cx), min(ytl, cy), 0, 0);
+		g.copyPixmap(frameTR[t][n], max(0, (int)xtr - (int)cx), 0,
+			     min(xtr, cx), min(ytr, cy),
+			     width() - min(xtr, cx), 0);
+		g.copyPixmap(frameBL[t][n], 0, max(0, (int)ybl - (int)cy),
+			     min(xbl, cx), min(ybl, cy),
+			     0, height() - min(ybl, cy));
+		g.copyPixmap(frameBR[t][n],
+			     max(0, (int)xbr - (int)cx), max(0, (int)ybr - (int)cy),
+			     min(xbr, cx), min(ybr, cy),
+			     width() - min(xbr, cx), height() - min(ybr, cy));
+
+		if (width() > (xtl + xtr))
+		    if (frameT[t][n]) g.repHorz(frameT[t][n],
+			xtl, 0, width() - xtl - xtr);
+#ifdef CONFIG_GRADIENTS
+		    else g.drawGradient(*rgbFrameT[t][n],
+			xtl, 0, width() - xtl - xtr, borderY());
+#endif
+
+		if (height() > (ytl + ybl))
+		    if (frameL[t][n]) g.repVert(frameL[t][n],
+			0, ytl, height() - ytl - ybl);
+#ifdef CONFIG_GRADIENTS
+		    else g.drawGradient(*rgbFrameL[t][n],
+			0, ytl, borderX(), height() - ytl - ybl);
+#endif
+
+		if (height() > (ytr + ybr))
+		    if (frameR[t][n]) g.repVert(frameR[t][n],
+			width() - borderX(), ytr, height() - ytr - ybr);
+#ifdef CONFIG_GRADIENTS
+		    else g.drawGradient(*rgbFrameR[t][n],
+			width() - borderX(), ytr,
+			borderX(), height() - ytr - ybr);
+#endif
+
+		if (width() > (xbl + xbr))
+		    if (frameB[t][n]) g.repHorz(frameB[t][n],
+			xbl, height() - borderY(), width() - xbl - xbr);
+#ifdef CONFIG_GRADIENTS
+		    else g.drawGradient(*rgbFrameB[t][n],
+			xbl, height() - borderY(),
+			width() - xbl - xbr, borderY());
+#endif
+
             } else {
                 g.fillRect(1, 1, width() - 3, height() - 3);
                 g.drawBorderW(0, 0, width() - 1, height() - 1, true);
@@ -1485,16 +1577,19 @@ void YFrameWindow::popupSystemMenu(int x, int y,
 
 void YFrameWindow::updateTitle() {
     titlebar()->repaint();
+    layoutShape();
     updateIconTitle();
 #ifdef CONFIG_WINLIST
-    if (fWinListItem) {
+    if (fWinListItem && windowList->visible())
         windowList->repaintItem(fWinListItem);
-    }
 #endif
 #ifdef CONFIG_TASKBAR
-    if (fTaskBarApp) {
+    if (fTaskBarApp)
         fTaskBarApp->setToolTip((const char *)client()->windowTitle());
-    }
+#endif
+#ifdef CONFIG_TRAY
+    if (fTrayApp)
+        fTrayApp->setToolTip((const char *)client()->windowTitle());
 #endif
 }
 
@@ -1504,6 +1599,10 @@ void YFrameWindow::updateIconTitle() {
         fTaskBarApp->repaint();
         fTaskBarApp->setToolTip((const char *)client()->windowTitle());
     }
+#endif
+#ifdef CONFIG_TRAY
+    if (fTrayApp)
+        fTrayApp->setToolTip((const char *)client()->windowTitle());
 #endif
     if (isIconic()) {
         fMiniIcon->repaint();
@@ -1521,6 +1620,10 @@ void YFrameWindow::wmOccupyAllOrCurrent() {
     if (taskBar && taskBar->taskPane())
         taskBar->taskPane()->relayout();
 #endif
+#ifdef CONFIG_TRAY
+    if (taskBar && taskBar->trayPane())
+        taskBar->trayPane()->relayout();
+#endif
 }
 
 void YFrameWindow::wmOccupyAll() {
@@ -1531,6 +1634,10 @@ void YFrameWindow::wmOccupyAll() {
     if (taskBar && taskBar->taskPane())
         taskBar->taskPane()->relayout();
 #endif
+#ifdef CONFIG_TRAY
+    if (taskBar && taskBar->trayPane())
+        taskBar->trayPane()->relayout();
+#endif
 }
 
 void YFrameWindow::wmOccupyWorkspace(long workspace) {
@@ -1627,8 +1734,8 @@ void YFrameWindow::getFrameHints() {
     fFrameFunctions |= wo.functions;
     fFrameDecors &= ~wo.decor_mask;
     fFrameDecors |= wo.decors;
-    fFrameOptions &= ~wo.option_mask;
-    fFrameOptions |= wo.options;
+    fFrameOptions &= ~(wo.option_mask & fWinOptionMask);
+    fFrameOptions |= (wo.options & fWinOptionMask);
 #endif
 }
 
@@ -1636,21 +1743,23 @@ void YFrameWindow::getFrameHints() {
 
 void YFrameWindow::getWindowOptions(WindowOption &opt, bool remove) {
     memset((void *)&opt, 0, sizeof(opt));
+
     opt.workspace = WinWorkspaceInvalid;
     opt.layer = WinLayerInvalid;
+#ifdef CONFIG_TRAY
+    opt.tray = WinTrayInvalid;
+#endif
 
-    if (defOptions)
-        getWindowOptions(defOptions, opt, false);
-    if (hintOptions)
-        getWindowOptions(hintOptions, opt, remove);
+    if (defOptions) getWindowOptions(defOptions, opt, false);
+    if (hintOptions) getWindowOptions(hintOptions, opt, remove);
 }
 
-void YFrameWindow::getWindowOptions(WindowOptions *list, WindowOption &opt, bool remove) {
-    XClassHint *h = client()->classHint();
+void YFrameWindow::getWindowOptions(WindowOptions *list, WindowOption &opt,
+                                    bool remove) {
+    XClassHint const *h(client()->classHint());
     WindowOption *wo;
 
-    if (!h)
-        return;
+    if (!h) return;
 
     if (h->res_name && h->res_class) {
         char *both = new char[strlen(h->res_name) + 1 +
@@ -1662,7 +1771,7 @@ void YFrameWindow::getWindowOptions(WindowOptions *list, WindowOption &opt, bool
         }
         wo = both ? list->getWindowOption(both, false, remove) : 0;
         if (wo) WindowOptions::combineOptions(opt, *wo);
-        delete both;
+        delete[] both;
     }
     if (h->res_class) {
         wo = list->getWindowOption(h->res_class, false, remove);
@@ -1684,8 +1793,7 @@ void YFrameWindow::getDefaultOptions() {
 
     if (wo.icon) {
 #ifndef LITE
-        if (fFrameIcon)
-            delete fFrameIcon;
+        if (fFrameIcon) delete fFrameIcon;
         fFrameIcon = ::getIcon(wo.icon);
 #endif
     }
@@ -1693,29 +1801,30 @@ void YFrameWindow::getDefaultOptions() {
         setWorkspace(wo.workspace);
     if (wo.layer != (long)WinLayerInvalid && wo.layer < WinLayerCount)
         setLayer(wo.layer);
+#ifdef CONFIG_TRAY
+    if (wo.tray != (long)WinTrayInvalid && wo.tray < WinTrayOptionCount)
+        setTrayOption(wo.tray);
+#endif
 #endif
 }
 
 #ifndef LITE
-YIcon *newClientIcon(int count, int reclen, long *elem) {
-    int i;
-    YPixmap *small = 0, *large = 0;
-
+YIcon *newClientIcon(int count, int reclen, long * elem) {
+    YIcon::Image * small(NULL), * large(NULL), * huge(NULL);
     if (reclen < 2)
         return 0;
 
-    for (i = 0; i < count; i++, elem += reclen) {
-        Pixmap pixmap, mask;
+    for (int i(0); i < count; i++, elem += reclen) {
+        Pixmap pixmap(elem[0]), mask(elem[1]);
 
-        pixmap = elem[0];
-        mask = elem[1];
-
-        if (pixmap == None)
+        if (pixmap == None) {
+            warn("pixmap == None for subicon #%d", i);
             continue;
+        }
 
-        int x, y;
         Window root;
-        unsigned int w = 0, h = 0, border, depth = 0;
+        int x, y;
+        unsigned w(0), h(0), border, depth(0);
 
         if (reclen >= 6) {
             w = elem[2];
@@ -1723,32 +1832,30 @@ YIcon *newClientIcon(int count, int reclen, long *elem) {
             depth = elem[4];
             root = elem[5];
         } else {
-            if (XGetGeometry(app->display(), pixmap,
-                             &root, &x, &y, &w, &h, &border, &depth) == BadDrawable)
+            if (BadDrawable == XGetGeometry(app->display(), pixmap,
+                                            &root, &x, &y, &w, &h,
+                                            &border, &depth)) {
+                warn("BadDrawable for subicon #%d", i);
                 continue;
+            }
         }
 
-        if (w != h || w == 0 || h == 0)
+        if (w != h || w == 0 || h == 0) {
+            MSG(("Invalid pixmap size for subicon #%d: %dx%d", i, w, h));
             continue;
+        }
 
-        if (depth == (unsigned)DefaultDepth(app->display(),
-                                            DefaultScreen(app->display())))
-        {
-#ifdef CONFIG_IMLIB
-            small = new YPixmap(pixmap, mask, w, h, ICON_SMALL, ICON_SMALL);
-	    large = new YPixmap(pixmap, mask, w, h, ICON_LARGE, ICON_LARGE);
-#else
-            if (w == ICON_SMALL)
-                small = new YPixmap(pixmap, mask, w, h);
-            else if (w == ICON_LARGE)
-                large = new YPixmap(pixmap, mask, w, h);
-#endif
+        if (depth == app->depth()) {
+            if (w == YIcon::sizeSmall)
+                small = new YIcon::Image(pixmap, mask, w, h);
+            else if (w == YIcon::sizeLarge)
+                large = new YIcon::Image(pixmap, mask, w, h);
+            else if (w == YIcon::sizeHuge)
+                huge = new YIcon::Image(pixmap, mask, w, h);
         }
     }
-    if (small || large)
-        return new YIcon(small, large);
-    else
-        return 0;
+
+    return (small || large || huge ? new YIcon(small, large, huge) : 0);
 }
 
 void YFrameWindow::updateIcon() {
@@ -1757,11 +1864,13 @@ void YFrameWindow::updateIcon() {
     Pixmap *pixmap;
     Atom type;
 
+    YIcon *oldFrameIcon(fFrameIcon);
+
     if (client()->getWinIcons(&type, &count, &elem)) {
         if (type == _XA_WIN_ICONS)
             fFrameIcon = newClientIcon(elem[0], elem[1], elem + 2);
         else // compatibility
-            fFrameIcon = newClientIcon(count, 2, elem);
+            fFrameIcon = newClientIcon(count/2, 2, elem);
         XFree(elem);
     } else if (client()->getKwmIcon(&count, &pixmap) && count == 2) {
         XWMHints *h = client()->hints();
@@ -1774,9 +1883,6 @@ void YFrameWindow::updateIcon() {
             fFrameIcon = newClientIcon(2, 2, pix);
         } else {
             long pix[2];
-            for (int i = 0; i < count; i++) {
-                pix[i] = pixmap[i];
-            }
             pix[0] = pixmap[0];
             pix[1] = pixmap[1];
             fFrameIcon = newClientIcon(count / 2, 2, pix);
@@ -1784,27 +1890,37 @@ void YFrameWindow::updateIcon() {
         XFree(pixmap);
     } else {
         XWMHints *h = client()->hints();
-        if (h && (h->flags & IconPixmapHint) && (h->flags & IconMaskHint)) {
+        if (h && (h->flags & IconPixmapHint)) {
             long pix[2];
             pix[0] = h->icon_pixmap;
             pix[1] = (h->flags & IconMaskHint) ? h->icon_mask : None;
             fFrameIcon = newClientIcon(1, 2, pix);
         }
     }
-    if (fFrameIcon) {
-        if (fFrameIcon->small() == 0 &&
-            fFrameIcon->large() == 0)
-        {
-            delete fFrameIcon; fFrameIcon = 0;
-        }
+
+    if (fFrameIcon && !(fFrameIcon->small() || fFrameIcon->large())) {
+	delete fFrameIcon;
+	fFrameIcon = NULL;
     }
+
+    if (NULL == fFrameIcon) fFrameIcon = oldFrameIcon;
+    else if (oldFrameIcon != fFrameIcon) delete oldFrameIcon;
+
+// !!! BAH, we need an internal signaling framework
+    if (menuButton()) menuButton()->repaint();
+    if (getMiniIcon()) getMiniIcon()->repaint();
+    if (fTrayApp) fTrayApp->repaint();
+    if (fTaskBarApp) fTaskBarApp->repaint();
+    if (windowList && fWinListItem)
+    	windowList->getList()->repaintItem(fWinListItem);
+
 }
 #endif
 
 YFrameWindow *YFrameWindow::nextLayer() {
     if (fNextFrame) return fNextFrame;
-	
-    for (long l(getLayer() - 1); l > 0; --l)
+
+    for (long l(getLayer() - 1); l > -1; --l)
         if (manager->top(l)) return manager->top(l);
 
     return 0;
@@ -1812,7 +1928,7 @@ YFrameWindow *YFrameWindow::nextLayer() {
 
 YFrameWindow *YFrameWindow::prevLayer() {
     if (fPrevFrame) return fPrevFrame;
-    
+
     for (long l(getLayer() + 1); l < WinLayerCount; ++l)
         if (manager->bottom(l)) return manager->bottom(l);
 
@@ -1828,7 +1944,7 @@ YMenu *YFrameWindow::windowMenu() {
 
 void YFrameWindow::addAsTransient() {
     Window groupLeader(client()->ownerWindow());
-    
+
     if (groupLeader) {
         fOwner = manager->findFrame(groupLeader);
 
@@ -1923,7 +2039,7 @@ bool YFrameWindow::isFocusable() {
         return true;
     if (hints->input)
         return true;
-#if 0
+#if 1
     if (client()->protocols() & YFrameClient::wpTakeFocus)
         return true;
 #endif
@@ -1948,6 +2064,7 @@ void YFrameWindow::setWorkspace(long workspace) {
 void YFrameWindow::setLayer(long layer) {
     if (layer >= WinLayerCount || layer < 0)
         return ;
+
     if (layer != fWinLayer) {
         long oldLayer = fWinLayer;
 
@@ -1964,6 +2081,20 @@ void YFrameWindow::setLayer(long layer) {
     }
 }
 
+#ifdef CONFIG_TRAY
+void YFrameWindow::setTrayOption(long option) {
+    if (option >= WinTrayOptionCount || option < 0)
+        return ;
+    if (option != fWinTrayOption) {
+        fWinTrayOption = option;
+        client()->setWinTrayHint(fWinTrayOption);
+#ifdef CONFIG_TASKBAR
+        updateTaskBar();
+#endif
+    }
+}
+#endif
+
 void YFrameWindow::updateState() {
     if (!isManaged())
         return ;
@@ -1985,18 +2116,10 @@ void YFrameWindow::updateState() {
         newState = IconicState;
     } else if (!visibleNow()) {
         show_frame = false;
-        if (isMinimized() || isIconic()) {
-            show_client = false;
-            newState = IconicState;
-        } else {
-            show_client = false;
-            newState = NormalState; // ?
-        }
+        show_client = false;
+        newState = isMinimized() || isIconic() ? IconicState : NormalState;
     } else if (isMinimized()) {
-        if (minimizeToDesktop)
-            show_frame = true;
-        else
-            show_frame = false;
+        show_frame = minimizeToDesktop;
         show_client = false;
         newState = IconicState;
     } else if (isRollup()) {
@@ -2011,20 +2134,20 @@ void YFrameWindow::updateState() {
     MSG(("updateState: winState=%lX, frame=%d, client=%d",
          fWinState, show_frame, show_client));
 
+    client()->setFrameState(newState);
+
     if (show_client) {
         client()->show();
         fClientContainer->show();
     }
-    if (show_frame)
-        show();
-    else
-        hide();
+
+    if (show_frame) show();
+    else hide();
+
     if (!show_client) {
         fClientContainer->hide();
         client()->hide();
     }
-
-    client()->setFrameState(newState);
 }
 
 void YFrameWindow::getNormalGeometry(int *x, int *y, int *w, int *h) {
@@ -2089,7 +2212,7 @@ void YFrameWindow::updateNormalSize() {
             normalHeight = sh ? (nh - sh->base_height) / sh->height_inc : nh;
     }
 
-    MSG(("updateNormalSize: (%d:%d %dx%d) icon (%d:%d)", 
+    MSG(("updateNormalSize: (%d:%d %dx%d) icon (%d:%d)",
     	 normalX, normalY, normalWidth, normalHeight, iconX, iconY));
 }
 
@@ -2110,38 +2233,55 @@ void YFrameWindow::updateLayout() {
 	int nh(sh ? normalHeight * sh->height_inc + sh->base_height
 		  : normalHeight);
 
-        if (isMaximizedHoriz())
-            nw = manager->maxWidth(getLayer());
-
-        if (isMaximizedVert())
-            nh = manager->maxHeight(getLayer()) - titleY();
+	int const maxWidth(manager->maxWidth(this));
+	int const maxHeight(manager->maxHeight(this) - titleY());
 
+        if (isMaximizedHoriz()) nw = maxWidth;
+        if (isMaximizedVert()) nh = maxHeight;
+/*
 	if (!doNotCover()) {
 	    nx = min(nx, manager->maxX(getLayer()) - nw);
 	    nx = max(nx, manager->minX(getLayer()));
-	    nw = min(nw, manager->maxX(getLayer()) - nx);
+	    nw = min(nw, manager->maxX(getLayer()) -
+                 (considerHorizBorder ? nx + 2 * (int) borderX() : nx));
 
-	    ny = min(ny, manager->maxY(getLayer()) - nh - int(titleY()));
+	    ny = min(ny, manager->maxY(getLayer()) - (int)titleY() - nh);
 	    ny = max(ny, manager->minY(getLayer()));
-	    nh = min(nh, manager->maxY(getLayer()) - ny - int(titleY()));
+	    nh = min(nh, manager->maxY(getLayer()) - (int)titleY() -
+                 (considerVertBorder ? ny + 2 * (int) borderY() : ny));
 	}
-
+*/
         client()->constrainSize(nw, nh, getLayer());
 
-        if (isRollup())
-            nh = 0;
+	if (!isMaximizedHoriz()) {
+            nx-= borderX();
+            nw+= 2 * borderX();
+        } else {
+	    nx = manager->minX(this);
 
-	if (!(isMaximizedHoriz() && considerHorizBorder)) {
-	    nx -= borderX();
-	    nw += 2 * borderX();
-	}
+            if (!considerHorizBorder) nw+= 2 * borderX();
+            if (centerMaximizedWindows && !(sh && (sh->flags & PMaxSize)))
+                nx+= (maxWidth - nw) / 2;
+            else if (!considerHorizBorder)
+                nx-= borderX();
+        }
 
-	bool const cv(!(isMaximizedVert() && considerVertBorder));
-	bool const ch(cv || isRollup());
+	if (!isMaximizedVert()) {
+	    ny-= borderY();
+            nh+= 2 * borderY();
+        } else {
+	    ny = manager->minY(this);
+
+	    if (!considerVertBorder) nh+= 2 * borderY();
+            if (centerMaximizedWindows && !(sh && (sh->flags & PMaxSize)))
+                ny+= (maxHeight - nh) / 2;
+            else if (!considerVertBorder)
+	        ny-= borderY();
+
+        }
+
+        if (isRollup()) nh = 2 * borderY();
 
-	if (cv) ny -= borderY();
-	if (ch) nh += 2 * borderY();
-	
 	setGeometry(nx, ny, nw, nh + titleY());
     }
 }
@@ -2239,7 +2379,7 @@ void YFrameWindow::setState(long mask, long state) {
     updateState();
     updateLayout();
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     if ((fOldState ^ fNewState) & (WinStateRollup | WinStateMinimized))
         setShape();
 #endif
@@ -2260,6 +2400,25 @@ void YFrameWindow::setState(long mask, long state) {
 
 void YFrameWindow::setSticky(bool sticky) {
     setState(WinStateAllWorkspaces, sticky ? WinStateAllWorkspaces : 0);
+
+    if (doNotCover())
+	manager->updateWorkArea();
+}
+
+void YFrameWindow::setDoNotCover(bool doNotCover) {
+    long winHints = client()->winHints();
+    fWinOptionMask&= ~foDoNotCover;
+
+    if (doNotCover) {
+	fFrameOptions|= foDoNotCover;
+	winHints|= WinHintsDoNotCover;
+    } else {
+	fFrameOptions&= ~foDoNotCover;
+	winHints&= ~WinHintsDoNotCover;
+    }
+
+    client()->setWinHintsHint(winHints);
+    manager->updateWorkArea();
 }
 
 void YFrameWindow::updateMwmHints() {
@@ -2288,13 +2447,11 @@ void YFrameWindow::updateMwmHints() {
 }
 
 #ifndef LITE
-YIcon *YFrameWindow::clientIcon() {
-    YFrameWindow *f = this;
-    while (f) {
+YIcon *YFrameWindow::clientIcon() const {
+    for(YFrameWindow const *f(this); f != NULL; f = f->owner())
         if (f->getClientIcon())
             return f->getClientIcon();
-        f = f->owner();
-    }
+
     return defaultAppIcon;
 }
 #endif
@@ -2302,16 +2459,56 @@ YIcon *YFrameWindow::clientIcon() {
 void YFrameWindow::updateProperties() {
     client()->setWinWorkspaceHint(fWinWorkspace);
     client()->setWinLayerHint(fWinLayer);
+#ifdef CONFIG_TRAY
+    client()->setWinTrayHint(fWinTrayOption);
+#endif
     client()->setWinStateHint(WIN_STATE_ALL, fWinState);
 }
 
 #ifdef CONFIG_TASKBAR
 void YFrameWindow::updateTaskBar() {
-    bool needTaskBarApp = false;
+#ifdef CONFIG_TRAY
+    bool needTrayApp(false);
+    int dw(0);
+
+    if (taskBar && fManaged && taskBar->trayPane()) {
+        if (!isHidden() &&
+            !(frameOptions() & foIgnoreTaskBar) &&
+	    (getTrayOption() != WinTrayIgnore))
+            if (trayShowAllWindows || visibleOn(manager->activeWorkspace()))
+                needTrayApp = true;
+
+        if (needTrayApp && fTrayApp == 0)
+            fTrayApp = taskBar->trayPane()->addApp(this);
+
+        if (fTrayApp) {
+            fTrayApp->setShown(needTrayApp);
+            if (fTrayApp->getShown()) ///!!! optimize
+                fTrayApp->repaint();
+        }
+        /// !!! optimize
+
+        TrayPane *tp = taskBar->trayPane();
+	int const nw(tp->getRequiredWidth());
+
+        if ((dw = nw - tp->width()))
+            taskBar->trayPane()->setGeometry
+		(tp->x() - dw, tp->y(), nw, tp->height());
+
+        taskBar->trayPane()->relayout();
+    }
+#endif
+
+    bool needTaskBarApp(false);
 
     if (taskBar && fManaged && taskBar->taskPane()) {
-        if (!isHidden() &&
-            !(frameOptions() & foIgnoreTaskBar))
+#ifndef CONFIG_TRAY
+        if (!(isHidden() || (frameOptions() & foIgnoreTaskBar))
+#else
+        if (!(isHidden() || (frameOptions() & foIgnoreTaskBar)) &&
+            (getTrayOption() == WinTrayIgnore ||
+            (getTrayOption() == WinTrayMinimized && !isMinimized())))
+#endif
             if (taskBarShowAllWindows || visibleOn(manager->activeWorkspace()))
                 needTaskBarApp = true;
 
@@ -2325,8 +2522,17 @@ void YFrameWindow::updateTaskBar() {
         }
         /// !!! optimize
 
+#ifdef CONFIG_TRAY
+	if (dw) taskBar->taskPane()->setSize
+	    (taskBar->taskPane()->width() - dw, taskBar->taskPane()->height());
+#endif
         taskBar->taskPane()->relayout();
     }
+
+    if (dw && NULL == taskBar->taskPane() && NULL != taskBar->addressBar())
+	taskBar->addressBar()->setSize
+	    (taskBar->addressBar()->width() - dw,
+	     taskBar->addressBar()->height());
 }
 #endif
 
