@@ -16,6 +16,7 @@
 class YClientContainer;
 class MiniIcon;
 class TaskBarApp;
+class TrayApp;
 class YFrameTitleBar;
 
 class YFrameWindow: public YWindow, public YActionListener, public YTimerListener, public PopDownListener, public YMsgBoxListener, public ClientData {
@@ -24,7 +25,7 @@ public:
     virtual ~YFrameWindow();
 
     void manage(YFrameClient *client);
-    void unmanage();
+    void unmanage(bool reparent = true);
     void sendConfigure();
 
     void createPointerWindows();
@@ -34,8 +35,7 @@ public:
     void activate(bool canWarp = false);
 
     void activateWindow(bool raise) {
-        if (raise)
-            wmRaise();
+        if (raise) wmRaise();
         activate(true);
     }
 
@@ -83,6 +83,10 @@ public:
     void wmOccupyOnlyWorkspace(long workspace);
     void wmMoveToWorkspace(long workspace);
     void wmSetLayer(long layer);
+#ifdef CONFIG_TRAY
+    void wmSetTrayOption(long option);
+#endif
+    void wmToggleDoNotCover();
 
     void minimizeTransients();
     void restoreMinimizedTransients();
@@ -111,7 +115,7 @@ public:
                 int &flags);
     void snapTo(int &wx, int &wy);
 
-    void drawOutline(int x, int y, int w, int h);
+    void drawMoveSizeFX(int x, int y, int w, int h, bool interior = true);
     int handleMoveKeys(const XKeyEvent &xev, int &newX, int &newY);
     int handleResizeKeys(const XKeyEvent &key,
                          int &newX, int &newY, int &newWidth, int &newHeight,
@@ -185,12 +189,14 @@ public:
     virtual void popupSystemMenu(void);
     virtual void handlePopDown(YPopupWindow *popup);
 
-    virtual void configure(int x, int y, unsigned int width, unsigned int height);
+    virtual void configure(const int x, const int y, 
+    			   const unsigned width, const unsigned height,
+			   const bool resized);
     
     void configureClient(const XConfigureRequestEvent &configureRequest);
     void configureClient(int cx, int cy, int cwidth, int cheight);
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     void setShape();
 #endif
 
@@ -257,13 +263,12 @@ public:
     void layoutTitleBar();
     void layoutButtons();
     void layoutResizeIndicators();
+    void layoutShape();
     void layoutClient();
 
     //void workspaceShow();
     //void workspaceHide();
 
-    void addToMenu(YMenu *menu);
-
     YFrameWindow *nextLayer();
     YFrameWindow *prevLayer();
 #ifdef CONFIG_WINLIST
@@ -285,7 +290,7 @@ public:
 
 #ifndef LITE
     YIcon *getClientIcon() const { return fFrameIcon; }
-    YIcon *clientIcon();
+    YIcon *clientIcon() const;
 #endif
 
     void getNormalGeometry(int *x, int *y, int *w, int *h);
@@ -310,6 +315,11 @@ public:
     void setWorkspace(long workspace);
     long getLayer() const { return fWinLayer; }
     void setLayer(long layer);
+#ifdef CONFIG_TRAY
+    long getTrayOption() const { return fWinTrayOption; }
+    void setTrayOption(long option);
+#endif
+    void setDoNotCover(bool flag);
     long getState() const { return fWinState; }
     void setState(long mask, long state);
 
@@ -338,24 +348,25 @@ public:
 
     void setSticky(bool sticky);
 
-    int visibleOn(long workspace) {
+    bool visibleOn(long workspace) const {
         return (isSticky() || getWorkspace() == workspace);
     }
-    int visibleNow() { return visibleOn(manager->activeWorkspace()); }
+    bool visibleNow() const { return visibleOn(manager->activeWorkspace()); }
 
     bool isModal();
     bool hasModal();
     bool isFocusable();
 
-    bool doNotCover() { return limitByDockLayer
-    			     ? getLayer() == WinLayerDock
-    			     : frameOptions() & foDoNotCover; }
+    bool doNotCover() const { return limitByDockLayer
+    			      ? getLayer() == WinLayerDock
+    			      : frameOptions() & foDoNotCover; }
 
 #ifndef LITE
-    virtual YIcon *getIcon() { return clientIcon(); }
+    virtual YIcon *getIcon() const { return clientIcon(); }
 #endif
-    virtual const char *getTitle() { return client()->windowTitle(); }
-    virtual const char *getIconTitle() { return client()->iconTitle(); }
+
+    virtual const char *getTitle() const { return client()->windowTitle(); }
+    virtual const char *getIconTitle() const { return client()->iconTitle(); }
 
     YFrameButton *getButton(char c);
     void positionButton(YFrameButton *b, int &xPos, bool onRight);
@@ -406,6 +417,9 @@ private:
 
 #ifdef CONFIG_TASKBAR
     TaskBarApp *fTaskBarApp;
+#endif
+#ifdef CONFIG_TRAY
+    TrayApp *fTrayApp;
 #endif
     MiniIcon *fMiniIcon;
 #ifdef CONFIG_WINLIST
@@ -422,9 +436,13 @@ private:
 
     long fWinWorkspace;
     long fWinLayer;
+#ifdef CONFIG_TRAY
+    long fWinTrayOption;
+#endif
     long fWinState;
     long fWinStateMask;
     bool fManaged;
+    long fWinOptionMask;
 
     YMsgBox *fKillMsgBox;
 };
@@ -451,6 +469,19 @@ extern YPixmap *titleR[2];
 extern YPixmap *titleQ[2];
 
 extern YPixmap *menuButton[2];
+
+#ifdef CONFIG_GRADIENTS
+class YPixbuf;
+
+extern YPixbuf *rgbFrameT[2][2];
+extern YPixbuf *rgbFrameL[2][2];
+extern YPixbuf *rgbFrameR[2][2];
+extern YPixbuf *rgbFrameB[2][2];
+extern YPixbuf *rgbTitleS[2];
+extern YPixbuf *rgbTitleT[2];
+extern YPixbuf *rgbTitleB[2];
+#endif
+
 #endif
 
 #endif
