@@ -9,6 +9,7 @@
 
 #include "aaddressbar.h"
 #include "atasks.h"
+#include "atray.h"
 #include "aworkspaces.h"
 #include "sysdep.h"
 #include "wmtaskbar.h"
@@ -31,13 +32,22 @@ XContext clientContext;
 
 YAction *layerActionSet[WinLayerCount];
 
-YWindowManager::YWindowManager(YWindow *parent, Window win): YDesktop(parent, win) {
+#ifdef CONFIG_TRAY
+YAction *trayOptionActionSet[WinTrayOptionCount];
+#endif
+
+YWindowManager::YWindowManager(YWindow *parent, Window win):
+YDesktop(parent, win) {
     fShuttingDown = false;
     fFocusWin = 0;
-    for (int l = 0; l < WinLayerCount; l++) {
+    for (int l(0); l < WinLayerCount; l++) {
         layerActionSet[l] = new YAction();
         fTop[l] = fBottom[l] = 0;
     }
+#ifdef CONFIG_TRAY
+    for (int k(0); k < WinTrayOptionCount; k++)
+        trayOptionActionSet[k] = new YAction();
+#endif
     fColormapWindow = 0;
     fActiveWorkspace = WinWorkspaceInvalid;
     fLastWorkspace = WinWorkspaceInvalid;
@@ -56,13 +66,13 @@ YWindowManager::YWindowManager(YWindow *parent, Window win): YDesktop(parent, wi
     setPointer(YApplication::leftPointer);
 
     fTopWin = new YWindow();;
-    if (edgeWorkspaceSwitching) {
-        fLeftSwitch = new EdgeSwitch(this, -1);
+    if (edgeHorzWorkspaceSwitching) {
+        fLeftSwitch = new EdgeSwitch(this, -1, false);
         if (fLeftSwitch) {
             fLeftSwitch->setGeometry(0, 0, 1, height());
             fLeftSwitch->show();
         }
-        fRightSwitch = new EdgeSwitch(this, 1);
+        fRightSwitch = new EdgeSwitch(this, +1, false);
         if (fRightSwitch) {
             fRightSwitch->setGeometry(width() - 1, 0, 1, height());
             fRightSwitch->show();
@@ -70,6 +80,22 @@ YWindowManager::YWindowManager(YWindow *parent, Window win): YDesktop(parent, wi
     } else {
         fLeftSwitch = fRightSwitch = 0;
     }
+
+    if (edgeVertWorkspaceSwitching) {
+        fTopSwitch = new EdgeSwitch(this, -1, true);
+        if (fTopSwitch) {
+            fTopSwitch->setGeometry(0, 0, width(), 1);
+            fTopSwitch->show();
+        }
+        fBottomSwitch = new EdgeSwitch(this, +1, true);
+        if (fBottomSwitch) {
+            fBottomSwitch->setGeometry(0, height() - 1, width(), 1);
+            fBottomSwitch->show();
+        }
+    } else {
+        fTopSwitch = fBottomSwitch = 0;
+    }
+
     YWindow::setWindowFocus();
 }
 
@@ -129,6 +155,15 @@ void YWindowManager::grabKeys() {
     GRAB_WMKEY(gKeySysWorkspace11TakeWin);
     GRAB_WMKEY(gKeySysWorkspace12TakeWin);
 
+    GRAB_WMKEY(gKeySysTileVertical);
+    GRAB_WMKEY(gKeySysTileHorizontal);
+    GRAB_WMKEY(gKeySysCascade);
+    GRAB_WMKEY(gKeySysArrange);
+    GRAB_WMKEY(gKeySysUndoArrange);
+    GRAB_WMKEY(gKeySysArrangeIcons);
+    GRAB_WMKEY(gKeySysMinimizeAll);
+    GRAB_WMKEY(gKeySysHideAll);
+
     {
         KProgram *k = keyProgs;
         while (k) {
@@ -136,18 +171,18 @@ void YWindowManager::grabKeys() {
             k = k->getNext();
         }
     }
-    if (win95keys && app->MetaMask) {
+    if (app->WinMask) {
         //fix -- allow apps to use remaining key combos (except single press)
-        grabKey(XK_Meta_L, 0);
-        grabKey(XK_Meta_R, 0);
+        if (app->Win_L) grabKey(app->Win_L, 0);
+        if (app->Win_R) grabKey(app->Win_R, 0);
     }
 
     if (useMouseWheel) {
         grabButton(4, ControlMask | app->AltMask);
         grabButton(5, ControlMask | app->AltMask);
-        if (modMetaIsCtrlAlt && app->MetaMask) {
-            grabButton(4, app->MetaMask);
-            grabButton(5, app->MetaMask);
+        if (app->WinMask) {
+            grabButton(4, app->WinMask);
+            grabButton(5, app->WinMask);
         }
     }
 }
@@ -277,6 +312,22 @@ bool YWindowManager::handleKey(const XKeyEvent &key) {
             switchToWorkspace(10, true);
         } else if (IS_WMKEY(k, vm, gKeySysWorkspace12TakeWin)) {
             switchToWorkspace(11, true);
+	} else if(IS_WMKEY(k, vm, gKeySysTileVertical)) {
+	    wmapp->actionPerformed(actionTileVertical, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysTileHorizontal)) {
+	    wmapp->actionPerformed(actionTileHorizontal, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysCascade)) {
+	    wmapp->actionPerformed(actionCascade, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysArrange)) {
+	    wmapp->actionPerformed(actionArrange, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysUndoArrange)) {
+	    wmapp->actionPerformed(actionUndoArrange, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysArrangeIcons)) {
+	    wmapp->actionPerformed(actionArrangeIcons, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysMinimizeAll)) {
+	    wmapp->actionPerformed(actionMinimizeAll, 0);
+	} else if(IS_WMKEY(k, vm, gKeySysHideAll)) {
+	    wmapp->actionPerformed(actionHideAll, 0);
 #ifdef CONFIG_ADDRESSBAR
         } else if (IS_WMKEY(k, vm, gKeySysAddressBar)) {
             if (taskBar) {
@@ -298,12 +349,14 @@ bool YWindowManager::handleKey(const XKeyEvent &key) {
             }
         }
 
-        if (k == XK_Meta_L || k == XK_Meta_R) {
-            /// !!! needs sync grab
-            XAllowEvents(app->display(), ReplayKeyboard, CurrentTime);
-        } else if (m & app->MetaMask) {
-            /// !!! needs sync grab
-            XAllowEvents(app->display(), ReplayKeyboard, CurrentTime);
+        if (app->WinMask) {
+            if (k == app->Win_L || k == app->Win_R) {
+                /// !!! needs sync grab
+                XAllowEvents(app->display(), ReplayKeyboard, CurrentTime);
+            } else if (m & app->WinMask) {
+                /// !!! needs sync grab
+                XAllowEvents(app->display(), ReplayKeyboard, CurrentTime);
+            }
         }
     } else if (key.type == KeyRelease) {
 #ifdef DEBUG
@@ -333,7 +386,7 @@ void YWindowManager::handleButton(const XButtonEvent &button) {
     }
     YFrameWindow *frame = 0;
     if (useMouseWheel && ((frame = getFocus()) != 0) && button.type == ButtonPress &&
-        ((KEY_MODMASK(button.state) == app->MetaMask && app->MetaMask) ||
+        ((KEY_MODMASK(button.state) == app->WinMask && app->WinMask) ||
          (KEY_MODMASK(button.state) == ControlMask + app->AltMask && app->AltMask)))
     {
         if (button.button == 4)
@@ -343,7 +396,7 @@ void YWindowManager::handleButton(const XButtonEvent &button) {
     }
     if (button.type == ButtonPress) do {
 #ifndef NO_CONFIGURE_MENUS
-        if (button.button + 10 == rootMenuButton) {
+        if (button.button + 10 == (unsigned) rootMenuButton) {
             if (rootMenu)
                 rootMenu->popup(0, 0, button.x, button.y, -1, -1,
                                 YPopupWindow::pfCanFlipVertical |
@@ -354,7 +407,7 @@ void YWindowManager::handleButton(const XButtonEvent &button) {
         }
 #endif
 #ifdef CONFIG_WINMENU
-        if (button.button + 10 == rootWinMenuButton) {
+        if (button.button + 10 == (unsigned) rootWinMenuButton) {
             popupWindowListMenu(button.x, button.y);
             break;
         }
@@ -375,7 +428,7 @@ void YWindowManager::handleButton(const XButtonEvent &button) {
 void YWindowManager::handleClick(const XButtonEvent &up, int count) {
     if (count == 1) do {
 #ifndef NO_CONFIGURE_MENUS
-        if (up.button == rootMenuButton) {
+        if (up.button == (unsigned) rootMenuButton) {
             if (rootMenu)
                 rootMenu->popup(0, 0, up.x, up.y, -1, -1,
                                 YPopupWindow::pfCanFlipVertical |
@@ -385,13 +438,13 @@ void YWindowManager::handleClick(const XButtonEvent &up, int count) {
         }
 #endif
 #ifdef CONFIG_WINMENU
-        if (up.button == rootWinMenuButton) {
+        if (up.button == (unsigned) rootWinMenuButton) {
             popupWindowListMenu(up.x, up.y);
             break;
         }
 #endif
 #ifdef CONFIG_WINLIST
-        if (up.button == rootWinListButton) {
+        if (up.button == (unsigned) rootWinListButton) {
             if (windowList)
                 windowList->showFocused(up.x_root, up.y_root);
             break;
@@ -461,7 +514,7 @@ void YWindowManager::handleClientMessage(const XClientMessageEvent &message) {
 
 Window YWindowManager::findWindow(char const * resource) {
     char *wmInstance(0), *wmClass(0);
-    
+
     char const * dot(resource ? strchr(resource, '.') : 0);
 
     if (dot) {
@@ -474,7 +527,7 @@ Window YWindowManager::findWindow(char const * resource) {
 
     delete[] wmClass;
     delete[] wmInstance;
-    
+
     return win;
 }
 
@@ -488,28 +541,28 @@ Window YWindowManager::findWindow(Window root, char const * wmInstance,
 
     if (clients) {
 	unsigned n;
-	
+
 	for (n = 0; !firstMatch && n < nClients; ++n) {
 	    XClassHint wmclass;
 
 	    if (XGetClassHint(app->display(), clients[n], &wmclass)) {
 		if ((wmInstance == NULL ||
 		    strcmp(wmInstance, wmclass.res_name) == 0) &&
-		    (wmClass == NULL || 
+		    (wmClass == NULL ||
 		    strcmp(wmClass, wmclass.res_class) == 0))
 		    firstMatch = clients[n];
-		    
+
 		XFree(wmclass.res_name);
 		XFree(wmclass.res_class);
 	    }
-	    
+
 	    if (!firstMatch)
 		firstMatch = findWindow(clients[n], wmInstance, wmClass);
 	}
 
 	XFree(clients);
     }
-    
+
     return firstMatch;
 }
 
@@ -548,13 +601,20 @@ void YWindowManager::setFocus(YFrameWindow *f, bool /*canWarp*/) {
         if (ff) switchFocusFrom(ff);
     }
 
+    bool input = true;
+    XWMHints *hints = c ? c->hints() : 0;
+
+    if (hints && (hints->flags & InputHint) && !hints->input)
+        input = false;
+
     if (f && f->visible()) {
         if (c && c->visible() && !(f->isRollup() || f->isIconic()))
             w = c->handle();
         else
             w = f->handle();
 
-        switchFocusTo(f);
+        if (input)
+            switchFocusTo(f);
     }
 #if 0
     if (w == desktop->handle()) {
@@ -571,10 +631,14 @@ void YWindowManager::setFocus(YFrameWindow *f, bool /*canWarp*/) {
             app->getEventTime(), w);
     }
 #endif
-    XSetInputFocus(app->display(), w, RevertToNone, CurrentTime);
 
-    if (c && w == c->handle() && c->protocols() & YFrameClient::wpTakeFocus)
+    if (input) {
+        XSetInputFocus(app->display(), w, RevertToNone, CurrentTime);
+    }
+
+    if (c && w == c->handle() && c->protocols() & YFrameClient::wpTakeFocus) {
         c->sendMessage(_XA_WM_TAKE_FOCUS);
+    }
 
     if (!pointerColormap)
         setColormapWindow(f);
@@ -663,7 +727,7 @@ void YWindowManager::installColormap(Colormap cmap) {
         //MSG(("installing colormap 0x%lX", cmap));
         if (app->grabWindow() == 0) {
             if (cmap == None) {
-                XInstallColormap(app->display(), defaultColormap);
+                XInstallColormap(app->display(), app->colormap());
             } else {
                 XInstallColormap(app->display(), cmap);
             }
@@ -898,20 +962,20 @@ void YWindowManager::smartPlace(YFrameWindow **w, int count) {
 
 void YWindowManager::getCascadePlace(YFrameWindow *frame, int &lastX, int &lastY, int &x, int &y, int w, int h) {
     /// !!! make auto placement cleaner and (optionally) smarter
-    if (lastX < minX(frame->getLayer())) lastX = minX(frame->getLayer());
-    if (lastY < minY(frame->getLayer())) lastY = minY(frame->getLayer());
+    if (lastX < minX(frame)) lastX = minX(frame);
+    if (lastY < minY(frame)) lastY = minY(frame);
 
     x = lastX;
     y = lastY;
 
     lastX += wsTitleBar;
     lastY += wsTitleBar;
-    if (int(y + h) >= maxY(frame->getLayer())) {
-        y = minY(frame->getLayer());
+    if (int(y + h) >= maxY(frame)) {
+        y = minY(frame);
         lastY = wsTitleBar;
     }
-    if (int(x + w) >= maxX(frame->getLayer())) {
-        x = minX(frame->getLayer());
+    if (int(x + w) >= maxX(frame)) {
+        x = minX(frame);
         lastX = wsTitleBar;
     }
 }
@@ -977,7 +1041,7 @@ void YWindowManager::getNewPosition(YFrameWindow *frame, int &x, int &y, int w,
 
 void YWindowManager::placeWindow(YFrameWindow *frame, int x, int y,
 				 bool newClient, bool &
-#ifdef SM
+#ifdef CONFIG_SESSION
 canActivate
 #endif
 ) {
@@ -986,7 +1050,7 @@ canActivate
     int posWidth = client->width() + 2 * frame->borderX();
     int posHeight = client->height() + 2 * frame->borderY() + frame->titleY();
 
-#ifdef SM
+#ifdef CONFIG_SESSION
     if (app->haveSessionManager() && findWindowInfo(frame)) {
         if (frame->getWorkspace() != manager->activeWorkspace())
             canActivate = false;
@@ -1041,13 +1105,16 @@ canActivate
 }
 
 YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
-    YFrameWindow *frame = 0;
-    YFrameClient *client = 0;
-    int cx = 0;
-    int cy = 0;
-    bool canManualPlace = false;
-    long workspace = 0, layer = 0, state_mask = 0, state = 0;
-    bool canActivate = true;
+    YFrameWindow *frame(NULL);
+    YFrameClient *client(NULL);
+    int cx(0);
+    int cy(0);
+    bool canManualPlace(false);
+    long workspace(0), layer(0), state_mask(0), state(0);
+    bool canActivate(true);
+#ifdef CONFIG_TRAY
+    long tray(0);
+#endif
 
     MSG(("managing window 0x%lX", win));
     frame = findFrame(win);
@@ -1085,6 +1152,10 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
         client->setColormap(attributes.colormap);
     }
 
+#ifdef CONFIG_WM_SESSION
+    setTopLevelProcess(client->pid());
+#endif
+
     MSG(("initial geometry (%d:%d %dx%d)",
          client->x(), client->y(), client->width(), client->height()));
 
@@ -1102,7 +1173,7 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
 
     placeWindow(frame, cx, cy, (phase != phaseStartup), canActivate);
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     frame->setShape();
 #endif
 
@@ -1139,7 +1210,12 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
     if (frame->client()->getWinWorkspaceHint(&workspace))
         frame->setWorkspace(workspace);
 
-    if ((limitSize || limitPosition) && 
+#ifdef CONFIG_TRAY
+    if (frame->client()->getWinTrayHint(&tray))
+        frame->setTrayOption(tray);
+#endif
+
+    if ((limitSize || limitPosition) &&
         (phase != phaseStartup) &&
 	!frame->doNotCover()) {
         int posX(frame->x() + frame->borderX()),
@@ -1148,10 +1224,8 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
 	    posHeight(frame->height() - 2 * frame->borderY());
 
         if (limitSize) {
-            posWidth = min(posWidth,
-	    		   maxX(frame->getLayer()) - minX(frame->getLayer()));
-            posHeight = min(posHeight,
-			    maxY(frame->getLayer()) - minY(frame->getLayer()));
+            posWidth = min(posWidth, maxWidth(frame));
+            posHeight = min(posHeight, maxHeight(frame));
 
             posHeight -= frame->titleY();
             frame->client()->constrainSize(posWidth, posHeight,
@@ -1162,10 +1236,10 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
         if (limitPosition &&
             !(client->sizeHints() &&
 	     (client->sizeHints()->flags & USPosition))) {
-            posX = clamp(posX, minX(frame->getLayer()),
-	    		       maxX(frame->getLayer()) - posWidth);
-            posY = clamp(posY, minY(frame->getLayer()),
-	    		       maxY(frame->getLayer()) - posHeight);
+            posX = clamp(posX, minX(frame),
+	    		       maxX(frame) - posWidth);
+            posY = clamp(posY, minY(frame),
+	    		       maxY(frame) - posHeight);
         }
 
         posX -= frame->borderX();
@@ -1214,6 +1288,7 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
                 frame->wmMove();
         }
     }
+
 end:
     XUngrabServer(app->display());
     return frame;
@@ -1230,10 +1305,12 @@ YFrameWindow *YWindowManager::mapClient(Window win) {
         if (clickFocus || !strongPointerFocus)
             frame->activate(true);/// !!! is this ok
     }
+    
     return frame;
 }
 
-void YWindowManager::unmanageClient(Window win, bool mapClient) {
+void YWindowManager::unmanageClient(Window win, bool mapClient,
+				    bool reparent) {
     YFrameWindow *frame = findFrame(win);
 
     MSG(("unmanaging window 0x%lX", win));
@@ -1243,9 +1320,11 @@ void YWindowManager::unmanageClient(Window win, bool mapClient) {
 
         // !!! cleanup
         client->hide();
+
         frame->hide();
-        frame->unmanage();
+        frame->unmanage(reparent);
         delete frame;
+
         if (mapClient)
             client->show();
         delete client;
@@ -1341,6 +1420,12 @@ void YWindowManager::restackWindows(YFrameWindow *win) {
     if (fRightSwitch && fRightSwitch->visible())
         count++;
 
+    if (fTopSwitch && fTopSwitch->visible())
+        count++;
+
+    if (fBottomSwitch && fBottomSwitch->visible())
+        count++;
+
     if (count == 0)
         return ;
 
@@ -1366,6 +1451,12 @@ void YWindowManager::restackWindows(YFrameWindow *win) {
     if (fRightSwitch && fRightSwitch->visible())
         w[i++] = fRightSwitch->handle();
 
+    if (fTopSwitch && fTopSwitch->visible())
+        w[i++] = fTopSwitch->handle();
+
+    if (fBottomSwitch && fBottomSwitch->visible())
+        w[i++] = fBottomSwitch->handle();
+
 #ifndef LITE
     if (ctrlAltDelete && ctrlAltDelete->visible())
         w[i++] = ctrlAltDelete->handle();
@@ -1419,6 +1510,7 @@ int YWindowManager::minX(long layer) const {
     else
         return fMinX;
 }
+
 int YWindowManager::minY(long layer) const {
     if (layer >= WinLayerDock || layer < 0)
         return 0;
@@ -1440,6 +1532,22 @@ int YWindowManager::maxY(long layer) const {
         return fMaxY;
 }
 
+int YWindowManager::minX(YFrameWindow const *frame) const {
+    return minX(frame->doNotCover() ? -1 : frame->getLayer());
+}
+
+int YWindowManager::minY(YFrameWindow const *frame) const {
+    return minY(frame->doNotCover() ? -1 : frame->getLayer());
+}
+
+int YWindowManager::maxX(YFrameWindow const *frame) const {
+    return maxX(frame->doNotCover() ? -1 : frame->getLayer());
+}
+
+int YWindowManager::maxY(YFrameWindow const *frame) const {
+    return maxY(frame->doNotCover() ? -1 : frame->getLayer());
+}
+
 void YWindowManager::updateWorkArea() {
     int nMinX(0),
 	nMinY(0),
@@ -1450,7 +1558,7 @@ void YWindowManager::updateWorkArea() {
 
     YFrameWindow * w;
 
-    if (limitByDockLayer)
+    if (limitByDockLayer)	// -------- find the first doNotCover window ---
 	w = top(WinLayerDock);
     else
 	for (w = topLayer();
@@ -1459,38 +1567,35 @@ void YWindowManager::updateWorkArea() {
 
     while(w) {
         // !!! FIX: WORKAREA windows must currently be sticky
-
-        if (w->isHidden() ||
-            w->isRollup() ||
-            w->isIconic() ||
-            w->isMinimized() ||
-            !w->visibleNow() ||
-            !w->isSticky());
-else {
-      //      continue;
-
+        if (!(w->isHidden() ||
+              w->isRollup() ||
+              w->isIconic() ||
+              w->isMinimized() ||
+              !w->visibleNow() ||
+              !w->isSticky())) {
         // hack
-        int wMinX(nMinX), wMinY(nMinY), wMaxX(nMaxX), wMaxY(nMaxY);
-        bool const isHoriz(w->width() > w->height());
+	    int wMinX(nMinX), wMinY(nMinY), wMaxX(nMaxX), wMaxY(nMaxY);
+	    bool const isHoriz(w->width() > w->height());
 
-        if (!isHoriz /*!!!&& !(w->getState() & WinStateDockHorizontal)*/) {
-            if (w->x() + int(w->width()) < midX)
-                wMinX = w->x() + w->width();
-            else if (w->x() > midX)
-                wMaxX = w->x();
-        } else {
-            if (w->y() + int(w->height()) < midY)
-                wMinY = w->y() + w->height();
-            else if (w->y() > midY)
-                wMaxY = w->y();
-        }
+	    if (!isHoriz /*!!!&& !(w->getState() & WinStateDockHorizontal)*/) {
+		if (w->x() + int(w->width()) < midX)
+		    wMinX = w->x() + w->width();
+		else if (w->x() > midX)
+		    wMaxX = w->x();
+	    } else {
+		if (w->y() + int(w->height()) < midY)
+		    wMinY = w->y() + w->height();
+		else if (w->y() > midY)
+		    wMaxY = w->y();
+	    }
 
-	nMinX = max(nMinX, wMinX);
-	nMinY = max(nMinY, wMinY);
-	nMaxX = min(nMaxX, wMaxX);
-	nMaxY = min(nMaxY, wMaxY);
-}
-	if (limitByDockLayer)
+	    nMinX = max(nMinX, wMinX);
+	    nMinY = max(nMinY, wMinY);
+	    nMaxX = min(nMaxX, wMaxX);
+	    nMaxY = min(nMaxY, wMaxY);
+	}
+
+	if (limitByDockLayer)	// --------- find the next doNotCover window ---
 	    w = w->next();
 	else
 	    do w = w->nextLayer();
@@ -1499,7 +1604,6 @@ else {
 
     if (fMinX != nMinX || fMinY != nMinY || // -- store the new workarea ---
         fMaxX != nMaxX || fMaxY != nMaxY) {
-
         int const deltaX(nMinX - fMinX);
         int const deltaY(nMinY - fMinY);
 
@@ -1540,10 +1644,10 @@ void YWindowManager::resizeWindows() {
 	if (!f->doNotCover()) {
 	    if (f->isMaximized() || f->canSize())
 		f->updateLayout();
-#if 0	
+#if 0
             if (f->isMaximized())
 		f->updateLayout();
-#endif	
+#endif
 #if 0
 	    if (isMaximizedFully())
 		f->setGeometry(fMinX, fMinY, fMaxX - fMinX, fMaxY - fMinY);
@@ -1558,7 +1662,8 @@ void YWindowManager::resizeWindows() {
 void YWindowManager::activateWorkspace(long workspace) {
     if (workspace != fActiveWorkspace) {
 #ifdef CONFIG_TASKBAR
-        if (taskBar && taskBar->workspacesPane() && fActiveWorkspace != (long)WinWorkspaceInvalid) {
+        if (taskBar && taskBar->workspacesPane() &&
+	    fActiveWorkspace != (long)WinWorkspaceInvalid) {
             if (taskBar->workspacesPane()->workspaceButton(fActiveWorkspace))
             {
                 taskBar->workspacesPane()->workspaceButton(fActiveWorkspace)->setPressed(0);
@@ -1619,6 +1724,10 @@ void YWindowManager::activateWorkspace(long workspace) {
         if (taskBar && taskBar->taskPane())
             taskBar->taskPane()->relayout();
 #endif
+#ifdef CONFIG_TRAY
+        if (taskBar && taskBar->trayPane())
+            taskBar->trayPane()->relayout();
+#endif
 #ifndef LITE
         if (workspaceSwitchStatus && (!showTaskBar || !taskBarShowWorkspaces))
             statusWorkspace->begin(workspace);
@@ -1647,19 +1756,17 @@ void YWindowManager::getIconPosition(YFrameWindow *frame, int *iconX, int *iconY
     static int x = 0, y = 0;
     MiniIcon *iw = frame->getMiniIcon();
 
-    if (x < minX(0))
-        x = minX(0);
-    if (y < minY(0))
-        y = minY(0);
+    x = max(x, minX(WinLayerDesktop));
+    y = max(y, minY(WinLayerDesktop));
 
     *iconX = x;
     *iconY = y;
 
     y += iw->height();
-    if (y >= maxY(0)) {
+    if (y >= maxY(WinLayerDesktop)) {
         x += iw->width();
-        y = minX(0);
-        if (x >= maxX(0)) {
+        y = minX(WinLayerDesktop);
+        if (x >= maxX(WinLayerDesktop)) {
             x = 0;
             y = 0;
         }
@@ -1744,28 +1851,27 @@ void YWindowManager::handleProperty(const XPropertyEvent &property) {
 }
 
 void YWindowManager::updateClientList() {
-    int w, count = 0;
-    XID *ids;
+    int count = 0;
+    XID *ids = 0;
 
-    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
+    for (YFrameWindow *f(topLayer()); f; f = f->nextLayer())
         if (f->client() && f->client()->adopted())
             count++;
 
-    if ((ids = new XID[count]) == NULL)
-        return ;
+    if ((ids = new XID[count]) != 0) {
+        int w = 0;
+        for (YFrameWindow *f(topLayer()); f; f = f->nextLayer())
+            if (f->client() && f->client()->adopted())
+                ids[w++] = f->client()->handle();
+        PRECONDITION(w == count);
+    }
 
-    w = 0;
-    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
-        if (f->client() && f->client()->adopted())
-            ids[w++] = f->client()->handle();
-
-    PRECONDITION(w == count);
     XChangeProperty(app->display(), desktop->handle(),
                     _XA_WIN_CLIENT_LIST,
                     XA_CARDINAL,
                     32, PropModeReplace,
                     (unsigned char *)ids, count);
-    delete ids;
+    delete [] ids;
     checkLogout();
 }
 
@@ -2051,28 +2157,60 @@ void YWindowManager::exitAfterLastClient(bool shuttingDown) {
     checkLogout();
 }
 
-YTimer *EdgeSwitch::fEdgeSwitchTimer = 0;
+#ifdef CONFIG_WM_SESSION
+void YWindowManager::setTopLevelProcess(pid_t p) {
+    if (p != getpid() && p != PID_MAX) {
+	msg("moving process %d to the top", p);
+	fProcessList.push(p);
+    }
+}
 
-EdgeSwitch::EdgeSwitch(YWindowManager *manager, int delta): YWindow(manager) {
+void YWindowManager::removeLRUProcess() {
+    pid_t const lru(fProcessList[0]);
+    msg("Kernel sent a cleanup request. Closing process %d.", lru);
+
+    Window leader(None);
+
+    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
+	if (f->client()->pid() == lru &&
+	   (None != (leader = f->client()->clientLeader()) ||
+	    None != (leader = f->client()->handle())))
+	    break;
+
+    if (leader != None) { // the group leader doesn't have to be mapped
+	msg("Sending WM_DELETE_WINDOW to %p", leader);
+	YFrameClient::sendMessage(leader, _XA_WM_DELETE_WINDOW, CurrentTime);
+    }
+
+/* !!! TODO:	- windows which do not support WM_DELETE_WINDOW
+		- unmapping -> removing from process list
+		- leader == None --> loop over all processes?
+		- s/msg/MSG/ 
+		- apps launched from icewm ignore the PRELOAD library
+*/
+}
+#endif
+
+YTimer *EdgeSwitch::fEdgeSwitchTimer(NULL);
+
+EdgeSwitch::EdgeSwitch(YWindowManager *manager, int delta, bool vertical): 
+YWindow(manager),
+fManager(manager),
+fCursor(delta < 0 ? vertical ? YWMApp::scrollUpPointer
+                             : YWMApp::scrollLeftPointer
+                  : vertical ? YWMApp::scrollDownPointer
+                             : YWMApp::scrollRightPointer),
+fDelta(delta) {
     setStyle(wsOverrideRedirect | wsInputOnly);
-    fManager = manager;
-    fDelta = delta;
-
-    int shape;
-    if (delta < 0)
-        shape = XC_sb_left_arrow;
-    else
-        shape = XC_sb_right_arrow;
-
-    cursor = XCreateFontCursor(app->display(), shape);
     setPointer(YApplication::leftPointer);
 }
 
 EdgeSwitch::~EdgeSwitch() {
     if (fEdgeSwitchTimer && fEdgeSwitchTimer->getTimerListener() == this) {
         fEdgeSwitchTimer->stopTimer();
-        fEdgeSwitchTimer->setTimerListener(0);
-        delete fEdgeSwitchTimer; fEdgeSwitchTimer = 0;
+        fEdgeSwitchTimer->setTimerListener(NULL);
+        delete fEdgeSwitchTimer;
+        fEdgeSwitchTimer = NULL;
     }
 }
 
@@ -2083,12 +2221,13 @@ void EdgeSwitch::handleCrossing(const XCrossingEvent &crossing) {
         if (fEdgeSwitchTimer) {
             fEdgeSwitchTimer->setTimerListener(this);
             fEdgeSwitchTimer->startTimer();
-            setPointer(cursor);
+            setPointer(fCursor);
         }
     } else if (crossing.type == LeaveNotify && crossing.mode == NotifyNormal) {
         if (fEdgeSwitchTimer && fEdgeSwitchTimer->getTimerListener() == this) {
             fEdgeSwitchTimer->stopTimer();
-            fEdgeSwitchTimer->setTimerListener(0);
+            fEdgeSwitchTimer->setTimerListener(NULL);
+            setPointer(YApplication::leftPointer);
         }
     }
 }
@@ -2101,6 +2240,11 @@ bool EdgeSwitch::handleTimer(YTimer *t) {
         fManager->switchToPrevWorkspace(false);
     else
         fManager->switchToNextWorkspace(false);
-    setPointer(YApplication::leftPointer);
-    return false;
+
+    if (edgeContWorkspaceSwitching) {
+        return true;
+    } else {
+        setPointer(YApplication::leftPointer);
+        return false;
+    }
 }
