@@ -7,6 +7,12 @@
 #include "WinMgr.h"
 #include "ytimer.h"
 
+#ifdef CONFIG_WM_SESSION
+#include "yarray.h"
+
+#define PROC_WM_SESSION "/proc/wm-session"
+#endif
+
 #define MAXWORKSPACES 64
 #define INVALID_WORKSPACE 0xFFFFFFFF
 
@@ -17,21 +23,25 @@ extern YAction *workspaceActionMoveTo[MAXWORKSPACES];
 
 extern YAction *layerActionSet[WinLayerCount];
 
+#ifdef CONFIG_TRAY
+extern YAction *trayOptionActionSet[WinTrayOptionCount];
+#endif
+
 class YWindowManager;
 class YFrameClient;
 class YFrameWindow;
 
 class EdgeSwitch: public YWindow, public YTimerListener {
 public:
-    EdgeSwitch(YWindowManager *manager, int delta);
+    EdgeSwitch(YWindowManager *manager, int delta, bool vertical);
     virtual ~EdgeSwitch();
 
     virtual void handleCrossing(const XCrossingEvent &crossing);
     virtual bool handleTimer(YTimer *t);
 private:
     YWindowManager *fManager;
+    YCursor & fCursor;
     int fDelta;
-    Cursor cursor;
 
     static YTimer *fEdgeSwitchTimer;
 };
@@ -72,7 +82,8 @@ public:
     YFrameWindow *findFrame(Window win);
     YFrameClient *findClient(Window win);
     YFrameWindow *manageClient(Window win, bool mapClient = false);
-    void unmanageClient(Window win, bool mapClient = false);
+    void unmanageClient(Window win, bool mapClient = false,
+			bool reparent = true);
     void destroyedClient(Window win);
     YFrameWindow *mapClient(Window win);
     
@@ -98,6 +109,15 @@ public:
     int maxWidth(long layer) const { return maxX(layer) - minX(layer); }
     int maxHeight(long layer) const { return maxY(layer) - minY(layer); }
 
+    int minX(YFrameWindow const *frame) const;
+    int minY(YFrameWindow const *frame) const;
+    int maxX(YFrameWindow const *frame) const;
+    int maxY(YFrameWindow const *frame) const;
+    int maxWidth(YFrameWindow const *frame) const {
+	return maxX(frame) - minX(frame); }
+    int maxHeight(YFrameWindow const *frame) const {
+	return maxY(frame) - minY(frame); }
+
     int calcCoverage(bool down, YFrameWindow *frame, int x, int y, int w, int h);
     void tryCover(bool down, YFrameWindow *frame, int x, int y, int w, int h,
                   int &px, int &py, int &cover);
@@ -174,6 +194,11 @@ public:
 
     void setWorkAreaMoveWindows(bool m) { fWorkAreaMoveWindows = m; }
 
+#ifdef CONFIG_WM_SESSION
+    void setTopLevelProcess(pid_t p);
+    void removeLRUProcess();
+#endif
+
 private:
     struct WindowPosState {
         int x, y, w, h;
@@ -188,13 +213,17 @@ private:
     long fLastWorkspace;
     YFrameWindow *fColormapWindow;
     int fMinX, fMinY, fMaxX, fMaxY;
-    EdgeSwitch *fLeftSwitch, *fRightSwitch;
+    EdgeSwitch *fLeftSwitch, *fRightSwitch, *fTopSwitch, *fBottomSwitch;
     bool fShuttingDown;
     int fArrangeCount;
     WindowPosState *fArrangeInfo;
     YProxyWindow *rootProxy;
     YWindow *fTopWin;
     bool fWorkAreaMoveWindows;
+    
+#ifdef CONFIG_WM_SESSION
+    YStackSet<pid_t> fProcessList;
+#endif
 };
 
 extern YWindowManager *manager;
@@ -207,6 +236,9 @@ extern Atom _XA_WIN_WORKSPACE_COUNT;
 extern Atom _XA_WIN_WORKSPACE_NAMES;
 extern Atom _XA_WIN_WORKAREA;
 extern Atom _XA_WIN_LAYER;
+#ifdef CONFIG_TRAY
+extern Atom _XA_WIN_TRAY;
+#endif
 extern Atom _XA_WIN_ICONS;
 extern Atom _XA_WIN_HINTS;
 extern Atom _XA_WIN_STATE;
