@@ -30,9 +30,9 @@ WindowOptions::~WindowOptions() {
 
     for (i = 0; i < winOptionsCount; i++) {
         if (winOptions[i].name)
-            delete winOptions[i].name;
+            delete[] winOptions[i].name;
         if (winOptions[i].icon)
-            delete winOptions[i].icon;
+            delete[] winOptions[i].icon;
     }
     FREE(winOptions);
     winOptions = 0;
@@ -85,6 +85,9 @@ WindowOption *WindowOptions::getWindowOption(const char *name, bool create, bool
     memset(winOptions + L, 0, sizeof(WindowOption));
     winOptions[L].workspace = WinWorkspaceInvalid;
     winOptions[L].layer = WinLayerInvalid;
+#ifdef CONFIG_TRAY
+    winOptions[L].tray = WinTrayInvalid;
+#endif
     winOptions[L].name = newstr(name);
 
     return winOptions + L;
@@ -145,6 +148,29 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
                 if (strcmp(layers[i].name, arg) == 0)
                     op->layer = layers[i].layer;
         }
+#ifdef CONFIG_TRAY
+    } else if (strcmp(opt, "tray") == 0) {
+        char *endptr;
+        long const t(strtol(arg, &endptr, 10));
+        
+        op->tray = WinTrayInvalid;
+
+        if (arg[0] && !endptr[0])
+            op->tray = t;
+        else {
+            struct {
+                const char *name;
+                int tray;
+            } tray_ops[] = {
+                { "Ignore", WinTrayIgnore },
+                { "Minimized", WinTrayMinimized },
+                { "Exclusive", WinTrayExclusive }
+            };
+            for (unsigned int i = 0; i < ACOUNT(tray_ops); i++)
+                if (strcmp(tray_ops[i].name, arg) == 0)
+                    op->tray = tray_ops[i].tray;
+        }
+#endif	
     } else {
         static struct {
             int what;
@@ -178,7 +204,6 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
             { 2, "ignorePositionHint", YFrameWindow::foIgnorePosition }, //
             { 2, "doNotCover", YFrameWindow::foDoNotCover } //
         };
-        int t = (atoi(arg) == 1) ? 1 : 0;
 
         for (unsigned int a = 0; a < ACOUNT(options); a++) {
             unsigned long *what, *what_mask;
@@ -198,7 +223,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
             }
 
             if (strcmp(opt, options[a].name) == 0) {
-                if (t)
+                if (atoi(arg) != 0)
                     *what = (*what) | options[a].flag;
                 else
                     *what = (*what) & ~options[a].flag;
@@ -211,9 +236,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
 }
 
 void WindowOptions::combineOptions(WindowOption &cm, WindowOption &n) {
-    if (cm.icon == 0)
-        if (n.icon != 0)
-            cm.icon = n.icon;
+    if (!cm.icon && n.icon) cm.icon = newstr(n.icon);
     cm.functions |= n.functions & ~cm.function_mask;
     cm.function_mask |= n.function_mask;
     cm.decors |= n.decors & ~cm.decor_mask;
@@ -224,6 +247,10 @@ void WindowOptions::combineOptions(WindowOption &cm, WindowOption &n) {
         cm.workspace = n.workspace;
     if (n.layer != (long)WinLayerInvalid)
         cm.layer = n.layer;
+#ifdef CONFIG_TRAY
+    if (n.tray != (long)WinTrayInvalid)
+        cm.tray = n.tray;
+#endif
     if (n.gflags & XValue)
         cm.gx = n.gx;
     if (n.gflags & YValue)
