@@ -12,6 +12,7 @@
 #include "wmprog.h"
 #include "wmoption.h"
 #include "wmaction.h"
+#include "wmconfig.h"
 #include "ybutton.h"
 #include "objbutton.h"
 #include "objbar.h"
@@ -24,7 +25,7 @@
 #include "gnomeapps.h"
 #include "themes.h"
 #include "browse.h"
-#ifdef GNOME
+#ifdef CONFIG_GNOME_MENUS
 #include <gnome.h>
 #endif
 #include "wmtaskbar.h"
@@ -39,7 +40,7 @@ DObjectMenuItem::DObjectMenuItem(DObject *object):
     fObject = object;
 #ifndef LITE
     if (object->getIcon())
-        setPixmap(object->getIcon()->small());
+        setIcon(object->getIcon()->small());
 #endif
 }
 
@@ -97,27 +98,11 @@ void ObjectMenu::addContainer(char *name, YIcon *icon, ObjectContainer *containe
 		addSubmenu(name, 0, (ObjectMenu *)container);
 #ifndef LITE
         if (item && icon)
-            item->setPixmap(icon->small());
+            item->setIcon(icon->small());
 #endif
     }
 }
 
-ObjectButton::ObjectButton(YWindow *parent, DObject *object): YButton(parent, 0)
-{
-    fObject = object;
-}
-
-void ObjectButton::actionPerformed(YAction * /*action*/, unsigned int /*modifiers*/) {
-#ifdef CONFIG_GUIEVENTS
-    wmapp->signalGuiEvent(geLaunchApp);
-#endif
-    fObject->open();
-}
-
-ObjectMenu *rootMenu = 0;
-
-#define ACOUNT(x) (sizeof(x)/sizeof(x[0]))
-
 DObject::DObject(const char *name, YIcon *icon) {
     fName = newstr(name);
     fIcon = icon;
@@ -132,15 +117,15 @@ DObject::~DObject() {
 void DObject::open() {
 }
 
-DProgram::DProgram(const char *name, YIcon *icon, bool restart,
-		   const char *wmclass, const char *exe, char **args):
+DProgram::DProgram(const char *name, YIcon *icon, const bool restart,
+		   const char *wmclass, const char *exe, const char **args):
     DObject(name, icon), fRestart(restart), fRes(newstr(wmclass)),
     fCmd(newstr(exe)), fArgs(args) {
 }
 
 DProgram::~DProgram() {
     if (fArgs)
-	for (char **p = fArgs; p && *p; ++p)
+	for (const char **p = fArgs; p && *p; ++p)
 	    delete[] *p;
 
     delete[] fArgs;
@@ -157,14 +142,14 @@ void DProgram::open() {
         app->runProgram(fCmd, fArgs);
 }
 
-DProgram *DProgram::newProgram(const char *name, YIcon *icon, bool restart,
-			       const char *wmclass, const char *exe,
-			       char **args) {
-    char *fullname = 0;
+DProgram *DProgram::newProgram(const char *name, YIcon *icon,
+			       const bool restart, const char *wmclass,
+			       const char *exe, const char **args) {
+    char *fullname(NULL);
 
     if (exe && exe[0] &&  // updates command with full path
-        findPath(getenv("PATH"), X_OK, exe, &fullname) == 0) {
-        for (char **p = args; p && *p; ++p) delete[] *p;
+        NULL == (fullname = findPath(getenv("PATH"), X_OK, exe))) {
+        for (const char **p = args; p && *p; ++p) delete[] *p;
         delete[] args;
 
         MSG(("Program %s (%s) not found.", name, exe));
@@ -188,8 +173,8 @@ char *getWord(char *word, int maxlen, char *p) {
     return p;
 }
 
-char *getCommandArgs(char *p, char *command, int command_len, char **&args, int &argCount) {
-
+char *getCommandArgs(char *p, char *command, int command_len,
+                     const char **&args, int &argCount) {
     p = getArgument(command, command_len, p, false);
     if (p == 0) {
         msg(_("Missing command argument"));
@@ -212,7 +197,8 @@ char *getCommandArgs(char *p, char *command, int command_len, char **&args, int
         }
 
         if (args == 0) {
-            args = (char **)REALLOC((void *)args, ((argCount) + 2) * sizeof(char *));
+            args = (const char **)
+		REALLOC((void *)args, ((argCount) + 2) * sizeof(char *));
             assert(args != NULL);
 
             args[argCount] = newstr(command);
@@ -222,7 +208,8 @@ char *getCommandArgs(char *p, char *command, int command_len, char **&args, int
             argCount++;
         }
 
-        args = (char **)REALLOC((void *)args, ((argCount) + 2) * sizeof(char *));
+        args = (const char **)
+	    REALLOC((void *)args, ((argCount) + 2) * sizeof(char *));
         assert(args != NULL);
 
         args[argCount] = newstr(argx);
@@ -258,30 +245,30 @@ char *parseMenus(char *data, ObjectContainer *container) {
         p = getWord(word, sizeof(word), p);
 
         if (container) {
-	    bool restart = false, runonce = false;
-
 	    if (strcmp(word, "separator") == 0)
 	        container->addSeparator();
-	    else if (strcmp(word, "prog") == 0 || 
-		    (restart = (strcmp(word, "restart") == 0)) ||
-		    (runonce = (strcmp(word, "runonce") == 0))) {
-		char wmclass[256];
-		char icons[128];
+	    else if (!(strcmp(word, "prog") &&
+		       strcmp(word, "restart") &&
+		       strcmp(word, "runonce"))) {
 		char name[64];
 
 		p = getArgument(name, sizeof(name), p, false);
 		if (p == 0) return p;
 
+		char icons[128];
+
 		p = getArgument(icons, sizeof(icons), p, false);
 		if (p == 0) return p;
 
-		if (runonce) {
+		char wmclass[256];
+
+		if (word[1] == 'u') {
 		    p = getArgument(wmclass, sizeof(wmclass), p, false);
 		    if (p == 0) return p;
 		}
 
 		char command[256];
-		char **args = 0;
+		const char **args = 0;
 		int argCount = 0;
 
 		p = getCommandArgs(p, command, sizeof(command), args, argCount);
@@ -293,17 +280,19 @@ char *parseMenus(char *data, ObjectContainer *container) {
 #ifndef LITE
 		if (icons[0] != '-') icon = getIcon(icons);
 #endif
-		DProgram *prog = DProgram::newProgram (name, icon, restart, 
-		     runonce ? wmclass : 0, command, args);
+		DProgram * prog(DProgram::newProgram(name, icon,
+		     word[1] == 'e', word[1] == 'u' ? wmclass : 0, 
+		     command, args));
 
 		if (prog) container->addObject(prog);
 	    } else if (strcmp(word, "menu") == 0) {
-		char icons[128];
 		char name[64];
 
 		p = getArgument(name, sizeof(name), p, false);
 		if (p == 0) return p;
 
+		char icons[128];
+
 		p = getArgument(icons, sizeof(icons), p, false);
 		if (p == 0) return p;
 
@@ -334,14 +323,22 @@ char *parseMenus(char *data, ObjectContainer *container) {
 	    else
 		return 0;
         } else {
-	    if (strcmp(word, "key") == 0) {
+	    if (!(strcmp(word, "key") &&
+	          strcmp(word, "runonce"))) {
 		char key[64];
 
 		p = getArgument(key, sizeof(key), p, false);
 		if (p == 0) return p;
 
+		char wmclass[256];
+
+		if (*word == 'r') {
+		    p = getArgument(wmclass, sizeof(wmclass), p, false);
+		    if (p == 0) return p;
+		}
+
 		char command[256];
-		char **args = 0;
+		const char **args = 0;
 		int argCount = 0;
 
 		p = getCommandArgs(p, command, sizeof(command), args, argCount);
@@ -350,8 +347,8 @@ char *parseMenus(char *data, ObjectContainer *container) {
 		    return p;
 		}
 
-		DProgram *prog =
-		    DProgram::newProgram (key, 0, false, 0, command, args);
+		DProgram *prog(DProgram::newProgram(key, 0,
+		    false, *word == 'r' ? wmclass : 0, command, args));
 
 		if (prog) new KProgram(key, prog);
             } else
@@ -467,7 +464,7 @@ bool StartMenu::handleKey(const XKeyEvent &key) {
 void StartMenu::updatePopup() {
     MenuFileMenu::updatePopup();
 
-#ifdef GNOME
+#ifdef CONFIG_GNOME_MENUS
     if (autoReloadMenus) {
         char *gnomeAppsMenu = gnome_datadir_file("gnome/apps/");
         char *gnomeUserMenu = gnome_util_home_file("apps/");
@@ -524,32 +521,38 @@ void StartMenu::refresh() {
     MenuFileMenu::refresh();
 
 #ifndef NO_CONFIGURE_MENUS
-    addSeparator();
-#ifdef GNOME
+    if (itemCount()) addSeparator();
+#ifdef CONFIG_WINLIST
+    int const oldItemCount(itemCount());
+#endif    
+#ifdef CONFIG_GNOME_MENUS
     {
-        YIcon *gnomeicon = 0;
-        YIcon *kdeicon = 0;
+        YIcon *gnomeIcon = 0;
+        YIcon *kdeIcon = 0;
 
         char *gnomeAppsMenu = gnome_datadir_file("gnome/apps/");
         char *gnomeUserMenu = gnome_util_home_file("apps/");
         const char *kdeMenu = strJoin(kdeDataDir, "/applnk", 0);
 
-        fHasGnomeAppsMenu = !access(gnomeAppsMenu, X_OK | R_OK);
-        fHasGnomeUserMenu = !access(gnomeUserMenu, X_OK | R_OK);
-        fHasKDEMenu       = !access(kdeMenu, X_OK | R_OK);
+        fHasGnomeAppsMenu = showGnomeAppsMenu &&
+			    !access(gnomeAppsMenu, X_OK | R_OK);
+        fHasGnomeUserMenu = showGnomeUserMenu &&
+			    !access(gnomeUserMenu, X_OK | R_OK);
+        fHasKDEMenu       = showKDEMenu &&
+			    !access(kdeMenu, X_OK | R_OK);
 
 	if (fHasGnomeAppsMenu || fHasGnomeUserMenu)
-	    gnomeicon = getIcon("gnome");
+	    gnomeIcon = getIcon("gnome");
 
 	if (fHasGnomeAppsMenu)
-	    kdeicon = getIcon("kde");
+	    kdeIcon = getIcon("kde");
 
         if (fHasGnomeAppsMenu)
             if (gnomeAppsMenuAtToplevel)
                 GnomeMenu::createToplevel(this, gnomeAppsMenu);
             else
                 GnomeMenu::createSubmenu(this, gnomeAppsMenu,
-		    _("Gnome"), gnomeicon ? gnomeicon->small () : 0);
+		    _("Gnome"), gnomeIcon ? gnomeIcon->small () : 0);
 
         if (fHasGnomeAppsMenu && fHasGnomeUserMenu &&
             (gnomeAppsMenuAtToplevel || gnomeUserMenuAtToplevel))
@@ -560,7 +563,7 @@ void StartMenu::refresh() {
                 GnomeMenu::createToplevel(this, gnomeUserMenu);
             else
                 GnomeMenu::createSubmenu(this, gnomeUserMenu,
-		    _("Gnome User Apps"), gnomeicon ? gnomeicon->small () : 0);
+		    _("Gnome User Apps"), gnomeIcon ? gnomeIcon->small () : 0);
 
         if (fHasGnomeUserMenu && fHasKDEMenu &&
             (gnomeUserMenuAtToplevel || kdeMenuAtToplevel))
@@ -571,14 +574,14 @@ void StartMenu::refresh() {
                 GnomeMenu::createToplevel(this, kdeMenu);
             else
                 GnomeMenu::createSubmenu(this, kdeMenu,
-		    _("KDE"), kdeicon ? kdeicon->small () : 0);
+		    _("KDE"), kdeIcon ? kdeIcon->small () : 0);
 
         g_free(gnomeAppsMenu);
         g_free(gnomeUserMenu);
         delete kdeMenu;
 	
-//	delete gnomeicon;
-//	delete kdeicon;
+//	delete gnomeIcon;
+//	delete kdeIcon;
     }
 #endif
     ObjectMenu *programs = new MenuFileMenu("programs", 0);
@@ -607,33 +610,27 @@ void StartMenu::refresh() {
                 item->setSubmenu(sub);
 #ifndef LITE
                 if (folder)
-                    item->setPixmap(folder->small());
+                    item->setIcon(folder->small());
 #endif
             }
         }
     }
 #ifdef CONFIG_WINLIST
-    addSeparator();
+    if (itemCount() != oldItemCount) addSeparator();
     addItem(_("_Windows"), -2, actionWindowList, windowListMenu);
 #endif
 
     if (runDlgCommand && runDlgCommand[0])
         addItem(_("_Run..."), -2, "", actionRun);
-    addSeparator();
 
-    if (showThemesMenu) {
-        YMenu *themes = new ThemesMenu();
-        if (themes->itemCount() > 1)
-            addSubmenu(_("_Themes"), -2, themes);
-    }
-    addItem(_("_Logout..."), -2, actionLogout, logoutMenu);
+    addSeparator();
 
 #ifndef LITE
     if (!showTaskBar)
         addItem(_("_About"), -2, actionAbout, 0);
 
     if (showHelp) {
-	char ** args = new (char*)[3];
+	const char ** args = new const char*[3];
 	args[0] = newstr(ICEHELPEXE);
 	args[1] = newstr(ICEHELPIDX);
 	args[2] = 0;
@@ -644,5 +641,14 @@ void StartMenu::refresh() {
 	if (help) addObject(help);
     }
 #endif
+
+    if (showThemesMenu) {
+        YMenu *themes = new ThemesMenu();
+        if (themes->itemCount() > 1)
+            addSubmenu(_("_Themes"), -2, themes);
+    }
+    
+    if (logoutMenu)
+	addItem(_("_Logout..."), -2, actionLogout, logoutMenu);
 }
 #endif
