@@ -17,19 +17,19 @@ public:
     virtual void open();
     
     static char *fullname(const char *exe);
-    static DProgram *newProgram(const char *name, YIcon *icon, bool restart, 
-    				const char *wmclass, const char *exe, 
-				char **args);
+    static DProgram *newProgram(const char *name, YIcon *icon,
+				const bool restart, const char *wmclass,
+				const char *exe, const char **args);
 
 protected:
-    DProgram(const char *name, YIcon *icon, bool restart, const char *wmclass,
-    	     const char *exe, char **args);
+    DProgram(const char *name, YIcon *icon, const bool restart,
+	     const char *wmclass, const char *exe, const char **args);
 
 private:
-    bool fRestart;
-    char *fRes;
-    char *fCmd;
-    char **fArgs;
+    const bool fRestart;
+    const char *fRes;
+    const char *fCmd;
+    const char **fArgs;
 };
 
 class DFile: public DObject {
@@ -89,26 +89,8 @@ private:
     DProgram *fProg;
 };
 
-#endif
-
-// !!! move there somewhere else probably
-int findPath(const char *path, int mode, const char *name, char **fullname, bool path_relative = false);
-
-typedef struct {
-    const char **root;
-    const char *rdir;
-    const char **sub;
-}  pathelem;
-
-extern pathelem icon_paths[10];
-
-char *joinPath(pathelem *pe, const char *base, const char *name);
-void verifyPaths(pathelem *search, const char *base);
-
-int is_reg(const char *path);
-
-void loadPixmap(pathelem *pe, const char *base, const char *name, YPixmap **pixmap);
-
 extern KProgram *keyProgs;
 
+#endif /* NO_CONFIGURE_MENUS */
+
 #endif
