@@ -8,6 +8,7 @@
 #include "config.h"
 
 #ifndef LITE
+#include "ypixbuf.h"
 #include "ykey.h"
 #include "wmswitch.h"
 
@@ -18,18 +19,22 @@
 
 #include <string.h>
 
-YColor *SwitchWindow::switchFg = 0;
-YColor *SwitchWindow::switchBg = 0;
+YColor *SwitchWindow::switchFg(NULL);
+YColor *SwitchWindow::switchBg(NULL);
+YColor *SwitchWindow::switchHl(NULL);
 
-YFont *SwitchWindow::switchFont = 0;
+YFont *SwitchWindow::switchFont(NULL);
 
-SwitchWindow *switchWindow = 0;
+SwitchWindow * switchWindow(NULL);
 
-SwitchWindow::SwitchWindow(YWindow *parent): YPopupWindow(parent) {
+SwitchWindow::SwitchWindow(YWindow *parent):
+    YPopupWindow(parent) INIT_GRADIENT(fGradient, NULL) {
     if (switchBg == 0)
         switchBg = new YColor(clrQuickSwitch);
     if (switchFg == 0)
         switchFg = new YColor(clrQuickSwitchText);
+    if (switchHl == 0 && clrQuickSwitchActive)
+        switchHl = new YColor(clrQuickSwitchActive);
     if (switchFont == 0)
         switchFont = YFont::getFont(switchFontName);
 
@@ -38,15 +43,9 @@ SwitchWindow::SwitchWindow(YWindow *parent): YPopupWindow(parent) {
     modsDown = 0;
     isUp = false;
 
-    int sW = 4 + manager->width() / 5 * 3;
-    int sH = 4 + 32;
-        //statusFont->max_bounds.ascent +
-        //statusFont->max_bounds.descent;
+    resize();
 
-    setGeometry(manager->width() / 2 - sW / 2, manager->height() / 2 - sH / 2,
-                sW, sH);
-
-    setStyle(wsOverrideRedirect);
+    setStyle(wsSaveUnder | wsOverrideRedirect);
 }
 
 SwitchWindow::~SwitchWindow() {
@@ -54,34 +53,192 @@ SwitchWindow::~SwitchWindow() {
         cancelPopup();
         isUp = false;
     }
+    
+#ifdef CONFIG_GRADIENTS
+    delete fGradient;
+#endif    
+}
+
+void SwitchWindow::resize() {
+    const char *cTitle(fActiveWindow ? fActiveWindow->client()->windowTitle()
+				     : 0);
+
+    int const iWidth
+	(max(quickSwitchSmallWindow ? (int) manager->width() * 1/3
+				    : (int) manager->width() * 3/5,
+         max(cTitle ? (int) switchFont->textWidth(cTitle) : 0,
+	     fIconCount * (YIcon::sizeLarge + 2 * quickSwitchIMargin) +
+	    (quickSwitchHugeIcon ? YIcon::sizeHuge - YIcon::sizeLarge : 0))));
+    int const mWidth(manager->width() * 6/7);
+    int const iHeight((quickSwitchHugeIcon ? YIcon::sizeHuge
+					   : YIcon::sizeLarge) +
+		       quickSwitchIMargin * 2);
+
+    int const w((quickSwitchAllIcons && iWidth < mWidth
+		? iWidth : mWidth) + quickSwitchHMargin * 2);
+    int const h((quickSwitchAllIcons
+		? iHeight + switchFont->height() + quickSwitchSepSize
+		: max(iHeight, (int)switchFont->height()))
+		+ quickSwitchVMargin * 2);
+
+    setGeometry((manager->width() - w) >> 1,
+    		(manager->height() - h) >> 1, w, h);
 }
 
 void SwitchWindow::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+#ifdef CONFIG_GRADIENTS
+    if (switchbackPixbuf && !(fGradient &&
+			      fGradient->width() == width() - 2 &&
+			      fGradient->height() == height() - 2)) {
+	delete fGradient;
+	fGradient = new YPixbuf(*switchbackPixbuf, width() - 2, height() - 2);
+    }
+#endif
+
     g.setColor(switchBg);
     g.drawBorderW(0, 0, width() - 1, height() - 1, true);
+    
+#ifdef CONFIG_GRADIENTS
+    if (fGradient)
+        g.copyPixbuf(*fGradient, 1, 1, width() - 2, height() - 2, 1, 1);
+    else
+#endif    
     if (switchbackPixmap)
         g.fillPixmap(switchbackPixmap, 1, 1, width() - 3, height() - 3);
     else
         g.fillRect(1, 1, width() - 3, height() - 3);
 
     if (fActiveWindow) {
-        int ofs = 0, pos;
-        if (fActiveWindow->clientIcon() && fActiveWindow->clientIcon()->large()) {
-            g.drawPixmap(fActiveWindow->clientIcon()->large(), 2, 2);
-            ofs = fActiveWindow->clientIcon()->large()->width() + 2;
-        }
+        int tOfs(0);
+
+	const int ih(quickSwitchHugeIcon ? YIcon::sizeHuge : YIcon::sizeLarge);
+
+        if (!quickSwitchAllIcons &&
+	    fActiveWindow->clientIcon()) {
+	    YIcon::Image * icon(quickSwitchHugeIcon
+		? fActiveWindow->clientIcon()->huge()
+		: fActiveWindow->clientIcon()->large());
+
+	    if (icon)
+		if (quickSwitchTextFirst) {
+		    g.drawImage(icon,
+			width() - icon->width() - quickSwitchIMargin, 
+			(height() - icon->height() - quickSwitchIMargin) / 2);
+		} else {
+		    g.drawImage(icon,
+			quickSwitchIMargin,
+			(height() - icon->height() - quickSwitchIMargin) / 2);
+
+		    tOfs = icon->width() + quickSwitchIMargin
+		         + quickSwitchSepSize;
+		}
+		
+		if (quickSwitchSepSize) {
+		    const int ip(icon->width() + 2 * quickSwitchIMargin +
+		    		 quickSwitchSepSize/2);
+		    const int x(quickSwitchTextFirst ? width() - ip : ip);
+		    
+		    g.setColor(switchBg->darker());
+		    g.drawLine(x + 0, 1, x + 0, width() - 2);
+		    g.setColor(switchBg->brighter());
+		    g.drawLine(x + 1, 1, x + 1, width() - 2);
+		}
+	}
+
         g.setColor(switchFg);
         g.setFont(switchFont);
-        const char *str = fActiveWindow->client()->windowTitle();
-        if (str) {
-            pos = ofs + (width() - ofs) / 2 - switchFont->textWidth(str) / 2;
-            if (pos < ofs)
-                pos = ofs;
-            g.drawChars(str, 0, strlen(str),
-                        pos,
-                        height() / 2 + (switchFont->height()) / 2
-                        - switchFont->descent());
+
+        const char *cTitle(fActiveWindow->client()->windowTitle());
+
+        if (cTitle) {
+	    const int x(max((width() - tOfs - 
+			     switchFont->textWidth(cTitle)) >> 1, 0U) + tOfs);
+	    const int y(quickSwitchAllIcons 	
+	    	      ? quickSwitchTextFirst
+		      ? quickSwitchVMargin + switchFont->ascent()
+		      : height() - quickSwitchVMargin - switchFont->descent()
+		      : ((height() + switchFont->height()) >> 1) -
+		        switchFont->descent());
+
+            g.drawChars(cTitle, 0, strlen(cTitle), x, y);
+	    
+	    if (quickSwitchAllIcons && quickSwitchSepSize) {
+		int const h(quickSwitchVMargin + ih + 
+			    quickSwitchIMargin * 2 + 
+			    quickSwitchSepSize / 2);
+		int const y(quickSwitchTextFirst ? height() - h : h);
+
+		g.setColor(switchBg->darker());
+		g.drawLine(1, y + 0, width() - 2, y + 0);
+		g.setColor(switchBg->brighter());
+		g.drawLine(1, y + 1, width() - 2, y + 1);
+	    }
         }
+	
+	if (quickSwitchAllIcons) {
+	    int const ds(quickSwitchHugeIcon ? YIcon::sizeHuge - 
+	    				       YIcon::sizeLarge : 0);
+	    int const dx(YIcon::sizeLarge + 2 * quickSwitchIMargin);
+
+	    const int visIcons((width() - 2 * quickSwitchHMargin) / dx);
+	    int curIcon(-1); fIconCount = 0;
+
+	    YFrameWindow * first(nextWindow(NULL, true, false));
+	    YFrameWindow * frame(first);
+	    
+	    do {
+		if (frame == fActiveWindow) curIcon = fIconCount;
+		++fIconCount;
+	    } while ((frame = nextWindow(frame, true, true)) != first);
+
+
+	    int const y(quickSwitchTextFirst
+		? height() - quickSwitchVMargin - ih - quickSwitchIMargin + ds / 2
+		: quickSwitchVMargin + ds + quickSwitchIMargin - ds / 2);
+
+	    g.setColor(switchHl ? switchHl : switchBg->brighter());
+
+	    const int off(max(1 + curIcon - visIcons, 0));
+	    const int end(off + visIcons);
+
+	    int x((width() - min(visIcons, fIconCount) * dx - ds) /  2 + 
+	    	   quickSwitchIMargin);
+	    int i(0);
+	    
+	    do {
+	    	if (frame->clientIcon()) {
+		    if (i >= off && i < end) {
+			if (frame == fActiveWindow) {
+			    if (quickSwitchFillSelection)
+				g.fillRect(x - quickSwitchIBorder,
+					   y - quickSwitchIBorder - ds/2, 
+					   ih + 2 * quickSwitchIBorder,
+					   ih + 2 * quickSwitchIBorder);
+			    else
+				g.drawRect(x - quickSwitchIBorder,
+					   y - quickSwitchIBorder - ds/2, 
+					   ih + 2 * quickSwitchIBorder,
+					   ih + 2 * quickSwitchIBorder);
+
+			    YIcon::Image * icon(quickSwitchHugeIcon
+                                ? frame->clientIcon()->huge()
+                                : frame->clientIcon()->large());
+
+			    if (icon) g.drawImage(icon, x, y - ds/2);
+
+			    x+= ds;
+			} else {
+			    YIcon::Image * icon(frame->clientIcon()->large());
+			    if (icon) g.drawImage(icon, x, y);
+			}
+
+			x+= dx;
+		    }
+
+		    ++i;
+		}
+	    } while ((frame = nextWindow(frame, true, true)) != first);
+	}
     }
 }
 
@@ -141,21 +298,50 @@ YFrameWindow *SwitchWindow::nextWindow(YFrameWindow *from, bool zdown, bool next
                              YFrameWindow::fwfSame);
     if (n == 0)
         n = fLastWindow;
+
     return n;
 }
 
 void SwitchWindow::begin(bool zdown, int mods) {
-    modsDown = mods & (app->AltMask | app->MetaMask | app->HyperMask | app->SuperMask | ControlMask);
+    modsDown = mods & (app->AltMask | app->MetaMask | 
+    		       app->HyperMask | app->SuperMask | 
+		       ControlMask);
+
     if (isUp) {
         cancelPopup();
         isUp = false;
-        return ;
-    }
-    fLastWindow = fActiveWindow = manager->getFocus();
-    fActiveWindow = nextWindow(fActiveWindow, zdown, true);
-    if (fActiveWindow) {
-        displayFocus(fActiveWindow);
-        isUp = popup(0, 0, YPopupWindow::pfNoPointerChange);
+    } else {
+	fLastWindow = fActiveWindow = manager->getFocus();
+	fActiveWindow = nextWindow(fLastWindow, zdown, true);
+
+	if (fActiveWindow &&
+	   (!fActiveWindow->isFocusable() ||
+	    !(quickSwitchToAllWorkspaces || fActiveWindow->visibleNow()) ||
+	    (fActiveWindow->frameOptions() & YFrameWindow::foIgnoreQSwitch) ||
+	    (!quickSwitchToMinimized && fActiveWindow->isMinimized()) ||
+	    (!quickSwitchToHidden && fActiveWindow->isHidden()))) {
+	    fActiveWindow = NULL;
+	    app->alert();
+	}
+
+	fIconCount = fIconOffset = 0;
+
+	if (quickSwitchAllIcons && fActiveWindow) {
+	    YFrameWindow * frame(fActiveWindow); do {
+	    	if (fActiveWindow->clientIcon() && 
+		    fActiveWindow->clientIcon()->large())
+		    ++fIconCount;
+	    } while ((frame = nextWindow(frame, zdown, true)) != fActiveWindow);
+	}
+
+	MSG(("fIconCount: %d, fIconOffset: %d", fIconCount, fIconOffset));
+
+	resize();
+
+	if (fActiveWindow) {
+	    displayFocus(fActiveWindow);
+	    isUp = popup(0, 0, YPopupWindow::pfNoPointerChange);
+	}
     }
 }
 
