@@ -9,6 +9,7 @@
 #include "config.h"
 
 #ifdef CONFIG_TASKBAR
+#include "ypixbuf.h"
 #include "yfull.h"
 #include "wmtaskbar.h"
 
@@ -28,106 +29,117 @@
 #include "apppstatus.h"
 #include "amailbox.h"
 #include "objbar.h"
+#include "objbutton.h"
 #include "objmenu.h"
 #include "atasks.h"
+#include "atray.h"
 #include "aworkspaces.h"
 
 #include "aapm.h"
 
 #include "intl.h"
 
-YColor *taskBarBg = 0;
+YColor *taskBarBg(NULL);
 
-YTimer *TaskBarApp::fRaiseTimer = 0;
-YTimer *WorkspaceButton::fRaiseTimer = 0;
+YTimer *TaskBarApp::fRaiseTimer(NULL);
+#ifdef CONFIG_TRAY
+YTimer *TrayApp::fRaiseTimer(NULL);
+#endif
+YTimer *WorkspaceButton::fRaiseTimer(NULL);
 
-TaskBar *taskBar = 0;
+TaskBar *taskBar(NULL);
 
-YPixmap *startPixmap = 0;
-YPixmap *windowsPixmap = 0;
-YPixmap *taskbackPixmap = 0;
+YIcon::Image *icewmImage(NULL);
+YIcon::Image *windowsImage(NULL);
+YPixmap *taskbackPixmap(NULL);
 
-static void initPixmaps() {
-    static const char *home = getenv("HOME");
-    const char *base = 0;
-    static char themeSubdir[256];
-
-    strcpy(themeSubdir, themeName);
-    { char *p = strchr(themeSubdir, '/'); if (p) *p = 0; }
-
-    static const char *themeDir = themeSubdir;
-    pathelem paths[] = {
-        { &home, "/.icewm/themes/", &themeDir },
-        { &home, "/.icewm/", 0,},
-        { &configDir, "/themes/", &themeDir },
-        { &configDir, "/", 0 },
-        { &libDir, "/themes/", &themeDir },
-        { &libDir, "/", 0 },
-        { 0, 0, 0 }
-    };
-    pathelem cpaths[sizeof(paths)/sizeof(paths[0])];
-    verifyPaths(paths, 0);
-
-    base = "taskbar/";
-    memcpy(cpaths, paths, sizeof(cpaths));
-    verifyPaths(cpaths, base);
-
-/** Use Linux 2.0 Penguin as start button */
-#ifndef START_PIXMAP
-#define START_PIXMAP "linux.xpm"
-//#define START_PIXMAP "debian.xpm"
-//#define START_PIXMAP "bsd-daemon.xpm"
-//#define START_PIXMAP "start.xpm"
-//#define START_PIXMAP "xfree86os2.xpm"
+#ifdef CONFIG_GRADIENTS
+YPixbuf *taskbackPixbuf(NULL);
+YPixbuf *taskbuttonPixbuf(NULL);
+YPixbuf *taskbuttonactivePixbuf(NULL);
+YPixbuf *taskbuttonminimizedPixbuf(NULL);
 #endif
 
-    loadPixmap(cpaths, base, START_PIXMAP, &startPixmap);
-    loadPixmap(cpaths, base, "windows.xpm", &windowsPixmap);
-    loadPixmap(cpaths, base, "taskbarbg.xpm", &taskbackPixmap);
-    loadPixmap(cpaths, base, "taskbuttonbg.xpm", &taskbuttonPixmap);
-    loadPixmap(cpaths, base, "taskbuttonactive.xpm", &taskbuttonactivePixmap);
-    loadPixmap(cpaths, base, "taskbuttonminimized.xpm", &taskbuttonminimizedPixmap);
+static void initPixmaps() {
+#ifndef ICEWM_PIXMAP
+#define ICEWM_PIXMAP "icewm.xpm"
+#endif
+
+#ifndef START_PIXMAP
+#define START_PIXMAP "linux.xpm"
+/*
+#define START_PIXMAP "debian.xpm"
+#define START_PIXMAP "bsd-daemon.xpm"
+#define START_PIXMAP "start.xpm"
+#define START_PIXMAP "xfree86os2.xpm"
+*/
+#endif
+    YResourcePaths const paths;
+
+    char const * base("taskbar/");
+    YResourcePaths themedirs(paths, base, true);
+    YResourcePaths subdirs(paths, base);
+
+    if (NULL == (icewmImage = themedirs.loadImage(base, ICEWM_PIXMAP)) &&
+        NULL == (icewmImage = themedirs.loadImage(base, START_PIXMAP)))
+        icewmImage = subdirs.loadImage(base, ICEWM_PIXMAP);
+
+    windowsImage = subdirs.loadImage(base, "windows.xpm");
+
+#ifdef CONFIG_GRADIENTS
+    if (!taskbackPixbuf)
+	taskbackPixmap = subdirs.loadPixmap(base, "taskbarbg.xpm");
+    if (!taskbuttonPixbuf)
+	taskbuttonPixmap = subdirs.loadPixmap(base, "taskbuttonbg.xpm");
+    if (!taskbuttonactivePixbuf)
+	taskbuttonactivePixmap = subdirs.loadPixmap(base, "taskbuttonactive.xpm");
+    if (!taskbuttonminimizedPixbuf)
+	taskbuttonminimizedPixmap = subdirs.loadPixmap(base, "taskbuttonminimized.xpm");
+#else
+    taskbackPixmap = subdirs.loadPixmap(base, "taskbarbg.xpm");
+    taskbuttonPixmap = subdirs.loadPixmap(base, "taskbuttonbg.xpm");
+    taskbuttonactivePixmap = subdirs.loadPixmap(base, "taskbuttonactive.xpm");
+    taskbuttonminimizedPixmap = subdirs.loadPixmap(base, "taskbuttonminimized.xpm");
+#endif    
 
 #ifdef CONFIG_APPLET_MAILBOX
     base = "mailbox/";
-    memcpy(cpaths, paths, sizeof(cpaths));
-    verifyPaths(cpaths, base);
-    loadPixmap(cpaths, base, "mail.xpm", &mailPixmap);
-    loadPixmap(cpaths, base, "nomail.xpm", &noMailPixmap);
-    loadPixmap(cpaths, base, "errmail.xpm", &errMailPixmap);
-    loadPixmap(cpaths, base, "unreadmail.xpm", &unreadMailPixmap);
-    loadPixmap(cpaths, base, "newmail.xpm", &newMailPixmap);
+    subdirs.init(paths, base);
+    mailPixmap = subdirs.loadPixmap(base, "mail.xpm");
+    noMailPixmap = subdirs.loadPixmap(base, "nomail.xpm");
+    errMailPixmap = subdirs.loadPixmap(base, "errmail.xpm");
+    unreadMailPixmap = subdirs.loadPixmap(base, "unreadmail.xpm");
+    newMailPixmap = subdirs.loadPixmap(base, "newmail.xpm");
 #endif
 
 #ifdef CONFIG_APPLET_CLOCK
     base = "ledclock/";
-    memcpy(cpaths, paths, sizeof(cpaths));
-    verifyPaths(cpaths, base);
-    loadPixmap(cpaths, base, "n0.xpm", &PixNum[0]);
-    loadPixmap(cpaths, base, "n1.xpm", &PixNum[1]);
-    loadPixmap(cpaths, base, "n2.xpm", &PixNum[2]);
-    loadPixmap(cpaths, base, "n3.xpm", &PixNum[3]);
-    loadPixmap(cpaths, base, "n4.xpm", &PixNum[4]);
-    loadPixmap(cpaths, base, "n5.xpm", &PixNum[5]);
-    loadPixmap(cpaths, base, "n6.xpm", &PixNum[6]);
-    loadPixmap(cpaths, base, "n7.xpm", &PixNum[7]);
-    loadPixmap(cpaths, base, "n8.xpm", &PixNum[8]);
-    loadPixmap(cpaths, base, "n9.xpm", &PixNum[9]);
-    loadPixmap(cpaths, base, "space.xpm", &PixSpace);
-    loadPixmap(cpaths, base, "colon.xpm", &PixColon);
-    loadPixmap(cpaths, base, "slash.xpm", &PixSlash);
-    loadPixmap(cpaths, base, "dot.xpm", &PixDot);
-    loadPixmap(cpaths, base, "a.xpm", &PixA);
-    loadPixmap(cpaths, base, "p.xpm", &PixP);
-    loadPixmap(cpaths, base, "m.xpm", &PixM);
+    subdirs.init(paths, base);
+    PixNum[0] = subdirs.loadPixmap(base, "n0.xpm");
+    PixNum[1] = subdirs.loadPixmap(base, "n1.xpm");
+    PixNum[2] = subdirs.loadPixmap(base, "n2.xpm");
+    PixNum[3] = subdirs.loadPixmap(base, "n3.xpm");
+    PixNum[4] = subdirs.loadPixmap(base, "n4.xpm");
+    PixNum[5] = subdirs.loadPixmap(base, "n5.xpm");
+    PixNum[6] = subdirs.loadPixmap(base, "n6.xpm");
+    PixNum[7] = subdirs.loadPixmap(base, "n7.xpm");
+    PixNum[8] = subdirs.loadPixmap(base, "n8.xpm");
+    PixNum[9] = subdirs.loadPixmap(base, "n9.xpm");
+    PixSpace = subdirs.loadPixmap(base, "space.xpm");
+    PixColon = subdirs.loadPixmap(base, "colon.xpm");
+    PixSlash = subdirs.loadPixmap(base, "slash.xpm");
+    PixDot = subdirs.loadPixmap(base, "dot.xpm");
+    PixA = subdirs.loadPixmap(base, "a.xpm");
+    PixP = subdirs.loadPixmap(base, "p.xpm");
+    PixM = subdirs.loadPixmap(base, "m.xpm");
 #endif
 }
 
 TaskBar::TaskBar(YWindow *aParent):
 #if 1
-YFrameClient(aParent, 0)
+    YFrameClient(aParent, 0) INIT_GRADIENT(fGradient, NULL)
 #else
-YWindow(aParent)
+    YWindow(aParent) INIT_GRADIENT(fGradient, NULL)
 #endif
 {
     unsigned int ht = 26;
@@ -204,21 +216,21 @@ YWindow(aParent)
     taskBarMenu = new YMenu();
     if (taskBarMenu) {
         taskBarMenu->setActionListener(this);
-        taskBarMenu->addItem(_("Tile _Vertically"), -2, "", actionTileVertical);
-        taskBarMenu->addItem(_("T_ile Horizontally"), -2, "", actionTileHorizontal);
-        taskBarMenu->addItem(_("Ca_scade"), -2, "", actionCascade);
-        taskBarMenu->addItem(_("_Arrange"), -2, "", actionArrange);
-        taskBarMenu->addItem(_("_Minimize All"), -2, "", actionMinimizeAll);
-        taskBarMenu->addItem(_("_Hide All"), -2, "", actionHideAll);
-        taskBarMenu->addItem(_("_Undo"), -2, "", actionUndoArrange);
+        taskBarMenu->addItem(_("Tile _Vertically"), -2, KEY_NAME(gKeySysTileVertical), actionTileVertical);
+        taskBarMenu->addItem(_("T_ile Horizontally"), -2, KEY_NAME(gKeySysTileHorizontal), actionTileHorizontal);
+        taskBarMenu->addItem(_("Ca_scade"), -2, KEY_NAME(gKeySysCascade), actionCascade);
+        taskBarMenu->addItem(_("_Arrange"), -2, KEY_NAME(gKeySysArrange), actionArrange);
+        taskBarMenu->addItem(_("_Minimize All"), -2, KEY_NAME(gKeySysMinimizeAll), actionMinimizeAll);
+        taskBarMenu->addItem(_("_Hide All"), -2, KEY_NAME(gKeySysHideAll), actionHideAll);
+        taskBarMenu->addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
         if (minimizeToDesktop)
-            taskBarMenu->addItem(_("Arrange _Icons"), -2, "", actionArrangeIcons)->setEnabled(false);
+            taskBarMenu->addItem(_("Arrange _Icons"), -2, KEY_NAME(gKeySysArrangeIcons), actionArrangeIcons)->setEnabled(false);
         taskBarMenu->addSeparator();
 #ifdef CONFIG_WINMENU
         taskBarMenu->addItem(_("_Windows"), -2, actionWindowList, windowListMenu);
 #endif
         taskBarMenu->addSeparator();
-        taskBarMenu->addItem(_("_Refresh"), -2, "", actionRefresh);
+        taskBarMenu->addItem(_("_Refresh"), -2, 0, actionRefresh);
 
 #ifndef LITE
 #if 0
@@ -232,8 +244,10 @@ YWindow(aParent)
 
         taskBarMenu->addItem(_("_About"), -2, actionAbout, 0);
 #endif
-        taskBarMenu->addSeparator();
-        taskBarMenu->addItem(_("_Logout..."), -2, actionLogout, logoutMenu);
+	if (logoutMenu) {
+	    taskBarMenu->addSeparator();
+	    taskBarMenu->addItem(_("_Logout..."), -2, actionLogout, logoutMenu);
+        }
     }
 
     fAddressBar = 0;
@@ -241,17 +255,30 @@ YWindow(aParent)
 #ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
     if (taskBarShowCPUStatus)
-        fCPUStatus = new CPUStatus(cpuCommand, this);
+        fCPUStatus = new CPUStatus(this);
     else
         fCPUStatus = 0;
 #endif
 #endif
 
 #ifdef HAVE_NET_STATUS
-    if (taskBarShowNetStatus && netDevice)
-        fNetStatus = new NetStatus(netCommand, this);
-    else
-        fNetStatus = 0;
+    fNetStatus = 0;
+
+    if (taskBarShowNetStatus && netDevice) {
+	unsigned cnt(strTokens(netDevice));
+
+	if (cnt) {
+	    fNetStatus = new NetStatus*[cnt + 1];
+	    fNetStatus[cnt--] = NULL;
+
+	    for (char const * s(netDevice + strspn(netDevice, " \t"));
+		 *s != '\0'; s = strnxt(s)) {
+		 char const * netdev(newstr(s, " \t"));
+		 fNetStatus[cnt--] = new NetStatus(netdev, this);
+		 delete[] netdev;
+	     }
+	}
+    }
 #endif
 
     if (taskBarShowClock) {
@@ -268,16 +295,45 @@ YWindow(aParent)
 #endif
 
 #ifdef CONFIG_APPLET_MAILBOX
-    if (taskBarShowMailboxStatus)
-        fMailBoxStatus = new MailBoxStatus(mailBoxPath, mailCommand, this);
-    else
-        fMailBoxStatus = 0;
+    fMailBoxStatus = 0;
+
+    if (taskBarShowMailboxStatus) {
+	char const * mailboxes(mailBoxPath ? mailBoxPath : getenv("MAIL"));
+	unsigned cnt(strTokens(mailboxes));
+	
+	if (cnt) {
+	    fMailBoxStatus = new MailBoxStatus*[cnt + 1];
+            fMailBoxStatus[cnt--] = NULL;
+
+	    for (char const * s(mailboxes + strspn(mailboxes, " \t"));
+		 *s != '\0'; s = strnxt(s)) {
+		char * mailbox(newstr(s, " \t"));
+		fMailBoxStatus[cnt--] = new MailBoxStatus(mailbox, this);
+		delete[] mailbox;
+	    }
+	} else if (getenv("MAIL")) {
+	    fMailBoxStatus = new MailBoxStatus*[2];
+            fMailBoxStatus[0] = new MailBoxStatus(getenv("MAIL"), this);
+            fMailBoxStatus[1] = NULL;
+	} else if (getlogin()) {
+	    char * mbox = strJoin("/var/spool/mail/", getlogin(), NULL);
+
+	    if (!access(mbox, R_OK)) {
+		fMailBoxStatus = new MailBoxStatus*[2];
+		fMailBoxStatus[0] = new MailBoxStatus(mbox, this);
+		fMailBoxStatus[1] = NULL;
+	    }
+
+	    delete[] mbox;
+	}
+    }
 #endif
 #ifndef NO_CONFIGURE_MENUS
     if (taskBarShowStartMenu) {
-        fApplications = new YButton(this, 0, rootMenu);
+        fApplications = new ObjectButton(this, rootMenu);
         fApplications->setActionListener(this);
-        fApplications->setPixmap(startPixmap);
+        fApplications->setImage(icewmImage);
+	fApplications->setToolTip(_("Favorite applications"));
         if (fApplications->height() + ADD1 > ht)
             ht = fApplications->height() + ADD1;
     } else
@@ -295,9 +351,10 @@ YWindow(aParent)
 
 #ifdef CONFIG_WINMENU
     if (taskBarShowWindowListMenu) {
-        fWinList = new YButton(this, 0, windowListMenu);
-        fWinList->setPixmap(windowsPixmap);
+        fWinList = new ObjectButton(this, windowListMenu);
+        fWinList->setImage(windowsImage);
         fWinList->setActionListener(this);
+	fWinList->setToolTip(_("Window list menu"));
         if (fWinList->height() + ADD1 > ht) ht = fWinList->height() + ADD1;
     } else
         fWinList = 0;
@@ -326,18 +383,21 @@ YWindow(aParent)
 #ifdef CONFIG_APPLET_APM
         if (fApm) {
             rightX -= 2;
-            fApm->setPosition(rightX - fApm->width(), BASE1 + (ht - ADD1 - fApm->height()) / 2);
+            fApm->setPosition(rightX - fApm->width(),
+                              BASE1 + (ht - ADD1 - fApm->height()) / 2);
             fApm->show();
             rightX -= fApm->width() + 2;
         }
 #endif
 #ifdef CONFIG_APPLET_MAILBOX
-        if (fMailBoxStatus) {
-            fMailBoxStatus->setPosition(rightX - fMailBoxStatus->width() - 1,
-                                        BASE2 + (ht - ADD2 - fMailBoxStatus->height()) / 2);
-            fMailBoxStatus->show();
-            rightX -= fMailBoxStatus->width() + 2;
-        }
+        if (fMailBoxStatus)
+	    for (MailBoxStatus ** mbox(fMailBoxStatus); *mbox; ++mbox) {
+		(*mbox)->setPosition(rightX - (*mbox)->width() - 1,
+                                  BASE2 + (ht - ADD2 - (*mbox)->height()) / 2);
+
+		(*mbox)->show();
+		rightX -= (*mbox)->width() + 2;
+	    }
 #endif
 #ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
@@ -351,13 +411,16 @@ YWindow(aParent)
 #endif
 
 #ifdef HAVE_NET_STATUS
-        if (fNetStatus) {
-            rightX -= 2;
-            fNetStatus->setPosition(rightX - fNetStatus->width() - 1,
-                        BASE1 + (ht - ADD1 - fNetStatus->height()) / 2);
-            // don't do a show() here because PPPStatus takes care of it
-            rightX -= fNetStatus->width() + 2;
-        }
+        if (fNetStatus)
+	    for (NetStatus ** netstat(fNetStatus); *netstat; ++netstat) {
+		rightX -= 2;
+
+		(*netstat)->setPosition(rightX - (*netstat)->width() - 1,
+				  BASE1 + (ht - ADD1 - (*netstat)->height()) / 2);
+
+		// don't do a show() here because PPPStatus takes care of it
+		rightX -= (*netstat)->width() + 2;
+	    }
 #endif
 
         if (fApplications) {
@@ -424,12 +487,14 @@ YWindow(aParent)
         }
 #endif
 #ifdef CONFIG_APPLET_MAILBOX
-        if (fMailBoxStatus) {
-            fMailBoxStatus->setPosition(rightX - fMailBoxStatus->width() - 1,
-                                        BASE2 + (ht - ADD2 - fMailBoxStatus->height()) / 2);
-            fMailBoxStatus->show();
-            rightX -= fMailBoxStatus->width() + 2;
-        }
+        if (fMailBoxStatus)
+	    for (MailBoxStatus ** mbox(fMailBoxStatus); *mbox; ++mbox) {
+		(*mbox)->setPosition(rightX - (*mbox)->width() - 1,
+				  BASE2 + (ht - ADD2 - (*mbox)->height()) / 2);
+
+		(*mbox)->show();
+		rightX -= (*mbox)->width() + 2;
+            }
 #endif
 #ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
@@ -442,13 +507,16 @@ YWindow(aParent)
 #endif
 #endif
 #ifdef HAVE_NET_STATUS
-        if (fNetStatus) {
-            rightX -= 2;
-            fNetStatus->setPosition(rightX - fNetStatus->width() - 1,
-                        BASE1 + (ht - ADD1 - fNetStatus->height()) / 2);
-            // don't do a show() here because PPPStatus takes care of it
-            rightX -= fNetStatus->width() + 2;
-        }
+        if (fNetStatus)
+	    for (NetStatus ** netstat(fNetStatus); *netstat; ++netstat) {
+		rightX -= 2;
+
+		(*netstat)->setPosition(rightX - (*netstat)->width() - 1,
+				  BASE1 + (ht - ADD1 - (*netstat)->height()) / 2);
+
+		// don't do a show() here because PPPStatus takes care of it
+		rightX -= (*netstat)->width() + 2;
+            }
 #endif
 #ifdef CONFIG_APPLET_APM
         if (fApm) {
@@ -490,18 +558,47 @@ YWindow(aParent)
         leftX += 2;
     }
 
+#ifdef CONFIG_TRAY
+    if (taskBarShowTray) {
+        fTray = new TrayPane(this);
+
+        if (fTray) {
+            int trayWidth(fTray->getRequiredWidth());
+            int w((rightX - leftX ) / 2);
+            if (trayWidth > w)
+		trayWidth = w;
+	    else
+		w = trayWidth;
+
+            rightX-= w;
+
+            int h((int) height() - ADD2 - ((wmLook == lookMetal) ? 0 : 1));
+            int y(BASE2 + ((int) height() - ADD2 - 1 - h) / 2);
+
+            if (taskBarDoubleHeight) {
+                h = h / 2 - 1;
+                y = 3 * height() / 4 - h / 2;
+            } else if (trayDrawBevel)
+		rightX-= 2;
+
+            fTray->setGeometry(rightX, y, w, h);
+            fTray->show();
+            rightX -= 2;
+        }
+    } else
+	fTray = 0;
+
+#endif
     if (taskBarShowWindows) {
         fTasks = new TaskPane(this);
         if (fTasks) {
-            int w = rightX - leftX;
-            int x = leftX;
-            int h = height() - ADD2 - ((wmLook == lookMetal) ? 0 : 1);
-            int y = BASE2 + (height() - ADD2 - 1 - h) / 2;
+            int h((int) height() - ADD2 - ((wmLook == lookMetal) ? 0 : 1));
+            int y(BASE2 + ((int) height() - ADD2 - 1 - h) / 2);
             if (taskBarDoubleHeight) {
-                h /= 2; h--;
-                y += height() / 2; y--;
+                h = h / 2 - 1;
+                y += height() / 2 - 1;
             }
-            fTasks->setGeometry(x, y, w, h);
+            fTasks->setGeometry(leftX, y, rightX - leftX, h);
             fTasks->show();
         }
     } else {
@@ -535,7 +632,8 @@ TaskBar::~TaskBar() {
     delete fClock; fClock = 0;
 #endif
 #ifdef CONFIG_APPLET_MAILBOX
-    delete fMailBoxStatus; fMailBoxStatus = 0;
+    for (MailBoxStatus ** m(fMailBoxStatus); m && *m; ++m) delete *m;
+    delete[] fMailBoxStatus; fMailBoxStatus = 0;
 #endif
     delete fApplications; fApplications = 0;
     delete fWinList; fWinList = 0;
@@ -547,8 +645,15 @@ TaskBar::~TaskBar() {
     delete taskbuttonPixmap;
     delete taskbuttonactivePixmap;
     delete taskbuttonminimizedPixmap;
-    delete startPixmap;
-    delete windowsPixmap;
+#ifdef CONFIG_GRADIENT
+    delete taskbackPixbuf;
+    delete taskbuttonPixbuf;
+    delete taskbuttonactivePixbuf;
+    delete taskbuttonminimizedPixbuf;
+    delete fGradient;
+#endif
+    delete icewmImage;
+    delete windowsImage;
 #ifdef CONFIG_APPLET_MAILBOX
     delete mailPixmap;
     delete noMailPixmap;
@@ -640,8 +745,23 @@ bool TaskBar::handleTimer(YTimer *t) {
 }
 
 void TaskBar::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+#ifdef CONFIG_GRADIENTS
+    if (taskbackPixbuf && !(fGradient &&
+    			    fGradient->width() == width() &&
+			    fGradient->height() == height())) {
+	delete fGradient;
+	fGradient = new YPixbuf(*taskbackPixbuf, width(), height());
+    }
+#endif
+
     g.setColor(taskBarBg);
     //g.draw3DRect(0, 0, width() - 1, height() - 1, true);
+
+#ifdef CONFIG_GRADIENTS
+    if (fGradient)
+        g.copyPixbuf(*fGradient, 0, 0, width(), height(), 0, 0);
+    else 
+#endif    
     if (taskbackPixmap)
         g.fillPixmap(taskbackPixmap, 0, 0, width(), height());
     else
