@@ -26,6 +26,7 @@ class MailBoxStatus;
 class YClock;
 class YApm;
 class TaskPane;
+class TrayPane;
 class WorkspacesPane;
 
 #ifdef CONFIG_TASKBAR
@@ -51,7 +52,7 @@ public:
 
     void updateLocation();
 
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
     YClock *clock() { return fClock; }
 #endif
 
@@ -67,27 +68,37 @@ public:
 
     AddressBar *addressBar() const { return fAddressBar; }
     TaskPane *taskPane() const { return fTasks; }
+#ifdef CONFIG_TRAY
+    TrayPane *trayPane() const { return fTray; }
+#endif
+
+#ifdef CONFIG_GRADIENTS
+    virtual class YPixbuf * getGradient() const { return fGradient; }
+#endif    
 
     void contextMenu(int x_root, int y_root);
 private:
     TaskPane *fTasks;
 
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_TRAY
+    TrayPane *fTray;
+#endif
+#ifdef CONFIG_APPLET_CLOCK
     YClock *fClock;
 #endif
-#ifdef CONFIG_MAILBOX
-    MailBoxStatus *fMailBoxStatus;
+#ifdef CONFIG_APPLET_MAILBOX
+    MailBoxStatus **fMailBoxStatus;
 #endif
-#ifdef CONFIG_CPUSTATUS
+#ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux)||defined(HAVE_KSTAT_H))
     CPUStatus *fCPUStatus;
 #endif
 #endif
-#ifdef CONFIG_APM
+#ifdef CONFIG_APPLET_APM
     YApm *fApm;
 #endif
 #ifdef HAVE_NET_STATUS
-    NetStatus *fNetStatus;
+    NetStatus **fNetStatus;
 #endif
 
 #ifndef NO_CONFIGURE_MENUS
@@ -108,6 +119,10 @@ private:
 
     friend class WindowList;
     friend class WindowListBox;
+    
+#ifdef CONFIG_GRADIENTS
+    class YPixbuf * fGradient;
+#endif    
 };
 
 extern TaskBar *taskBar; // !!! get rid of this
@@ -120,6 +135,15 @@ extern YPixmap *taskbuttonPixmap;
 extern YPixmap *taskbuttonactivePixmap;
 extern YPixmap *taskbuttonminimizedPixmap;
 
+#ifdef CONFIG_GRADIENTS
+class YPixbuf;
+
+extern YPixbuf *taskbackPixbuf;
+extern YPixbuf *taskbuttonPixbuf;
+extern YPixbuf *taskbuttonactivePixbuf;
+extern YPixbuf *taskbuttonminimizedPixbuf;
+#endif
+
 #endif
 
 #endif
