@@ -250,22 +250,26 @@ WindowList::WindowList(YWindow *aParent): YFrameClient(aParent, 0) {
     windowListPopup->addItem(_("_Minimize"), -2, 0, actionMinimize);
     windowListPopup->addSubmenu(_("Move _To"), -2, moveMenu);
     windowListPopup->addSeparator();
-    windowListPopup->addItem(_("Tile _Vertically"), -2, 0, actionTileVertical);
-    windowListPopup->addItem(_("T_ile Horizontally"), -2, 0, actionTileHorizontal);
-    windowListPopup->addItem(_("Ca_scade"), -2, 0, actionCascade);
-    windowListPopup->addItem(_("_Arrange"), -2, 0, actionArrange);
+    windowListPopup->addItem(_("Tile _Vertically"), -2, KEY_NAME(gKeySysTileVertical), actionTileVertical);
+    windowListPopup->addItem(_("T_ile Horizontally"), -2, KEY_NAME(gKeySysTileHorizontal), actionTileHorizontal);
+    windowListPopup->addItem(_("Ca_scade"), -2, KEY_NAME(gKeySysCascade), actionCascade);
+    windowListPopup->addItem(_("_Arrange"), -2, KEY_NAME(gKeySysArrange), actionArrange);
+    windowListPopup->addSeparator();
+    windowListPopup->addItem(_("_Minimize All"), -2, KEY_NAME(gKeySysMinimizeAll), actionMinimizeAll);
+    windowListPopup->addItem(_("_Hide All"), -2, KEY_NAME(gKeySysHideAll), actionHideAll);
+    windowListPopup->addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
     windowListPopup->addSeparator();
     windowListPopup->addItem(_("_Close"), -2, actionClose, closeSubmenu);
 
     windowListAllPopup = new YMenu();
     windowListAllPopup->setActionListener(wmapp);
-    windowListAllPopup->addItem(_("Tile _Vertically"), -2, 0, actionTileVertical);
-    windowListAllPopup->addItem(_("T_ile Horizontally"), -2, 0, actionTileHorizontal);
-    windowListAllPopup->addItem(_("Ca_scade"), -2, 0, actionCascade);
-    windowListAllPopup->addItem(_("_Arrange"), -2, 0, actionArrange);
-    windowListAllPopup->addItem(_("_Minimize All"), -2, "", actionMinimizeAll);
-    windowListAllPopup->addItem(_("_Hide All"), -2, "", actionHideAll);
-    windowListAllPopup->addItem(_("_Undo"), -2, "", actionUndoArrange);
+    windowListAllPopup->addItem(_("Tile _Vertically"), -2, KEY_NAME(gKeySysTileVertical), actionTileVertical);
+    windowListAllPopup->addItem(_("T_ile Horizontally"), -2, KEY_NAME(gKeySysTileHorizontal), actionTileHorizontal);
+    windowListAllPopup->addItem(_("Ca_scade"), -2, KEY_NAME(gKeySysCascade), actionCascade);
+    windowListAllPopup->addItem(_("_Arrange"), -2, KEY_NAME(gKeySysArrange), actionArrange);
+    windowListAllPopup->addItem(_("_Minimize All"), -2, KEY_NAME(gKeySysMinimizeAll), actionMinimizeAll);
+    windowListAllPopup->addItem(_("_Hide All"), -2, KEY_NAME(gKeySysHideAll), actionHideAll);
+    windowListAllPopup->addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
 
     int w = desktop->width();
     int h = desktop->height();
@@ -320,9 +324,11 @@ void WindowList::removeWindowListApp(WindowListItem *item) {
     }
 }
 
-void WindowList::configure(int x, int y, unsigned int width, unsigned int height) {
-    YFrameClient::configure(x, y, width, height);
-    scroll->setGeometry(0, 0, width, height);
+void WindowList::configure(const int x, const int y, 
+			   const unsigned width, const unsigned height, 
+			   const bool resized) {
+    YFrameClient::configure(x, y, width, height, resized);
+    if (resized) scroll->setGeometry(0, 0, width, height);
 }
 
 void WindowList::handleClose() {
