@@ -13,10 +13,11 @@
 #include "wmmgr.h"
 #include "MwmUtil.h"
 #include "prefs.h"
+#include "ypixbuf.h"
 
 #include "intl.h"
 
-#ifdef SM
+#ifdef CONFIG_SESSION
 #include <X11/SM/SMlib.h>
 #endif
 
@@ -45,6 +46,9 @@ Atom _XA_WIN_WORKSPACE;
 Atom _XA_WIN_WORKSPACE_COUNT;
 Atom _XA_WIN_WORKSPACE_NAMES;
 Atom _XA_WIN_WORKAREA;
+#ifdef CONFIG_TRAY
+Atom _XA_WIN_TRAY;
+#endif
 Atom _XA_WIN_ICONS;
 Atom _XA_WIN_STATE;
 Atom _XA_WIN_LAYER;
@@ -65,18 +69,38 @@ Atom XA_XdndStatus;
 Atom XA_XdndDrop;
 Atom XA_XdndFinished;
 
-Colormap defaultColormap;
-
 YCursor YApplication::leftPointer;
 YCursor YApplication::rightPointer;
 YCursor YApplication::movePointer;
 
-YColor *YColor::black = 0;
-YColor *YColor::white = 0;
+YColor *YColor::black(NULL);
+YColor *YColor::white(NULL);
 
-YPixmap *menubackPixmap = 0;
-YPixmap *switchbackPixmap = 0;
-YPixmap *logoutPixmap = 0;
+YPixmap *buttonIPixmap(NULL);
+YPixmap *buttonAPixmap(NULL);
+
+YPixmap *logoutPixmap(NULL);
+YPixmap *switchbackPixmap(NULL);
+YPixmap *listbackPixmap(NULL);
+YPixmap *dialogbackPixmap(NULL);
+
+YPixmap *menubackPixmap(NULL);
+YPixmap *menusepPixmap(NULL);
+YPixmap *menuselPixmap(NULL);
+
+#ifdef CONFIG_GRADIENTS
+YPixbuf *buttonIPixbuf(NULL);
+YPixbuf *buttonAPixbuf(NULL);
+
+YPixbuf *logoutPixbuf(NULL);
+YPixbuf *switchbackPixbuf(NULL);
+YPixbuf *listbackPixbuf(NULL);
+YPixbuf *dialogbackPixbuf(NULL);
+
+YPixbuf *menubackPixbuf(NULL);
+YPixbuf *menuselPixbuf(NULL);
+YPixbuf *menusepPixbuf(NULL);
+#endif
 
 YPixmap *closePixmap[2] = { 0, 0 };
 YPixmap *minimizePixmap[2] = { 0, 0 };
@@ -89,20 +113,21 @@ YPixmap *depthPixmap[2] = { 0, 0 };
 
 YIcon *defaultAppIcon = 0;
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
 int shapesSupported;
 int shapeEventBase, shapeErrorBase;
 #endif
 
 int xeventcount = 0;
 
-#ifdef SM
+#ifdef CONFIG_SESSION
 int IceSMfd = -1;
 IceConn IceSMconn = NULL;
 SmcConn SMconn = NULL;
 char *oldSessionId = NULL;
 char *newSessionId = NULL;
 char *sessionProg;
+
 char *getsesfile() {
     static char name[1024] = "";
 
@@ -116,10 +141,10 @@ char *getsesfile() {
     return name;
 }
 
-void iceWatchFD(IceConn conn,
-                IcePointer /*client_data*/,
-                Bool opening,
-                IcePointer */*watch_data*/)
+static void iceWatchFD(IceConn conn,
+                       IcePointer /*client_data*/,
+                       Bool opening,
+                       IcePointer */*watch_data*/)
 {
     if (opening) {
         if (IceSMfd != -1) { // shouldn't happen
@@ -300,7 +325,7 @@ void YApplication::smRequestShutdown() {
                            True);
 }
 
-#endif
+#endif /* CONFIG_SESSION */
 
 class YClipboard: public YWindow {
 public:
@@ -392,6 +417,8 @@ void initSignals() {
     if (pipe(signalPipe) != 0)
         die(2, _("Pipe creation failed (errno=%d)."), errno);
     fcntl(signalPipe[1], F_SETFL, O_NONBLOCK);
+    fcntl(signalPipe[0], F_SETFD, FD_CLOEXEC);
+    fcntl(signalPipe[1], F_SETFD, FD_CLOEXEC);
 }
 
 static void initAtoms() {
@@ -415,6 +442,9 @@ static void initAtoms() {
         { &_XA_WIN_WORKAREA, XA_WIN_WORKAREA },
         { &_XA_WIN_ICONS, XA_WIN_ICONS },
         { &_XA_WIN_LAYER, XA_WIN_LAYER },
+#ifdef CONFIG_TRAY
+        { &_XA_WIN_TRAY, XA_WIN_TRAY },
+#endif
         { &_XA_WIN_STATE, XA_WIN_STATE },
         { &_XA_WIN_HINTS, XA_WIN_HINTS },
         { &_XA_WIN_PROTOCOLS, XA_WIN_PROTOCOLS },
@@ -464,54 +494,11 @@ static void initColors() {
     YColor::white = new YColor("rgb:FF/FF/FF");
 }
 
-char *joinPath(pathelem *pe, const char *base, const char *name) {
-    const char *b = base ? base : "";
-
-    if (pe->sub)
-        return strJoin(*pe->root, pe->rdir, *pe->sub, "/", b, name, NULL);
-    else
-        return strJoin(*pe->root, pe->rdir, b, name, NULL);
-}
-
-void verifyPaths(pathelem *search, const char *base) {
-    unsigned int j = 0, i = 0;
-
-    for (; search[i].root; i++) {
-        char *path = joinPath(search + i, base, "");
-        if (access(path, R_OK) == 0)
-            search[j++] = search[i];
-        delete path;
-    }
-    search[j] = search[i];
-}
-
 #ifndef LITE
-pathelem icon_paths[10];
+YResourcePaths YApplication::iconPaths;
 
 void initIcons() {
-    static const char *home = getenv("HOME");
-    static char themeSubdir[256];
-    const char *base = 0;
-
-    strcpy(themeSubdir, themeName);
-    { char *p = strchr(themeSubdir, '/'); if (p) *p = 0; }
-
-    static const char *themeDir = themeSubdir;
-
-    pathelem i_paths[10] = {
-        { &home, "/.icewm/themes/", &themeDir },
-        { &home, "/.icewm/", 0,},
-        { &configDir, "/themes/", &themeDir },
-        { &configDir, "/", 0 },
-        { &libDir, "/themes/", &themeDir },
-        { &libDir, "/", 0 },
-        { 0, 0, 0 }
-    };
-
-    base = "icons/";
-    verifyPaths(i_paths, base);
-    memcpy(icon_paths, i_paths, sizeof(icon_paths));
-
+    YApplication::iconPaths.init("icons/");
     defaultAppIcon = getIcon("app");
 }
 #endif
@@ -554,13 +541,25 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
     fClip = 0;
     fReplayEvent = false;
 
-    bool sync = false;
+    {
+	char const * cmd(**argv);
+	char cwd[PATH_MAX + 1];
+
+	if ('/' == *cmd)
+	    fExecutable = newstr(cmd);
+	else if (strchr (cmd, '/'))
+	    fExecutable = strJoin(getcwd(cwd, sizeof(cwd)), "/", cmd, NULL);
+	else
+	    fExecutable = findPath(getenv("PATH"), X_OK, cmd);
+    }
+
+    bool sync(false);
 
     for (int i = 1; i < *argc; i++) {
         if ((*argv)[i][0] == '-') {
             if (strcmp((*argv)[i], "-display") == 0) {
                 displayName = (*argv)[++i];
-#ifdef SM
+#ifdef CONFIG_SESSION
             } else if (strcmp((*argv)[i], "-clientId") == 0) {
                 oldSessionId = (*argv)[++i];
 #endif
@@ -569,7 +568,7 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
             }
         }
     }
-
+    
     if (displayName == 0)
         displayName = getenv("DISPLAY");
     else {
@@ -579,18 +578,25 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
     }
 
     if (!(fDisplay = XOpenDisplay(displayName)))
-        die(1, _("Can't open display: %s. "
-                 "X must be running and $DISPLAY set."),
-            displayName ? displayName : _("<none>"));
+        die(1, _("Can't open display: %s. X must be running and $DISPLAY set."),
+	         displayName ? displayName : _("<none>"));
 
     if (sync)
         XSynchronize(display(), True);
 
-    defaultColormap = DefaultColormap(display(), DefaultScreen(display()));
+#if CONFIG_XFREETYPE
+    int renderEvents, renderErrors;
+
+    haveXft&= (XRenderQueryExtension(display(), &renderEvents, &renderErrors) &&
+	       XftDefaultHasRender(display()));
+
+    MSG(("RENDER extension: %d", haveXft));
+#endif
 
     windowContext = XUniqueContext();
 
     new YDesktop(0, RootWindow(display(), DefaultScreen(display())));
+    YPixbuf::init();
 
     initSignals();
     initAtoms();
@@ -598,7 +604,7 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
     initPointers();
     initColors();
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     shapesSupported = XShapeQueryExtension(display(),
                                            &shapeEventBase, &shapeErrorBase);
 #endif
@@ -607,7 +613,7 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
     initIcons();
 #endif
 
-#ifdef SM
+#ifdef CONFIG_SESSION
     sessionProg = (*argv)[0]; //ICEWMEXE;
     initSM();
 #endif
@@ -622,7 +628,7 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
 }
 
 YApplication::~YApplication() {
-#ifdef SM
+#ifdef CONFIG_SESSION
     if (SMconn != 0) {
         SmcCloseConnection(SMconn, 0, NULL);
         SMconn = NULL;
@@ -630,12 +636,22 @@ YApplication::~YApplication() {
     }
 #endif
     XCloseDisplay(display());
-    app = 0;
+    delete[] fExecutable;
+
+    app = NULL;
 }
 
 bool YApplication::hasColormap() {
-    if (DefaultVisual(display(), DefaultScreen(display()))->c_class & 1 )
-        return true;
+    XVisualInfo pattern;
+    pattern.screen = DefaultScreen(display());
+
+    int nVisuals;
+    XVisualInfo * visual(XGetVisualInfo(display(), VisualScreenMask,
+    					&pattern, &nVisuals));
+
+    while(visual && nVisuals--)
+	if ((visual++)->c_class & 1) return true;
+
     return false;
 }
 
@@ -831,7 +847,7 @@ int YApplication::mainLoop() {
             FD_SET(ConnectionNumber(app->display()), &read_fds);
             if (signalPipe[0] != -1)
                 FD_SET(signalPipe[0], &read_fds);
-#ifdef SM
+#ifdef CONFIG_SESSION
             if (IceSMfd != -1)
                 FD_SET(IceSMfd, &read_fds);
 #endif
@@ -903,7 +919,7 @@ int YApplication::mainLoop() {
                     }
                 }
             }
-#ifdef SM
+#ifdef CONFIG_SESSION
             if (IceSMfd != -1 && FD_ISSET(IceSMfd, &read_fds)) {
                 Bool rep;
                 if (IceProcessMessages(IceSMconn, NULL, &rep)
@@ -1198,7 +1214,9 @@ static KeyCode sym2code(KeySym k) {
 
 void YApplication::initModifiers() {
     XModifierKeymap *xmk = XGetModifierMapping(app->display());
-    AltMask = MetaMask = NumLockMask = ScrollLockMask = SuperMask = HyperMask = 0;
+    AltMask = MetaMask = WinMask = SuperMask = HyperMask = 
+	NumLockMask = ScrollLockMask = ModeSwitchMask = 0;
+    
     if (xmk) {
         KeyCode numLockKeyCode = sym2code(XK_Num_Lock);
         KeyCode scrollLockKeyCode = sym2code(XK_Scroll_Lock);
@@ -1246,8 +1264,9 @@ void YApplication::initModifiers() {
     if (MetaMask == AltMask)
         MetaMask = 0;
 
-    MSG(("alt:%d meta:%d num:%d scroll:%d",
-         AltMask, MetaMask, NumLockMask, ScrollLockMask));
+    MSG(("alt:%d meta:%d super:%d hyper:%d mode:%d num:%d scroll:%d",
+         AltMask, MetaMask, SuperMask, HyperMask, ModeSwitchMask,
+	 NumLockMask, ScrollLockMask));
 
     // some hacks for "broken" modifier configurations
 
@@ -1292,10 +1311,47 @@ void YApplication::initModifiers() {
         Button5Mask;
 
     ButtonKeyMask = KeyMask | ButtonMask;
+
+#if 0
+    KeySym wl = XKeycodeToKeysym(app->display(), 115, 0);
+    KeySym wr = XKeycodeToKeysym(app->display(), 116, 0);
+
+    if (wl == XK_Super_L) {
+    } else if (wl == XK_Meta_L) {
+    }
+#endif
+    // this will do for now, but we should actualy check the keycodes
+    Win_L = Win_R = 0;
+
+    if (SuperMask != 0) {
+        WinMask = SuperMask;
+	
+	if (win95keys) {
+	    Win_L = XK_Super_L;
+	    Win_R = XK_Super_R;
+	}
+    } else if (MetaMask != 0) {
+        WinMask = MetaMask;
+	
+	if (win95keys) {
+            Win_L = XK_Meta_L;
+            Win_R = XK_Meta_R;
+	}
+    }
+
+    MSG(("alt:%d meta:%d super:%d hyper:%d win:%d mode:%d num:%d scroll:%d",
+         AltMask, MetaMask, SuperMask, HyperMask, WinMask, ModeSwitchMask,
+	 NumLockMask, ScrollLockMask));
+
+}
+
+void YApplication::alert() {
+    XBell(display(), 100);
 }
 
 void YApplication::runProgram(const char *str, const char *const *args) {
     XSync(app->display(), False);
+
     if (fork() == 0) {
         app->resetSignals();
         sigemptyset(&signalMask);
@@ -1319,15 +1375,9 @@ void YApplication::runProgram(const char *str, const char *const *args) {
     }
 }
 
-void YApplication::runCommand(const char *prog) {
-    const char *cmd[4];
-
-    cmd[0] = "/bin/sh";
-    cmd[1] = "-c";
-    cmd[2] = prog;
-    cmd[3] = 0;
-
-    runProgram(cmd[0], cmd);
+void YApplication::runCommand(const char *cmdline) {
+    char const * argv[] = { "/bin/sh", "-c", cmdline, NULL };
+    runProgram(argv[0], argv);
 }
 
 void YApplication::setClipboardText(char *data, int len) {
@@ -1338,16 +1388,21 @@ void YApplication::setClipboardText(char *data, int len) {
     fClip->setData(data, len);
 }
 
-bool YApplication::detectGNOME() {
-    Atom GNOME_NAME_SERVER(XInternAtom(display(), "GNOME_NAME_SERVER", true));
+bool YApplication::hasGNOME() {
+#ifdef CONFIG_GNOME_ROOT_PROPERTY
+    Atom GNOME_ROOT_PROPERTY(XInternAtom(display(),
+    			     CONFIG_GNOME_ROOT_PROPERTY, true));
 
     Atom r_type; int r_format;
     unsigned long count, bytes_remain;
     Window gnomeName;
 
-    return (GNOME_NAME_SERVER != None &&
+    return (GNOME_ROOT_PROPERTY != None &&
 	    XGetWindowProperty(display(), desktop->handle(),
-			       GNOME_NAME_SERVER, 0, 1, false, XA_WINDOW,
+			       GNOME_ROOT_PROPERTY, 0, 1, false, XA_WINDOW,
 			       &r_type, &r_format, &count, &bytes_remain,
 			       (unsigned char **) &gnomeName) == Success);
+#else // this also detects xsm as GNOME, how to detect KDE?
+    return getenv("SESSION_MANAGER");
+#endif
 }
