@@ -12,16 +12,16 @@ public:
     YButton(YWindow *parent, YAction *action, YMenu *popup = 0);
     virtual ~YButton();
 
-    virtual void paint(Graphics &g, int x, int y, unsigned int width, unsigned int height);
-    virtual void paintFocus(Graphics &g, int x, int y, unsigned int w, unsigned int h);
+    virtual void paint(Graphics &g, int x, int y, unsigned w, unsigned h);
+    virtual void paintFocus(Graphics &g, int x, int y, unsigned w, unsigned h);
     virtual bool handleKey(const XKeyEvent &key);
     virtual void handleButton(const XButtonEvent &button);
     virtual void handleCrossing(const XCrossingEvent &crossing);
 
-    void setAction(YAction *action);
-    void setPopup(YMenu *popup);
-    void setPixmap(YPixmap *pixmap);
-    void setText(const char *str, int hot = -1);
+    void setAction(YAction * action);
+    void setPopup(YMenu * popup);
+    void setImage(YIcon::Image * image);
+    void setText(const char * str, int hot = -1);
 
     void setPressed(int pressed);
     virtual bool isFocusTraversable();
@@ -42,11 +42,17 @@ public:
     bool isPopupActive() const { return fPopupActive; }
 
     virtual void actionPerformed(YAction *action, unsigned int modifiers);
+    virtual YFont * getFont();
+    virtual YColor * getColor();
+    virtual YSurface getSurface();
     
 private:
+    void paint(Graphics &g, int const d, int const x, int const y,
+    			    unsigned const w, unsigned const h);
+
     YAction *fAction;
     YMenu *fPopup;
-    YPixmap *fPixmap;
+    YIcon::Image *fImage;
     char *fText;
     int fPressed;
     int fHotCharPos;
@@ -72,4 +78,12 @@ private:
     static YFont *activeButtonFont;
 };
 
+extern YPixmap *buttonIPixmap;
+extern YPixmap *buttonAPixmap;
+
+#ifdef CONFIG_GRADIENTS
+extern class YPixbuf *buttonIPixbuf;
+extern class YPixbuf *buttonAPixbuf;
+#endif
+
 #endif
