@@ -9,9 +9,9 @@
 
 #ifndef LITE
 
+#include "ypixbuf.h"
 #include "ykey.h"
 #include "ylistbox.h"
-#include "ymenu.h" //!!! pixmaps
 
 #include "yscrollview.h"
 
@@ -68,7 +68,8 @@ int YListItem::getOffset() {
     return 0;
 }
 
-YListBox::YListBox(YScrollView *view, YWindow *aParent): YWindow(aParent) {
+YListBox::YListBox(YScrollView *view, YWindow *aParent): 
+    YWindow(aParent) INIT_GRADIENT(fGradient, NULL) {
     if (listBoxFont == 0)
         listBoxFont = YFont::getFont(listBoxFontName);
     if (listBoxBg == 0)
@@ -106,6 +107,9 @@ YListBox::YListBox(YScrollView *view, YWindow *aParent): YWindow(aParent) {
 YListBox::~YListBox() {
     fFirst = fLast = 0;
     freeItems();
+#ifdef CONFIG_GRADIENTS
+    delete fGradient;
+#endif
 }
 
 bool YListBox::isFocusTraversable() {
@@ -242,13 +246,7 @@ YListItem *YListBox::getItem(int no) {
 }
 
 int YListBox::getLineHeight() {
-    int lh, fh = listBoxFont->height();
-    if (ICON_SMALL > fh)
-        lh = ICON_SMALL;
-    else
-        lh = fh;
-    lh += 2;
-    return lh;
+    return max((int) YIcon::sizeSmall, (int) listBoxFont->height()) + 2;
 }
 
 void YListBox::ensureVisibility(int item) { //!!! horiz too
@@ -282,11 +280,25 @@ void YListBox::focusVisible() {
     }
 }
 
-void YListBox::configure(int x, int y, unsigned int width, unsigned int height) {
-    YWindow::configure(x, y, width, height);
+void YListBox::configure(const int x, const int y, 
+			 const unsigned width, const unsigned height, 
+			 const bool resized) {
+    YWindow::configure(x, y, width, height, resized);
     //if (fFocusedItem != -1)
     //    paintItem(fFocusedItem);
-    resetScrollBars();
+    if (resized) {
+        resetScrollBars();
+
+#ifdef CONFIG_GRADIENTS
+	if (listbackPixbuf && !(fGradient &&
+				 fGradient->width() == width &&
+				 fGradient->height() == height)) {
+	    delete fGradient;
+	    fGradient = new YPixbuf(*listbackPixbuf, width, height);
+	    repaint();
+	}
+#endif
+    }
 }
 
 bool YListBox::handleKey(const XKeyEvent &key) {
@@ -327,14 +339,27 @@ bool YListBox::handleKey(const XKeyEvent &key) {
             if (getItemCount() > 0)
                 setFocusedItem(getItemCount() - 1, clear, extend, false);
             break;
-        case XK_Up:
+        case XK_Up: {
+	    int const oldFocus(fFocusedItem);
+
+	    focusVisible();
             if (fFocusedItem > 0)
-                setFocusedItem(fFocusedItem - 1, clear, extend, false);
+                setFocusedItem(oldFocus == fFocusedItem ? fFocusedItem - 1
+							: fFocusedItem,
+			       clear, extend, false);
+
             break;
-        case XK_Down:
+	}
+        case XK_Down: {
+	    int const oldFocus(fFocusedItem);
+
+	    focusVisible();
             if (fFocusedItem < getItemCount() - 1)
-                setFocusedItem(fFocusedItem + 1, clear, extend, false);
+                setFocusedItem(oldFocus == fFocusedItem ? fFocusedItem + 1
+							: fFocusedItem,
+			       clear, extend, false);
             break;
+	}
 #if 0
         case XK_Prior:
             fVerticalScroll->setValue(fVerticalScroll->getValue() -
@@ -495,12 +520,10 @@ void YListBox::scroll(YScrollBar *scroll, int delta) {
 
     if (scroll == fVerticalScroll) {
         fOffsetY += delta;
-        focusVisible();
         needRepaint = true;
     }
     if (scroll == fHorizontalScroll) {
         fOffsetX += delta;
-        focusVisible();
         needRepaint = true;
     }
     if (needRepaint)
@@ -511,12 +534,10 @@ void YListBox::move(YScrollBar *scroll, int pos) {
     bool needRepaint = false;
     if (scroll == fVerticalScroll) {
         fOffsetY = pos;
-        focusVisible();
         needRepaint = true;
     }
     if (scroll == fHorizontalScroll) {
         fOffsetX = pos;
-        focusVisible();
         needRepaint = true;
     }
     if (needRepaint)
@@ -537,26 +558,34 @@ void YListBox::paintItem(Graphics &g, int n) {
 
     xpos += a->getOffset();
 
-    bool s = a->getSelected();
+    bool s(a->getSelected());
 
     if (fDragging) {
-        int beg = (fSelectStart < fSelectEnd) ? fSelectStart : fSelectEnd;
-        int end = (fSelectStart < fSelectEnd) ? fSelectEnd : fSelectStart;
+        int const beg(fSelectStart < fSelectEnd ? fSelectStart : fSelectEnd);
+        int const end(fSelectStart < fSelectEnd ? fSelectEnd : fSelectStart);
+
         if (n >= beg && n <= end)
-            if (fSelect)
-                s = true;
-            else
-                s = false;
+            s = fSelect;
     }
 
-    if (s)
+    if (s) {
         g.setColor(listBoxSelBg);
-    else
-        g.setColor(listBoxBg);
-    if (menubackPixmap && !s)
-        g.fillPixmap(menubackPixmap, 0, y - fOffsetY, width(), lh);
-    else
         g.fillRect(0, y - fOffsetY, width(), lh);
+    } else {
+#ifdef CONFIG_GRADIENTS
+        if (fGradient)
+	    g.copyPixbuf(*fGradient, 0, y - fOffsetY, width(), lh,
+	    			     0, y - fOffsetY);
+        else 
+#endif	
+	if (listbackPixmap)
+	    g.fillPixmap(listbackPixmap, 0, y - fOffsetY, width(), lh);
+	else {
+	    g.setColor(listBoxBg);
+	    g.fillRect(0, y - fOffsetY, width(), lh);
+	}
+    }
+
     if (fFocusedItem == n) {
         g.setColor(YColor::black);
         g.setPenStyle(true);
@@ -569,41 +598,45 @@ void YListBox::paintItem(Graphics &g, int n) {
         g.drawRect(0 - fOffsetX, y - fOffsetY, cw - 1, lh - 1);
         g.setPenStyle(false);
     }
+
     YIcon *icon = a->getIcon();
+
     if (icon && icon->small())
-        g.drawPixmap(icon->small(), xpos + x - fOffsetX, y - fOffsetY + 1);
-    if (s)
-        g.setColor(listBoxSelFg);
-    else
-        g.setColor(listBoxFg);
-    g.setFont(listBoxFont);
+        g.drawImage(icon->small(), xpos + x - fOffsetX, y - fOffsetY + 1);
+
     const char *title = a->getText();
+
     if (title) {
+	g.setColor(s ? listBoxSelFg : listBoxFg);
+	g.setFont(listBoxFont);
+
         g.drawChars(title, 0, strlen(title),
                     xpos + x + 20 - fOffsetX, yPos - fOffsetY);
     }
 }
 
 void YListBox::paint(Graphics &g, int /*x*/, int ry, unsigned int /*width*/, unsigned int rheight) {
-    int n = 0, min, max;
-    int lh = getLineHeight();
+    int const lh(getLineHeight());
+    int const min((fOffsetY + ry) / lh);
+    int const max((fOffsetY + ry + rheight) / lh);
 
-    min = (fOffsetY + ry) / lh;
-    max = (fOffsetY + ry + rheight) / lh;
-
-    for (n = min; n <= max; n++)
-        paintItem(g, n);
+    for (int n(min); n <= max; n++) paintItem(g, n);
     resetScrollBars();
 
-
-    unsigned int y = contentHeight();
+    unsigned const y(contentHeight());
 
     if (y < height()) {
-        g.setColor(listBoxBg);
-        if (menubackPixmap)
-            g.fillPixmap(menubackPixmap, 0, y, width(), height() - y);
-        else
+#ifdef CONFIG_GRADIENTS
+        if (fGradient)
+            g.copyPixbuf(*fGradient, 0, y, width(), height() - y, 0, y);
+        else 
+#endif	
+	if (listbackPixmap)
+            g.fillPixmap(listbackPixmap, 0, y, width(), height() - y);
+        else {
+	    g.setColor(listBoxBg);
             g.fillRect(0, y, width(), height() - y);
+	}
     }
 }
 
