@@ -5,6 +5,7 @@
  */
 #include "config.h"
 
+#include "yfull.h"
 #include "ykey.h"
 #include "ymenu.h"
 #include "yaction.h"
@@ -13,6 +14,8 @@
 #include "yapp.h"
 #include "prefs.h"
 
+#include "ypixbuf.h"
+
 #include <string.h>
 
 YColor *menuBg = 0;
@@ -20,11 +23,14 @@ YColor *menuItemFg = 0;
 YColor *activeMenuItemBg = 0;
 YColor *activeMenuItemFg = 0;
 YColor *disabledMenuItemFg = 0;
+YColor *disabledMenuItemSt = 0;
 
 YFont *menuFont = 0;
 
 int YMenu::fAutoScrollDeltaX = 0;
 int YMenu::fAutoScrollDeltaY = 0;
+int YMenu::fAutoScrollMouseX = -1;
+int YMenu::fAutoScrollMouseY = -1;
 
 void YMenu::setActionListener(YActionListener *actionListener) {
     fActionListener = actionListener;
@@ -41,19 +47,25 @@ int YMenu::fTimerX = 0, YMenu::fTimerY = 0, YMenu::fTimerItem = 0,
     YMenu::fTimerSubmenu = 0;
 bool YMenu::fTimerSlow = false;
 
-YMenu::YMenu(YWindow *parent): YPopupWindow(parent) {
+YMenu::YMenu(YWindow *parent): 
+    YPopupWindow(parent) INIT_GRADIENT(fGradient, NULL) {
     if (menuFont == 0)
         menuFont = YFont::getFont(menuFontName);
     if (menuBg == 0)
         menuBg = new YColor(clrNormalMenu);
     if (menuItemFg == 0)
         menuItemFg = new YColor(clrNormalMenuItemText);
-    if (activeMenuItemBg == 0)
+    if (*clrActiveMenuItem && activeMenuItemBg == 0)
         activeMenuItemBg = new YColor(clrActiveMenuItem);
     if (activeMenuItemFg == 0)
         activeMenuItemFg = new YColor(clrActiveMenuItemText);
     if (disabledMenuItemFg == 0)
         disabledMenuItemFg = new YColor(clrDisabledMenuItemText);
+    if (disabledMenuItemSt == 0)
+        disabledMenuItemSt = *clrDisabledMenuItemShadow
+			   ? new YColor(clrDisabledMenuItemShadow)
+			   : menuBg->brighter();
+
     fItems = 0;
     fItemCount = 0;
     paintedItem = selectedItem = -1;
@@ -74,10 +86,14 @@ YMenu::~YMenu() {
         fPopup->popdown();
         fPopup = 0;
     }
-
+    
     for (int i = 0; i < fItemCount; i++)
         delete fItems[i];
     FREE(fItems); fItems = 0;
+
+#ifdef CONFIG_GRADIENTS
+    delete fGradient;
+#endif    
 }
 
 void YMenu::activatePopup() {
@@ -108,7 +124,8 @@ void YMenu::donePopup(YPopupWindow *popup) {
 
 bool YMenu::isCondCascade(int selItem) {
     if (selItem != -1 &&
-        item(selItem)->name() && item(selItem)->submenu())
+        item(selItem)->action() &&
+        item(selItem)->submenu())
     {
         return true;
     }
@@ -124,8 +141,9 @@ int YMenu::onCascadeButton(int selItem, int x, int /*y*/, bool /*checkPopup*/) {
 
         unsigned int h = fontHeight;
 
-        if (item(selItem)->getPixmap() && item(selItem)->getPixmap()->height() > h)
-            h = item(selItem)->getPixmap()->height();
+        if (item(selItem)->getIcon() && 
+	    item(selItem)->getIcon()->height() > h)
+            h = item(selItem)->getIcon()->height();
 
         if (x <= int(width() - h - 4))
             return 1;
@@ -229,7 +247,7 @@ int YMenu::activateItem(int /*no*/, int byMouse, unsigned int modifiers) {
             finishPopup(item(selectedItem),
 	    		item(selectedItem)->action(), modifiers);
     } else {
-        //XBell(app->display(), 50);
+        //app->alert();
         return -1;
     }
     return 0;
@@ -312,37 +330,61 @@ bool YMenu::handleKey(const XKeyEvent &key) {
 }
 
 void YMenu::handleButton(const XButtonEvent &button) {
-    if (button.button != 0) {
-        int selItem = findItem(button.x_root - x(), button.y_root - y());
-        int nocascade = (onCascadeButton(selItem,
-                                         button.x_root - x(),
-                                         button.y_root - y(), true) &&
-                         !(button.state & ControlMask)) ? 0 : 1;
-        if (button.type == ButtonRelease && fPopupActive == fPopup && fPopup != 0 && nocascade) {
+    if (button.button == Button4) {
+	setPosition(x(), clamp(y() - (int)(button.state & ShiftMask ? 
+					   menuFont->height() * 5/2 :
+					   menuFont->height()),
+			       button.y_root - (int)height() + 1,
+			       button.y_root));
+        if (menuMouseTracking)
+	    trackMotion(clamp(button.x_root, x() + 2, x() + (int)width() - 3),
+			      button.y_root, button.state);
+    } else if (button.button == Button5) {
+	setPosition(x(), clamp(y() + (int)(button.state & ShiftMask ? 
+					   menuFont->height() * 5/2 :
+					   menuFont->height()),
+			       button.y_root - (int)height() + 1,
+			       button.y_root));
+        if (menuMouseTracking)
+	    trackMotion(clamp(button.x_root, x() + 2, x() + (int)width() - 3),
+			      button.y_root, button.state);
+    } else if (button.button) {
+        int const selItem(findItem(button.x_root - x(), button.y_root - y()));
+        bool const nocascade(!onCascadeButton(selItem,
+					      button.x_root - x(),
+                                              button.y_root - y(), true) ||
+                             (button.state & ControlMask));
+
+        if (button.type == ButtonRelease &&
+	    fPopupActive == fPopup && fPopup != NULL &&
+	    nocascade) {
             fPopup->popdown();
             fPopupActive = fPopup = 0;
             focusItem(selItem, 0, 1);
-            if (nocascade)
-                paintItems();
-            return ;
-        } else if (button.type == ButtonPress) {
+            if (nocascade) paintItems();
+            return;
+        } else if (button.type == ButtonPress)
             fPopupActive = fPopup;
-        }
+
         focusItem(selItem, nocascade, 1);
+
         if (selectedItem != -1 &&
             button.type == ButtonRelease &&
-            (item(selectedItem)->submenu() != 0 ||
-             item(selectedItem)->action() != 0)
+            (item(selectedItem)->submenu() != NULL ||
+             item(selectedItem)->action() != NULL)
             &&
-            (item(selectedItem)->action() == 0 ||
-             !item(selectedItem)->submenu() || !nocascade)) { // ??? !!! ??? WTF
+            (item(selectedItem)->action() == NULL ||
+             item(selectedItem)->submenu() == NULL ||
+	     !nocascade)) { // ??? !!! ??? WTF
             activatedX = button.x_root;
             activatedY = button.y_root;
             activateItem(selectedItem, 1, button.state);
             return;
         }
+
         if (button.type == ButtonRelease &&
-            (selectedItem == -1 || (item(selectedItem)->action() == 0 && item(selectedItem)->submenu() == 0)))
+            (selectedItem == -1 || (item(selectedItem)->action() == 0 &&
+	    item(selectedItem)->submenu() == 0)))
             focusItem(findActiveItem(itemCount() - 1, 1), 0, 0);
     }
     YPopupWindow::handleButton(button);
@@ -356,106 +398,108 @@ void YMenu::handleMotion(const XMotionEvent &motion) {
                          Button4Mask |
                          Button5Mask)) ? true : false;
 
-    if (menuMouseTracking || isButton) {
-        int selItem = findItem(motion.x_root - x(), motion.y_root - y());
-        if (fMenuTimer && fMenuTimer->getTimerListener() == this) {
-            //msg("sel=%d timer=%d listener=%p =? this=%p", selItem, fTimerItem, fMenuTimer->getTimerListener(), this);
-            if (selItem != fTimerItem || fTimerSlow) {
-                fTimerItem = -1;
-                if (fMenuTimer)
-                    fMenuTimer->stopTimer();
-            }
+    if (menuMouseTracking || isButton)
+	trackMotion(motion.x_root, motion.y_root, motion.state);
+
+    if (menuFont != NULL) { // ================ autoscrolling of large menus ===
+        int const fh(menuFont->height());
+
+        int const sx(motion.x_root < fh ? +fh :
+		     motion.x_root >= (int)(desktop->width() - fh - 1) ? -fh :
+		     0),
+        	  sy(motion.y_root < fh ? +fh :
+		     motion.y_root >= (int)(desktop->height() - fh - 1) ? -fh :
+		     0);
+
+	autoScroll(sx, sy, motion.x_root, motion.y_root, &motion);
+    }
+
+    YPopupWindow::handleMotion(motion); // ========== default implementation ===
+}
+
+void YMenu::trackMotion(const int x_root, const int y_root,
+			const unsigned state) {
+    int selItem = findItem(x_root - x(), y_root - y());
+    if (fMenuTimer && fMenuTimer->getTimerListener() == this) {
+        //msg("sel=%d timer=%d listener=%p =? this=%p", selItem, fTimerItem, fMenuTimer->getTimerListener(), this);
+        if (selItem != fTimerItem || fTimerSlow) {
+            fTimerItem = -1;
+            if (fMenuTimer)
+                fMenuTimer->stopTimer();
         }
-        if (selItem != -1 || app->popup() == this) {
-            int submenu = (onCascadeButton(selItem,
-                                           motion.x_root - x(),
-                                           motion.y_root - y(), false)
-                           && !(motion.state & ControlMask)) ? 0 : 1;
-            //if (selItem != -1)
-            bool canFast = true;
+    }
 
-            if (fPopup && activatedX != -1 && SubmenuActivateDelay != 0) {
-                int dx = 0;
-                int dy = motion.y_root - activatedY;
-                int ty = fPopup->y() - activatedY;
-                int by = fPopup->y() + fPopup->height() - activatedY;
-                int px;
+    if (selItem != -1 || app->popup() == this) {
+        const bool submenu(state & ControlMask || 
+			  !onCascadeButton(selItem,
+					   x_root - x(), y_root - y(), false));
+        //if (selItem != -1)
+        bool canFast = true;
 
-                if (fPopup->x() < activatedX)
-                    px = activatedX - (fPopup->x() + fPopup->width());
-                else
-                    px = fPopup->x() - activatedX;
+        if (fPopup && activatedX != -1 && SubmenuActivateDelay != 0) {
+            int dx = 0;
+            int dy = y_root - activatedY;
+            int ty = fPopup->y() - activatedY;
+            int by = fPopup->y() + fPopup->height() - activatedY;
+            int px;
 
-                if (fPopup->x() > motion.x_root)
-                    dx = motion.x_root - activatedX;
-                else
-                    dx = activatedX - motion.x_root;
+            if (fPopup->x() < activatedX)
+                px = activatedX - (fPopup->x() + fPopup->width());
+            else
+                px = fPopup->x() - activatedX;
 
-                dy = dy * px;
+            if (fPopup->x() > x_root)
+                dx = x_root - activatedX;
+            else
+                dx = activatedX - x_root;
 
-                if (dy >= ty * dx * 2 && dy <= by * dx * 2)
-                    canFast = false;
-            }
+            dy = dy * px;
 
-            if (canFast) {
-                YPopupWindow *p = fPopup;
+            if (dy >= ty * dx * 2 && dy <= by * dx * 2)
+                canFast = false;
+        }
 
-                if (MenuActivateDelay != 0 && selItem != -1) {
-                    if (fMenuTimer == 0)
-                        fMenuTimer = new YTimer();
-                    if (fMenuTimer) {
-                        fMenuTimer->setInterval(MenuActivateDelay);
-                        fMenuTimer->setTimerListener(this);
-                        if (!fMenuTimer->isRunning())
-                            fMenuTimer->startTimer();
-                    }
-                    fTimerItem = selItem;
-                    fTimerX = motion.x_root;
-                    fTimerY = motion.y_root;
-                    fTimerSubmenu = submenu;
-                    fTimerSlow = false;
-                } else {
-                    focusItem(selItem, submenu, 1);
-                    if (fPopup && p != fPopup) {
-                        activatedX = motion.x_root;
-                        activatedY = motion.y_root;
-                    }
-                }
-            } else {
-                //focusItem(selItem, 0, 1);
-                fTimerItem = selItem;
-                fTimerX = motion.x_root;
-                fTimerY = motion.y_root;
-                fTimerSubmenu = submenu;
-                fTimerSlow = true;
+        if (canFast) {
+            YPopupWindow *p = fPopup;
+
+            if (MenuActivateDelay != 0 && selItem != -1) {
                 if (fMenuTimer == 0)
                     fMenuTimer = new YTimer();
                 if (fMenuTimer) {
-                    fMenuTimer->setInterval(SubmenuActivateDelay);
+                    fMenuTimer->setInterval(MenuActivateDelay);
                     fMenuTimer->setTimerListener(this);
                     if (!fMenuTimer->isRunning())
                         fMenuTimer->startTimer();
                 }
+                fTimerItem = selItem;
+                fTimerX = x_root;
+                fTimerY = y_root;
+                fTimerSubmenu = submenu;
+                fTimerSlow = false;
+            } else {
+                focusItem(selItem, submenu, 1);
+                if (fPopup && p != fPopup) {
+                    activatedX = x_root;
+                    activatedY = y_root;
+                }
+            }
+        } else {
+            //focusItem(selItem, 0, 1);
+            fTimerItem = selItem;
+            fTimerX = x_root;
+            fTimerY = y_root;
+            fTimerSubmenu = submenu;
+            fTimerSlow = true;
+            if (fMenuTimer == 0)
+                fMenuTimer = new YTimer();
+            if (fMenuTimer) {
+                fMenuTimer->setInterval(SubmenuActivateDelay);
+                fMenuTimer->setTimerListener(this);
+                if (!fMenuTimer->isRunning())
+                    fMenuTimer->startTimer();
             }
         }
     }
-
-    int sx = 0;
-    int sy = 0;
-
-    if (motion.x_root == 0)
-        sx = 10;
-    else if (motion.x_root == int(desktop->width()) - 1)
-        sx = -10;
-
-    if (motion.y_root == 0)
-        sy = 10;
-    else if (motion.y_root == int(desktop->height()) - 1)
-        sy = -10;
-
-    autoScroll(sx, sy, &motion);
-
-    YPopupWindow::handleMotion(motion);
 }
 
 bool YMenu::handleTimer(YTimer */*timer*/) {
@@ -488,12 +532,21 @@ bool YMenu::handleAutoScroll(const XMotionEvent & /*mouse*/) {
         }
     }
     setPosition(px, py);
+    {
+        int mx = fAutoScrollMouseX - x();
+        int my = fAutoScrollMouseY - y();
+
+        int selItem = findItem(mx, my);
+        focusItem(selItem, 0, 1);
+    }
     return true;
 }
 
-void YMenu::autoScroll(int deltaX, int deltaY, const XMotionEvent *motion) {
+void YMenu::autoScroll(int deltaX, int deltaY, int mx, int my, const XMotionEvent *motion) {
     fAutoScrollDeltaX = deltaX;
     fAutoScrollDeltaY = deltaY;
+    fAutoScrollMouseX = mx;
+    fAutoScrollMouseY = my;
     beginAutoScroll((deltaX != 0 || deltaY != 0) ? true : false, motion);
 }
 
@@ -560,7 +613,7 @@ YMenuItem *YMenu::findSubmenu(const YMenu *sub) {
     return 0;
 }
 
-YMenuItem *YMenu::findName(const char *name, const int first = 0) {
+YMenuItem *YMenu::findName(const char *name, const int first) {
     if (name != NULL)
         for (int i = first; i < itemCount(); i++) {
             const char *iname = item(i)->name();
@@ -604,9 +657,9 @@ int YMenu::getItemHeight(int itemNo, int &h, int &top, int &bottom, int &pad) {
         if (fontHeight < 16)
             fontHeight = 16;
 
-        if (item(itemNo)->getPixmap() &&
-            item(itemNo)->getPixmap()->height() > ih)
-            ih = item(itemNo)->getPixmap()->height();
+        if (item(itemNo)->getIcon() &&
+            item(itemNo)->getIcon()->height() > ih)
+            ih = item(itemNo)->getIcon()->height();
 
         if (wmLook == lookWarp4 || wmLook == lookWin95) {
             top = bottom = 0;
@@ -633,22 +686,15 @@ int YMenu::getItemHeight(int itemNo, int &h, int &top, int &bottom, int &pad) {
 void YMenu::getItemWidth(int i, int &iw, int &nw, int &pw) {
     iw = nw = pw = 0;
 
-    if (item(i)->name() == 0 && item(i)->submenu() == 0) {
-        iw = 0;
-        nw = 0;
-        pw = 0;
-    } else {
-        YPixmap *p = item(i)->getPixmap();
-        if (p)
-            iw = p->height();
+    if (item(i)->name() != NULL || item(i)->submenu() != NULL) {
+        YIcon::Image const *icon(item(i)->getIcon());
+        if (icon) iw = icon->height();
 
-        const char *name = item(i)->name();
-        if (name)
-            nw = menuFont->textWidth(name);
+        const char *name(item(i)->name());
+        if (name) nw = menuFont->textWidth(name);
 
-        const char *param = item(i)->param();
-        if (param)
-            pw = menuFont->textWidth(param);
+        const char *param(item(i)->param());
+        if (param) pw = menuFont->textWidth(param);
     }
 }
 
@@ -707,14 +753,14 @@ int YMenu::findItem(int mx, int my) {
     return -1;
 }
 
-void YMenu::sizePopup() {
-    int width, height;
-    int maxName = 0;
-    int maxParam = 0;
-    int maxIcon = 16;
+void YMenu::sizePopup(int hspace) {
+    unsigned width, height;
+    int maxName(0);
+    int maxParam(0);
+    int maxIcon(16);
     int l, t, r, b;
-    int padx = 1;
-    int left = 1;
+    int padx(1);
+    int left(1);
 
     getOffsets(l, t, r, b);
 
@@ -747,14 +793,30 @@ void YMenu::sizePopup() {
             maxParam = pw;
     }
 
-    //maxWidth = maxName + maxParam + (maxParam ? 8 : 0);
-    //width += 3 + maxh + 2 + 2 + maxWidth + 4 + 4;
+    maxName = min(maxName, (int)(MenuMaximalWidth ? MenuMaximalWidth
+                                 : desktop->width() * 2/3));
+
+    hspace -= 4 + r + maxIcon + l + left + padx + 2 + maxParam + 6 + 2;
+    hspace = max(hspace, (int) desktop->width() / 3);
+
+    // !!! not correct, to be fixed
+    if (maxName > hspace)
+        maxName = hspace;
 
     namePos = l + left + padx + maxIcon + 2;
     paramPos = namePos + 2 + maxName + 6;
     width = paramPos + maxParam + 4 + r;
     height += b;
 
+#ifdef CONFIG_GRADIENTS
+    if (menubackPixbuf && !(fGradient &&
+    			    fGradient->width() == width &&
+			    fGradient->height() == height)) {
+	delete fGradient;
+	fGradient = new YPixbuf(*menubackPixbuf, width, height);
+    }
+#endif
+
     setSize(width, height);
 }
 
@@ -764,60 +826,118 @@ void YMenu::paintItems() {
     getOffsets(l, t, r, b);
 
     for (int i = 0; i < itemCount(); i++)
-        paintItem(g, i, l, t, r, (i == selectedItem || i == paintedItem) ? 1 : 0);
+        paintItem(g, i, l, t, r, 0, height(), (i == selectedItem || i == paintedItem) ? 1 : 0);
+
     paintedItem = selectedItem;
 }
 
+void YMenu::drawBackground(Graphics &g, int x, int y, int w, int h) {
+#ifdef CONFIG_GRADIENTS
+    if (fGradient)
+	g.copyPixbuf(*fGradient, x, y, w, h, x, y);
+    else 
+#endif    
+    if (menubackPixmap)
+	g.fillPixmap(menubackPixmap, x, y, w, h);
+    else
+	g.fillRect(x, y, w, h);
+}
+
 void YMenu::drawSeparator(Graphics &g, int x, int y, int w) {
-    if (wmLook == lookMetal) {
-        g.setColor(menuBg);
-        g.drawLine(x, y + 0, w, y + 0);
-        g.setColor(activeMenuItemBg);
+    g.setColor(menuBg);
+    
+#ifdef CONFIG_GRADIENTS
+    if (menusepPixbuf) {
+    	drawBackground(g, x, y, w, 2 - menusepPixmap->height()/2);
+
+	g.drawGradient(*menusepPixbuf,
+		       x, y + 2 - menusepPixmap->height()/2,
+		       w, menusepPixmap->height());
+
+	drawBackground(g, x, y + 2 + (menusepPixmap->height()+1)/2,
+		       w, 2 - (menusepPixmap->height()+1)/2);
+    } else
+#endif    
+    if (menusepPixmap) {
+    	drawBackground(g, x, y, w, 2 - menusepPixmap->height()/2);
+
+	g.fillPixmap(menusepPixmap,
+		     x, y + 2 - menusepPixmap->height()/2,
+		     w, menusepPixmap->height());
+
+	drawBackground(g, x, y + 2 + (menusepPixmap->height()+1)/2,
+		       w, 2 - (menusepPixmap->height()+1)/2);
+    } else if (wmLook == lookMetal) {
+	drawBackground(g, x, y + 0, w, 1);
+
+	if (activeMenuItemBg)
+            g.setColor(activeMenuItemBg);
+
         g.drawLine(x, y + 1, w, y + 1);;
         g.setColor(menuBg->brighter());
         g.drawLine(x, y + 2, w, y + 2);;
         g.drawLine(x, y, x, y + 2);
-        g.setColor(menuBg);
     } else {
-        //g.setColor(menuBg); // ASSUMED
-        g.drawLine(x, y + 0, w, y + 0);
+	drawBackground(g, x, y + 0, w, 1);
+
         g.setColor(menuBg->darker());
         g.drawLine(x, y + 1, w, y + 1);
         g.setColor(menuBg->brighter());
         g.drawLine(x, y + 2, w, y + 2);
         g.setColor(menuBg);
-        g.drawLine(x, y + 3, w, y + 3); y++;
+
+	drawBackground(g, x, y + 3, w, 1);
     }
 }
 
-void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
-    int fontHeight = menuFont->height() + 1;
-    int fontBaseLine = menuFont->ascent();
+void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int minY, int maxY, int paint) {
+    int const fontHeight(menuFont->height() + 1);
+    int const fontBaseLine(menuFont->ascent());
+
     YMenuItem *mitem = item(i);
     const char *name = mitem->name();
     const char *param = mitem->param();
+    
 
-    g.setColor(menuBg);
     if (mitem->name() == 0 && mitem->submenu() == 0) {
-        if (paint)
-            drawSeparator(g, 1, t, width() - 2);
+        if (t + 4 >= minY && t <= maxY)
+            if (paint) drawSeparator(g, 1, t, width() - 2);
+
         t += (wmLook == lookMetal) ? 3 : 4;
     } else {
-        int eh, top, bottom, pad, ih;
-        bool active = (i == selectedItem && 
-	    (mitem->action () || mitem->submenu()));
+        bool const active(i == selectedItem && 
+		         (mitem->action () || mitem->submenu()));
 
+        int eh, top, bottom, pad, ih;
         getItemHeight(i, eh, top, bottom, pad);
         ih = eh - top - bottom - pad - pad;
 
-        if (active)
-            g.setColor(activeMenuItemBg);
+        if (t + eh >= minY && t <= maxY) {
+
+	int const cascadePos(width() - r - 2 - ih - pad);
+
+        g.setColor(active && activeMenuItemBg ? activeMenuItemBg : menuBg);
 
         if (paint) {
-            if (menubackPixmap)
-                g.fillPixmap(menubackPixmap, l, t, width() -r -l, eh);
-            else
-                g.fillRect(l, t, width() - r - l, eh);
+            if (active) {
+#ifdef CONFIG_GRADIENTS
+		if (menuselPixbuf)
+		    g.drawGradient(*menuselPixbuf, l, t, width() - r - l, eh);
+		else 
+#endif
+		if (menuselPixmap)
+		    g.fillPixmap(menuselPixmap, l, t, width() - r - l, eh);
+		else if (activeMenuItemBg) {
+		    g.setColor(activeMenuItemBg);
+                    g.fillRect(l, t, width() - r - l, eh);
+		} else {
+		    g.setColor(menuBg);
+		    drawBackground(g, l, t, width() - r - l, eh);
+		}
+            } else {
+		g.setColor(menuBg);
+		drawBackground(g, l, t, width() - r - l, eh);
+	    }
 
             if (wmLook == lookMetal && i != selectedItem) {
                 g.setColor(menuBg->brighter());
@@ -826,9 +946,8 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
             }
 
             if (wmLook != lookWin95 && wmLook != lookWarp4 &&
-                active)
-            {
-                bool raised = false;
+                active) {
+                bool raised(false);
 #ifdef OLDMOTIF
                 if (wmLook == lookMotif)
                     raised = true;
@@ -838,9 +957,11 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                 if (wmLook == lookGtk)
                     g.drawBorderW(l, t, width() - r - l - 1, eh - 1, true);
                 else if (wmLook == lookMetal) {
-                    g.setColor(activeMenuItemBg->darker());
+                    g.setColor((activeMenuItemBg ? activeMenuItemBg
+		    				 : menuBg)->darker());
                     g.drawLine(l, t, width() - r - l, t);
-                    g.setColor(activeMenuItemBg->brighter());
+                    g.setColor((activeMenuItemBg ? activeMenuItemBg
+		    				 : menuBg)->brighter());
                     g.drawLine(l, t + eh - 1, width() - r - l, t + eh - 1);
                 } else
                     g.draw3DRect(l, t, width() - r - l - 1, eh - 1, raised);
@@ -851,13 +972,9 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                                  width() - r - l - 3, eh - 3, raised);
             }
 
-            YColor *fg;
-            if (!mitem->isEnabled())
-                fg = disabledMenuItemFg;
-            else if (active)
-                fg = activeMenuItemFg;
-            else
-                fg = menuItemFg;
+            YColor *fg(mitem->isEnabled() ? active ? activeMenuItemFg
+						   : menuItemFg
+					  : disabledMenuItemFg);
             g.setColor(fg);
             g.setFont(menuFont);
 
@@ -882,37 +999,38 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                 points[3].y = 5;
 
                 g.fillPolygon(points, 4, Convex, CoordModePrevious);
-            } else {
-                if (mitem->getPixmap())
-                    g.drawPixmap(mitem->getPixmap(),
-                                 l + 1 + delta,
-                                 t + delta + top + pad + (eh - top - pad * 2 - bottom - mitem->getPixmap()->height()) / 2);
+            } else if (mitem->getIcon()) {
+		g.drawImage(mitem->getIcon(),
+		    l + 1 + delta, t + delta + top + pad +
+		    (eh - top - pad * 2 - bottom - 
+		    mitem->getIcon()->height()) / 2);
             }
 
             if (name) {
-                if (!mitem->isEnabled()) {
-                    g.setColor(menuBg->brighter());
-                    g.drawChars(name, 0, strlen(name),
-                                1 + delta + namePos, 1 + baseLine);
+		int const maxWidth((param ? paramPos - delta :
+				    mitem->submenu() != NULL ? cascadePos :
+				    width()) - namePos);
 
-                    if (mitem->hotCharPos() != -1) {
+                if (!mitem->isEnabled()) {
+                    g.setColor(disabledMenuItemSt);
+		    g.drawStringEllipsis(1 + delta + namePos, 1 + baseLine, 
+					 name, maxWidth);
+
+                    if (mitem->hotCharPos() != -1)
                         g.drawCharUnderline(1 + delta +  namePos, 1 + baseLine,
                                             name, mitem->hotCharPos());
-                    }
                 }
                 g.setColor(fg);
-                g.drawChars(name, 0, strlen(name),
-                            delta + namePos, baseLine);
+                g.drawStringEllipsis(delta + namePos, baseLine, name, maxWidth);
 
-                if (mitem->hotCharPos() != -1) {
+                if (mitem->hotCharPos() != -1)
                     g.drawCharUnderline(delta + namePos, baseLine,
                                         name, mitem->hotCharPos());
-                }
             }
 
             if (param) {
                 if (!mitem->isEnabled()) {
-                    g.setColor(menuBg->brighter());
+                    g.setColor(disabledMenuItemSt);
                     g.drawChars(param, 0, strlen(param),
                                 paramPos + delta + 1,
                                 baseLine + 1);
@@ -928,19 +1046,16 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                 if (mitem->action()) {
                     g.setColor(menuBg);
                     if (0) {
-                        if (menubackPixmap)
-                            g.fillPixmap(menubackPixmap, width() - r - 1 -ih - pad, t + top + pad, ih, ih);
-                        else
-                            g.fillRect(width() - r - 1 - ih - pad, t + top + pad, ih, ih);
+			drawBackground(g, width() - r - 1 -ih - pad, t + top + pad, ih, ih);
                         g.drawBorderW(width() - r - 1 - ih - pad, t + top + pad, ih - 1, ih - 1,
                                       active ? false : true);
                     } else {
                         g.setColor(menuBg->darker());
-                        g.drawLine(width() - r - 2 - ih - pad, t + top + pad,
-                                   width() - r - 2 - ih - pad, t + top + pad + ih);
+                        g.drawLine(cascadePos, t + top + pad,
+                                   cascadePos, t + top + pad + ih);
                         g.setColor(menuBg->brighter());
-                        g.drawLine(width() - r - 2 - ih - pad + 1, t + top + pad,
-                                   width() - r - 2 - ih - pad + 1, t + top + pad + ih);
+                        g.drawLine(cascadePos + 1, t + top + pad,
+                                   cascadePos + 1, t + top + pad + ih);
 
                     }
                     delta = delta ? active ? 1 : 0 : 0;
@@ -951,25 +1066,32 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                     int ax = delta + width() - r - 1 - asize * 3 / 2;
                     int ay = delta + t + top + pad + (ih - asize) / 2;
                     g.setColor(menuBg);
-                    g.drawArrow(0, active ? 1 : -1, ax, ay, asize);
+                    g.drawArrow(Right, ax, ay, asize, active);
                 } else {
                     int asize = 9;
                     int ax = width() - r - 1 - asize;
                     int ay = delta + t + top + pad + (ih - asize) / 2;
 
                     g.setColor(fg);
-                    g.drawArrow(0, 0, ax, ay, asize);
+		    
+		    if (wmLook == lookWarp3) {
+			wmLook = lookNice;
+			g.drawArrow(Right, ax, ay, asize);
+			wmLook = lookWarp3;
+		    } else
+			g.drawArrow(Right, ax, ay, asize);
                 }
 
             }
         }
+        }
         t += eh;
     }
 }
 
-void YMenu::paint(Graphics &g, int /*_x*/, int /*_y*/, unsigned int /*_width*/, unsigned int /*_height*/) {
+void YMenu::paint(Graphics &g, int /*_x*/, int _y, unsigned int /*_width*/, unsigned int _height) {
     if (wmLook == lookMetal) {
-        g.setColor(activeMenuItemBg);
+        g.setColor(activeMenuItemBg ? activeMenuItemBg : menuBg);
         g.drawLine(0, 0, width() - 1, 0);
         g.drawLine(0, 0, 0, height() - 1);
         g.drawLine(width() - 1, 0, width() - 1, height() - 1);
@@ -990,7 +1112,8 @@ void YMenu::paint(Graphics &g, int /*_x*/, int /*_y*/, unsigned int /*_width*/,
     int l, t, r, b;
     getOffsets(l, t, r, b);
 
-    for (int i = 0; i < itemCount(); i++)
-        paintItem(g, i, l, t, r, 1);
+    for (int i = 0; i < itemCount(); i++) {
+        paintItem(g, i, l, t, r, _y, _y + _height, 1);
+    }
     paintedItem = selectedItem;
 }
