@@ -3,12 +3,12 @@
 
 class YMenu;
 class YAction;
-class YPixmap;
 class YActionListener;
 
 class YMenuItem {
 public:
     YMenuItem(const char *name, int hotCharPos, const char *param, YAction *action, YMenu *submenu);
+    YMenuItem(const char *name);
     YMenuItem() { fName = 0; fHotCharPos = -1; fParam = 0; fAction = 0; fEnabled = 0; fSubmenu = 0; }
     virtual ~YMenuItem();
     const char *name() const { return fName; }
@@ -18,8 +18,8 @@ public:
     int hotChar() const { return (fName && fHotCharPos >= 0) ? fName[fHotCharPos] : -1; }
     int hotCharPos() const { return fHotCharPos; }
 
-    YPixmap *getPixmap() const { return fPixmap; }
-    void setPixmap(YPixmap *pixmap);
+    YIcon::Image *getIcon() const { return fIcon; }
+    void setIcon(YIcon::Image *icon);
     void setChecked(bool c);
     int isChecked() const { return fChecked; }
     int isEnabled() const { return fEnabled; }
@@ -33,7 +33,7 @@ private:
     YAction *fAction;
     int fHotCharPos;
     YMenu *fSubmenu;
-    YPixmap *fPixmap;
+    YIcon::Image *fIcon;
     bool fChecked;
     bool fEnabled;
 };
