@@ -14,9 +14,9 @@ bool YApplication::popup(YWindow *forWindow, YPopupWindow *popup) {
     PRECONDITION(popup != 0);
     if (fPopup == 0) {
 //        Cursor changePointer = None; //!!!(popup->popupFlags() & YPopupWindow::pfNoPointerChange) ? None : rightPointer;
-        Cursor changePointer =
-	    (popup->popupFlags() & YPopupWindow::pfNoPointerChange)
-	    ? None : rightPointer.handle();
+        Cursor changePointer = (dontRotateMenuPointer ||
+	    (popup->popupFlags() & YPopupWindow::pfNoPointerChange) ?
+	     None : rightPointer.handle());
 
         if (!grabEvents(forWindow ? forWindow : popup, changePointer,
                         ButtonPressMask | ButtonReleaseMask |
@@ -54,13 +54,13 @@ YPopupWindow::YPopupWindow(YWindow *aParent): YWindow(aParent) {
 }
 
 YPopupWindow::~YPopupWindow() {
-    PRECONDITION(fUp == false);
+//    PRECONDITION(fUp == false); // ^C whilest a popup is open breaks this...
 }
 
 void YPopupWindow::updatePopup() {
 }
 
-void YPopupWindow::sizePopup() {
+void YPopupWindow::sizePopup(int /*hspace*/) {
 }
 
 void YPopupWindow::activatePopup() {
@@ -105,7 +105,15 @@ bool YPopupWindow::popup(YWindow *forWindow,
 
     updatePopup();
 
-    sizePopup();
+    { // check available space on left and right
+        int spaceRight = desktop->width() - x;
+        int spaceLeft = x - x_delta;
+
+
+        int hspace = (spaceLeft < spaceRight) ? spaceRight : spaceLeft;
+
+        sizePopup(hspace);
+    }
 
     /* !!! FIX this to maximize visible area */
     if ((x + width() > desktop->width()) || (fFlags & pfFlipHorizontal))
@@ -182,14 +190,15 @@ bool YPopupWindow::handleKey(const XKeyEvent &/*key*/) {
 }
 
 void YPopupWindow::handleButton(const XButtonEvent &button) {
-    if (button.x_root >= x() &&
-        button.y_root >= y() &&
-        button.x_root < int (x() + width()) &&
-        button.y_root < int (y() + height()) &&
-        button.window == handle())
-    {
+    if ((button.x_root >= x() &&
+         button.y_root >= y() &&
+         button.x_root < int (x() + width()) &&
+         button.y_root < int (y() + height()) &&
+         button.window == handle()) |
+	 button.button == Button4 ||
+	 button.button == Button5)
         YWindow::handleButton(button);
-    } else {
+    else {
         if (fForWindow) {
             XEvent xev;
 
