@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "ytooltip.h"
@@ -49,7 +49,7 @@ void YToolTip::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/,
         int y = toolTipFont->ascent() + 2;
         g.setFont(toolTipFont);
         g.setColor(toolTipFg);
-        g.drawChars(fText, 0, strlen(fText), 3, y);
+	g.drawStringMultiline(3, y, fText);
     }
 }
 
@@ -58,10 +58,8 @@ void YToolTip::setText(const char *tip) {
     if (tip) {
         fText = newstr(tip);
         if (fText) {
-            int w = toolTipFont->textWidth(fText);
-            int h = toolTipFont->ascent();
-
-            setSize(w + 6, h + 7);
+	    YDimension const size(toolTipFont->multilineAlloc(fText));
+            setSize(size.w + 6, size.h + 7);
         } else {
             setSize(20, 20);
         }
@@ -93,8 +91,10 @@ bool YToolTip::handleTimer(YTimer *t) {
 void YToolTip::display() {
     raise();
     show();
-    if (!fToolTipVisibleTimer)
+
+    if (!fToolTipVisibleTimer && ToolTipTime > 0)
         fToolTipVisibleTimer = new YTimer(ToolTipTime);
+
     if (fToolTipVisibleTimer) {
         fToolTipVisibleTimer->setTimerListener(this);
         fToolTipVisibleTimer->startTimer();
