@@ -13,10 +13,11 @@
 #include "sysdep.h"
 #include "prefs.h"
 
-extern XContext windowContext;
-
 #include "ytimer.h"
 
+/******************************************************************************/
+/******************************************************************************/
+
 class AutoScroll: public YTimerListener {
 public:
     AutoScroll();
@@ -79,8 +80,11 @@ void AutoScroll::autoScroll(YWindow *w, bool autoScroll, const XMotionEvent *mot
     }
 }
 
-AutoScroll *YWindow::fAutoScroll = 0;
+/******************************************************************************/
+/******************************************************************************/
 
+extern XContext windowContext;
+AutoScroll *YWindow::fAutoScroll = 0;
 YWindow *YWindow::fClickWindow = 0;
 Time YWindow::fClickTime = 0;
 int YWindow::fClickCount = 0;
@@ -89,6 +93,26 @@ int YWindow::fClickDrag = 0;
 unsigned int YWindow::fClickButton = 0;
 unsigned int YWindow::fClickButtonDown = 0;
 
+/******************************************************************************/
+/******************************************************************************/
+
+YWindowAttributes::YWindowAttributes(Window window) {
+    if (!XGetWindowAttributes(app->display(), window, &attributes)) {
+	XGetGeometry(app->display(), window, &attributes.root,
+		     &attributes.x, &attributes.y,
+		     (unsigned *) &attributes.width,
+		     (unsigned *) &attributes.height,
+		     (unsigned *) &attributes.border_width, 
+		     (unsigned *) &attributes.depth);
+
+	attributes.visual = app->visual();
+	attributes.colormap = app->colormap();
+   }
+}
+
+/******************************************************************************/
+/******************************************************************************/
+
 YWindow::YWindow(YWindow *parent, Window win):
     fParentWindow(parent),
     fNextWindow(0), fPrevWindow(0), fFirstWindow(0), fLastWindow(0),
@@ -168,7 +192,7 @@ void YWindow::setStyle(unsigned long aStyle) {
         if (flags & wfCreated) {
             if (aStyle & wsManager)
                 fEventMask |=
-                    SubstructureRedirectMask | SubstructureNotifyMask ;
+                    SubstructureRedirectMask | SubstructureNotifyMask;
 
             if (fStyle & wsPointerMotion)
                 fEventMask |= PointerMotionMask;
@@ -185,30 +209,22 @@ void YWindow::setStyle(unsigned long aStyle) {
 }
 
 Graphics &YWindow::getGraphics() {
-    if (fGraphics == 0)
-        fGraphics = new Graphics(this);
-    return *fGraphics;
+    return *(NULL == fGraphics ? fGraphics = new Graphics(*this) : fGraphics);
 }
 
 void YWindow::repaint() {
-    if ((flags & (wfCreated | wfVisible)) == (wfCreated | wfVisible)) {
-        Graphics &g = getGraphics();
-
-        paint(g, 0, 0, width(), height());
-    }
+//    if ((flags & (wfCreated | wfVisible)) == (wfCreated | wfVisible)) {
+    if (viewable()) paint(getGraphics(), 0, 0, width(), height());
 }
 
 void YWindow::repaintFocus() {
-    if ((flags & (wfCreated | wfVisible)) == (wfCreated | wfVisible)) {
-        Graphics &g = getGraphics();
-
-        paintFocus(g, 0, 0, width(), height());
-    }
+//    if ((flags & (wfCreated | wfVisible)) == (wfCreated | wfVisible)) {
+    if (viewable()) paintFocus(getGraphics(), 0, 0, width(), height());
 }
 
 void YWindow::create() {
-    if (flags & wfCreated)
-        return ;
+    if (flags & wfCreated) return;
+
     if (fHandle == None) {
         XSetWindowAttributes attributes;
         unsigned int attrmask = CWEventMask;
@@ -221,10 +237,9 @@ void YWindow::create() {
             fEventMask |= PointerMotionMask;
 
         if (fParentWindow == desktop && !(fStyle & wsOverrideRedirect))
-            fEventMask |= StructureNotifyMask;
+            fEventMask |= StructureNotifyMask | SubstructureRedirectMask;
         if (fStyle & wsManager)
-            fEventMask |=
-                SubstructureRedirectMask | SubstructureNotifyMask;
+            fEventMask |= SubstructureRedirectMask | SubstructureNotifyMask;
 
         if (fStyle & wsSaveUnder) {
             attributes.save_under = True;
@@ -246,6 +261,7 @@ void YWindow::create() {
             attributes.win_gravity = fWinGravity;
             attrmask |= CWWinGravity;
         }
+	
         attributes.event_mask = fEventMask;
         int zw = width();
         int zh = height();
@@ -375,9 +391,7 @@ void YWindow::reparent(YWindow *parent, int x, int y) {
 }
 
 Window YWindow::handle() {
-    if (!(flags & wfCreated))
-        create();
-
+    if (!(flags & wfCreated)) create();
     return fHandle;
 }
 
@@ -503,9 +517,10 @@ void YWindow::handleEvent(const XEvent &event) {
 	handleMapRequest(event.xmaprequest);
 	break;
 
-    case ConfigureRequest:
-        handleConfigureRequest(event.xconfigurerequest);
-        break;
+    case ReparentNotify: 
+	handleReparentNotify(event.xreparent);
+	break;
+
     case ConfigureNotify:
 #if 1
          {
@@ -533,6 +548,10 @@ void YWindow::handleEvent(const XEvent &event) {
 #endif
         break;
 
+    case ConfigureRequest:
+        handleConfigureRequest(event.xconfigurerequest);
+        break;
+
     case DestroyNotify:
 	handleDestroyWindow(event.xdestroywindow);
 	break;
@@ -568,7 +587,7 @@ void YWindow::handleEvent(const XEvent &event) {
 	handleSelection(event.xselection);
 	break;
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     default:
         if (shapesSupported && event.type == (shapeEventBase + ShapeNotify))
             handleShapeNotify(*(const XShapeEvent *)&event);
@@ -619,19 +638,22 @@ void YWindow::handleGraphicsExpose(const XGraphicsExposeEvent &graphicsExpose) {
 }
 
 void YWindow::handleConfigure(const XConfigureEvent &configure) {
-    if (configure.window == handle())
+    if (configure.window == handle()) {
+	const bool resized((unsigned int)configure.width != fWidth ||
+			   (unsigned int)configure.height != fHeight);
+	
         if (configure.x != fX ||
             configure.y != fY ||
-            (unsigned int)configure.width != fWidth ||
-            (unsigned int)configure.height != fHeight)
+            resized)
         {
             fX = configure.x;
             fY = configure.y;
             fWidth = configure.width;
             fHeight = configure.height;
 
-            this->configure(fX, fY, fWidth, fHeight);
+            this->configure(fX, fY, fWidth, fHeight, resized);
         }
+    }	
 }
 
 bool YWindow::handleKey(const XKeyEvent &key) {
@@ -680,17 +702,9 @@ void YWindow::handleButton(const XButtonEvent &button) {
     }
 #endif
 
-    int x_dif = button.x_root - fClickEvent.x_root;
-    int y_dif = button.y_root - fClickEvent.y_root;
-
-    unsigned int motionDelta = 0;
-    unsigned int motionDeltaX = (x_dif < 0) ? - x_dif : x_dif;
-    unsigned int motionDeltaY = (y_dif < 0) ? - y_dif : y_dif;
-
-    if (motionDeltaX > motionDeltaY)
-        motionDelta = motionDeltaX;
-    else
-        motionDelta = motionDeltaY;
+    int const dx(abs(button.x_root - fClickEvent.x_root));
+    int const dy(abs(button.y_root - fClickEvent.y_root));
+    int const motionDelta(max(dx, dy));
 
     if (button.type == ButtonPress) {
         fClickDrag = 0;
@@ -699,7 +713,7 @@ void YWindow::handleButton(const XButtonEvent &button) {
             fClickWindow = this;
             fClickCount = 1;
         } else {
-            if ((button.time - fClickTime < MultiClickTime) &&
+            if ((button.time - fClickTime < (unsigned) MultiClickTime) &&
                 fClickButton == button.button &&
                 motionDelta <= ClickMotionDistance &&
                 button.x >= 0 && button.y >= 0 &&
@@ -740,19 +754,11 @@ void YWindow::handleMotion(const XMotionEvent &motion) {
         if (fClickDrag) {
             handleDrag(fClickEvent, motion);
         } else {
-            int x_dif = motion.x_root - fClickEvent.x_root;
-            int y_dif = motion.y_root - fClickEvent.y_root;
+            int const dx(abs(motion.x_root - fClickEvent.x_root));
+            int const dy(abs(motion.y_root - fClickEvent.y_root));
+            int const motionDelta(max(dx, dy));
 
-            unsigned int motionDelta = 0;
-            unsigned int motionDeltaX = (x_dif < 0) ? - x_dif : x_dif;
-            unsigned int motionDeltaY = (y_dif < 0) ? - y_dif : y_dif;
-
-            if (motionDeltaX > motionDeltaY)
-                motionDelta = motionDeltaX;
-            else
-                motionDelta = motionDeltaY;
-
-            if ((motion.time - fClickTime > ClickMotionDelay) ||
+            if ((motion.time - fClickTime > (unsigned) ClickMotionDelay) ||
                 (motionDelta >= ClickMotionDistance)) {
                 fClickDrag = 1;
                 handleBeginDrag(fClickEvent, motion);
@@ -821,15 +827,6 @@ void YWindow::handleCrossing(const XCrossingEvent &crossing) {
 #endif
 }
 
-void YWindow::handleProperty(const XPropertyEvent &) {
-}
-
-void YWindow::handleColormap(const XColormapEvent &) {
-}
-
-void YWindow::handleFocus(const XFocusChangeEvent &) {
-}
-
 void YWindow::handleClientMessage(const XClientMessageEvent &message) {
     if (message.message_type == _XA_WM_PROTOCOLS
         && message.format == 32
@@ -853,7 +850,7 @@ void YWindow::handleClientMessage(const XClientMessageEvent &message) {
 #endif
 
 void YWindow::handleMap(const XMapEvent &) {
-//    flags |= wfVisible;
+//    flags |= wfVisible; // !!! WTF does this 'cause such odd side effects?
 }
 
 void YWindow::handleUnmap(const XUnmapEvent &) {
@@ -871,39 +868,10 @@ void YWindow::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow) {
         flags |= wfDestroyed;
 }
 
-void YWindow::handleConfigureRequest(const XConfigureRequestEvent &) {
-}
-
-void YWindow::handleMapRequest(const XMapRequestEvent &) {
-}
-
-#ifdef SHAPE
-void YWindow::handleShapeNotify(const XShapeEvent &) {
-}
-#endif
-
-void YWindow::handleClickDown(const XButtonEvent &/*down*/, int /*count*/) {
-}
-
-void YWindow::handleClick(const XButtonEvent &/*up*/, int /*count*/) {
-}
-
-void YWindow::handleBeginDrag(const XButtonEvent &/*down*/, const XMotionEvent &/*motion*/) {
-}
-
-void YWindow::handleDrag(const XButtonEvent &/*down*/, const XMotionEvent &/*motion*/) {
-}
-
-void YWindow::handleEndDrag(const XButtonEvent &/*down*/, const XButtonEvent &/*up*/) {
-}
-
 void YWindow::paint(Graphics &g, int x, int y, unsigned int w, unsigned int h) {
     g.fillRect(x, y, w, h);
 }
 
-void YWindow::paintFocus(Graphics &/*g*/, int /*x*/, int /*y*/, unsigned int /*w*/, unsigned int /*h*/) {
-}
-
 bool YWindow::nullGeometry() {
     bool zero;
 
@@ -928,11 +896,9 @@ bool YWindow::nullGeometry() {
 }
 
 void YWindow::setGeometry(int x, int y, unsigned int width, unsigned int height) {
-    if (x != fX ||
-        y != fY ||
-        width != fWidth ||
-        height != fHeight)
-    {
+    const bool resized(width != fWidth || height != fHeight);
+
+    if (x != fX || y != fY || resized) {
         fX = x;
         fY = y;
         fWidth = width;
@@ -945,7 +911,7 @@ void YWindow::setGeometry(int x, int y, unsigned int width, unsigned int height)
                                   fX, fY, fWidth, fHeight);
         }
 
-        configure(fX, fY, fWidth, fHeight);
+        configure(fX, fY, fWidth, fHeight, resized);
     }
 }
 
@@ -957,7 +923,7 @@ void YWindow::setPosition(int x, int y) {
         if (flags & wfCreated)
             XMoveWindow(app->display(), fHandle, fX, fY);
 
-        configure(fX, fY, width(), height());
+        configure(fX, fY, width(), height(), false);
     }
 }
 
@@ -970,7 +936,7 @@ void YWindow::setSize(unsigned int width, unsigned int height) {
             if (!nullGeometry())
                 XResizeWindow(app->display(), fHandle, fWidth, fHeight);
 
-        configure(x(), y(), fWidth, fHeight);
+        configure(x(), y(), fWidth, fHeight, true);
     }
 }
 
@@ -1000,7 +966,8 @@ void YWindow::mapToLocal(int &x, int &y) {
     y = dy;
 }
 
-void YWindow::configure(int, int, unsigned int, unsigned int) {
+void YWindow::configure(const int, const int, const unsigned, const unsigned, 
+			const bool) {
 }
 
 void YWindow::setPointer(const YCursor& pointer) {
@@ -1079,6 +1046,12 @@ void YWindow::setEnabled(bool enable) {
     }
 }
 
+bool YWindow::viewable(Drawable drawable) {
+    XWindowAttributes attributes;
+    XGetWindowAttributes(app->display(), drawable, &attributes);
+    return (IsViewable == attributes.map_state);
+}
+
 bool YWindow::isFocusTraversable() {
     return false;
 }
@@ -1490,11 +1463,12 @@ void YWindow::grabVKey(int key, unsigned int vm) {
            (!(vm & kfHyper) || app->HyperMask))
             grabKey(key, m);
 
+        // !!! recheck this
         if (((vm & (kfAlt | kfCtrl)) == (kfAlt | kfCtrl)) &&
             modMetaIsCtrlAlt &&
-            app->MetaMask)
+            app->WinMask)
         {
-            m = app->MetaMask;
+            m = app->WinMask;
             if (vm & kfShift)
                 m |= ShiftMask;
            if (vm & kfSuper)
@@ -1508,22 +1482,25 @@ void YWindow::grabVKey(int key, unsigned int vm) {
 
 unsigned int YWindow::VMod(int m) {
     int vm = 0;
+    int m1 = m & ~app->WinMask;
 
-    if (m & ShiftMask)
+    if (m1 & ShiftMask)
         vm |= kfShift;
-    if (m & ControlMask)
+    if (m1 & ControlMask)
         vm |= kfCtrl;
-    if (m & app->AltMask)
+    if (m1 & app->AltMask)
         vm |= kfAlt;
-    if (m & app->MetaMask)
+    if (m1 & app->MetaMask)
         vm |= kfMeta;
-    if (m & app->SuperMask)
+    if (m1 & app->SuperMask)
        vm |= kfSuper;
-    if (m & app->HyperMask)
+    if (m1 & app->HyperMask)
        vm |= kfHyper;
 
-    if (modMetaIsCtrlAlt && (vm & (kfCtrl | kfAlt | kfMeta)) == kfMeta)
-        vm = (vm & ~kfMeta) | kfCtrl | kfAlt;
+    if (modMetaIsCtrlAlt && (m & app->WinMask))
+            vm |= kfCtrl + kfAlt;
+    //&& (vm & (kfCtrl | kfAlt | kfMeta)) == kfMeta)
+    //vm = (vm & ~kfMeta) | kfCtrl | kfAlt;
 
     return vm;
 }
