@@ -9,6 +9,30 @@ class YToolTip;
 class YTimer;
 class AutoScroll;
 
+#ifdef CONFIG_GRADIENTS
+#define INIT_GRADIENT(Member, Value) , Member(Value)
+#else
+#define INIT_GRADIENT(Member, Value)
+#endif
+
+class YWindowAttributes {
+public:
+    YWindowAttributes(Window window);
+    
+    Window root() const { return attributes.root; }
+    int x() const { return attributes.x; }
+    int y() const { return attributes.y; }
+    unsigned width() const { return attributes.width; }
+    unsigned height() const { return attributes.height; }
+    unsigned border() const { return attributes.border_width; }
+    unsigned depth() const { return attributes.depth; }
+    Visual * visual() const { return attributes.visual; }
+    Colormap colormap() const { return attributes.colormap; }
+
+private:
+    XWindowAttributes attributes;
+};
+
 class YWindow {
 public:
     YWindow(YWindow *aParent = 0, Window win = 0);
@@ -31,13 +55,15 @@ public:
     void setTitle(char const * title);
     void setClassHint(char const * rName, char const * rClass);
 
-    void setGeometry(int x, int y, unsigned int width, unsigned int height);
-    void setSize(unsigned int width, unsigned int height);
+    void setGeometry(int x, int y, unsigned width, unsigned height);
+    void setSize(unsigned width, unsigned height);
     void setPosition(int x, int y);
-    virtual void configure(int x, int y, unsigned int width, unsigned int height);
+    virtual void configure(const int x, const int y, 
+    			   const unsigned width, const unsigned height,
+			   const bool resized);
 
-    virtual void paint(Graphics &g, int x, int y, unsigned int width, unsigned int height);
-    virtual void paintFocus(Graphics &g, int x, int y, unsigned int w, unsigned int h);
+    virtual void paint(Graphics &g, int x, int y, unsigned width, unsigned height);
+    virtual void paintFocus(Graphics &, int, int, unsigned, unsigned) {}
 
     virtual void handleEvent(const XEvent &event);
 
@@ -48,9 +74,9 @@ public:
     virtual void handleButton(const XButtonEvent &button);
     virtual void handleMotion(const XMotionEvent &motion);
     virtual void handleCrossing(const XCrossingEvent &crossing);
-    virtual void handleProperty(const XPropertyEvent &property);
-    virtual void handleColormap(const XColormapEvent &colormap);
-    virtual void handleFocus(const XFocusChangeEvent &focus);
+    virtual void handleProperty(const XPropertyEvent &) {}
+    virtual void handleColormap(const XColormapEvent &) {}
+    virtual void handleFocus(const XFocusChangeEvent &) {}
     virtual void handleClientMessage(const XClientMessageEvent &message);
     virtual void handleSelectionClear(const XSelectionClearEvent &clear);
     virtual void handleSelectionRequest(const XSelectionRequestEvent &request);
@@ -62,17 +88,18 @@ public:
     virtual void handleMap(const XMapEvent &map);
     virtual void handleUnmap(const XUnmapEvent &unmap);
     virtual void handleDestroyWindow(const XDestroyWindowEvent &destroyWindow);
-    virtual void handleConfigureRequest(const XConfigureRequestEvent &configureRequest);
-    virtual void handleMapRequest(const XMapRequestEvent &mapRequest);
-#ifdef SHAPE
-    virtual void handleShapeNotify(const XShapeEvent &shape);
+    virtual void handleReparentNotify(const XReparentEvent &) {}
+    virtual void handleConfigureRequest(const XConfigureRequestEvent &) {}
+    virtual void handleMapRequest(const XMapRequestEvent &) {}
+#ifdef CONFIG_SHAPE
+    virtual void handleShapeNotify(const XShapeEvent &) {}
 #endif
 
-    virtual void handleClickDown(const XButtonEvent &down, int count);
-    virtual void handleClick(const XButtonEvent &up, int count);
-    virtual void handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion);
-    virtual void handleDrag(const XButtonEvent &down, const XMotionEvent &motion);
-    virtual void handleEndDrag(const XButtonEvent &down, const XButtonEvent &up);
+    virtual void handleClickDown(const XButtonEvent &, int) {}
+    virtual void handleClick(const XButtonEvent &, int) {}
+    virtual void handleBeginDrag(const XButtonEvent &, const XMotionEvent &) {}
+    virtual void handleDrag(const XButtonEvent &, const XMotionEvent &) {}
+    virtual void handleEndDrag(const XButtonEvent &, const XButtonEvent &) {}
 
     virtual void handleClose();
 
@@ -81,12 +108,12 @@ public:
 
     void setPointer(const YCursor& pointer);
     void setGrabPointer(const YCursor& pointer);
-    void grabKeyM(int key, unsigned int modifiers);
-    void grabKey(int key, unsigned int modifiers);
-    void grabVKey(int key, unsigned int vmodifiers);
-    unsigned int VMod(int modifiers);
-    void grabButtonM(int button, unsigned int modifiers);
-    void grabButton(int button, unsigned int modifiers);
+    void grabKeyM(int key, unsigned modifiers);
+    void grabKey(int key, unsigned modifiers);
+    void grabVKey(int key, unsigned vmodifiers);
+    unsigned VMod(int modifiers);
+    void grabButtonM(int button, unsigned modifiers);
+    void grabButton(int button, unsigned modifiers);
 
     void captureEvents();
     void releaseEvents();
@@ -94,18 +121,26 @@ public:
     Window handle();
     YWindow *parent() const { return fParentWindow; }
 
-    Graphics &getGraphics();
+    Graphics & getGraphics();
+#ifdef CONFIG_GRADIENTS
+    virtual class YPixbuf * getGradient() const { 
+	return (parent() ? parent()->getGradient() : NULL); }
+#endif    
 
     int x() const { return fX; }
     int y() const { return fY; }
-    unsigned int width() const { return fWidth; }
-    unsigned int height() const { return fHeight; }
+    unsigned width() const { return fWidth; }
+    unsigned height() const { return fHeight; }
 
-    int visible() const { return (flags & wfVisible) ? 1 : 0; }
-    int created() const { return (flags & wfCreated) ? 1 : 0; }
-    int adopted() const { return (flags & wfAdopted) ? 1 : 0; }
-    int destroyed() const { return (flags & wfDestroyed) ? 1 : 0; }
-    int unmapped() const { return (flags & wfUnmapped) ? 1 : 0; }
+    bool visible() const { return (flags & wfVisible); }
+    bool created() const { return (flags & wfCreated); }
+    bool adopted() const { return (flags & wfAdopted); }
+    bool destroyed() const { return (flags & wfDestroyed); }
+    bool unmapped() const { return (flags & wfUnmapped); }
+
+    static bool viewable(Drawable drawable);
+
+    bool viewable() const { return viewable(fHandle); }
 
     virtual void donePopup(YPopupWindow * /*command*/);
 
@@ -136,8 +171,8 @@ public:
 
     YWindow *toplevel();
 
-    void installAccelerator(unsigned int key, unsigned int mod, YWindow *win);
-    void removeAccelerator(unsigned int key, unsigned int mod, YWindow *win);
+    void installAccelerator(unsigned key, unsigned mod, YWindow *win);
+    void removeAccelerator(unsigned key, unsigned mod, YWindow *win);
 
     void setToolTip(const char *tip);
 
@@ -197,7 +232,7 @@ private:
     unsigned long flags;
     unsigned long fStyle;
     int fX, fY;
-    unsigned int fWidth, fHeight;
+    unsigned fWidth, fHeight;
     YCursor fPointer;
     int unmapCount;
     Graphics *fGraphics;
@@ -208,8 +243,8 @@ private:
     bool fToplevel;
 
     typedef struct _YAccelerator {
-        unsigned int key;
-        unsigned int mod;
+        unsigned key;
+        unsigned mod;
         YWindow *win;
         struct _YAccelerator *next;
     } YAccelerator;
@@ -224,8 +259,8 @@ private:
     static Time fClickTime;
     static int fClickCount;
     static int fClickDrag;
-    static unsigned int fClickButton;
-    static unsigned int fClickButtonDown;
+    static unsigned fClickButton;
+    static unsigned fClickButtonDown;
     static YTimer *fToolTipTimer;
 
     bool fDND;
@@ -245,7 +280,7 @@ public:
 
 extern YDesktop *desktop;
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
 extern int shapesSupported;
 extern int shapeEventBase, shapeErrorBase;
 #endif
